function uv = combine_seg_flow_proposals(this, puv, tu, tv, L)

%COMBINE_FLOW_PROPOSALS combines the flow proposals using the combined
%local minima (CLM) method by Jung et al. ECCV2008

% Image segements (L) are used instead of rectangle regions.


this.display = false;
% Precompute 
nP = length(puv);

% data term
data = cell(1,nP);
for i = 1:nP
    It  = partial_deriv(this.images, puv{i});    
    temp  = reshape(evaluate_log(this.rho_data, It(:)'), size(It));    
    data{i} = temp;
end;

% spatial term
%SF = {[1 -1], [1; -1]};
S = this.spatial_filters;

mask = ones(size(puv{1}(:,:,1)));
mask(:,end) = 0;
mask(end,:) = 0;
mask = (mask ==0);

spatial = cell(1,nP);
for i = 1:nP
    ux = conv2(puv{i}(:,:,1), S{1}, 'same');
    vx = conv2(puv{i}(:,:,2), S{1}, 'same');
    uy = conv2(puv{i}(:,:,1), S{2}, 'same');
    vy = conv2(puv{i}(:,:,2), S{2}, 'same');
    % Remove these invalid regions
    ux(mask) = 0;
    uy(mask) = 0;
    vx(mask) = 0;
    vy(mask) = 0;
    
    sux = reshape(evaluate_log(this.rho_spatial_u{1}, ux(:)'), size(ux));
    svx = reshape(evaluate_log(this.rho_spatial_v{1}, vx(:)'), size(ux));
    suy = reshape(evaluate_log(this.rho_spatial_u{2}, uy(:)'), size(ux));
    svy = reshape(evaluate_log(this.rho_spatial_v{2}, vy(:)'), size(ux));
    
    spatial{i} = sux+suy+svx+svy;
end;

% start from the one with high log posterior
for i = 1:nP
    logp(i) = sum(data{i}(:)) + this.lambda*sum(spatial{i}(:));
end;
[tmp ind] = max(logp);

% select the one with lowest energy as initial value
uv = puv{ind};

% CLM
isz  = size(ux);
npr  = round(isz(1)/psz(1));
npc  = round(isz(2)/psz(2));

fprintf('log-posterior\t%3.2e\n', evaluate_log_posterior(this, uv));
    
energy = zeros(1,nP);

indicator = zeros(npr, npc);


PSZ = [10 5 2];

for itr = 1:niters;
    
    for i = 1:max(L(:));

        % Crop a rectangle region that contains the segment
        mask    = (L == i);
        indx    = find(mask);
        [iy ix] = ind2sub([H W], indx);

        xt  = min(ix);
        xb  = max(ix);
        yt  = min(iy);
        yb  = max(iy);

        % compute energy function for each candidate segment
        %   not trivial because the change in spatial term is around the
        %   segment boundaries
        
    end;
    
end;


% do a little gradient ascent
niters = 100;
tau    = 0.01;
for i = 1:niters
    duv = evaluate_log_posterior_grad(this, uv);
    uv  = uv + duv*tau;
end;
