function [small layer_uv layer_g layerLabels layer_uvb]  = ...
    compute_flow_hidden_field(this, layerLabels, layer_uv,...
    layer_uvb, pyramid_images, color_pyramid_images, layer_g)
% Subprogram to perform pyramid flow and hiddenf field estimation

T     = length(layer_uv)+1;
small = this;   

% Iterate through all pyramid levels starting at the top
for l = this.pyramid_levels:-1:1
    
    if this.display
        disp(['-Pyramid level: ', num2str(l)])
    end      
    nsz  = [size(pyramid_images{l}, 1) size(pyramid_images{l}, 2)];
    
    % Rescale the flow field 
    for t=1:T-1;
        for i1 = 1:this.nLayers;
            layer_uv{t}{i1}  = resample_flow(layer_uv{t}{i1}, nsz);
            if this.is_symmetric
                layer_uvb{t}{i1} = resample_flow(layer_uvb{t}{i1}, nsz);
            end;
        end;
    end;    
    
    % Rescale the hidden field 
    for t=1:T;  
        layer_g{t} = imresize(layer_g{t}, nsz);  
    end;      

    %Update soft assignment
    for t=1:T
        layerLabels{t} = g2labels(layer_g{t}, small.lambda4);
    end;
    
%     % Rescale the soft label
%     for t=1:T;
%         layerLabels{t}   = imresize(layerLabels{t}, nsz);        
%         % Normalize, though the program does not normalize strictly
%         layerLabels{t}   = layerLabels{t}./repmat(sum(layerLabels{t},3),...
%                            [1 1 this.nLayers]);        
%     end;
    
    
    for i = 1:this.max_iters
        
        fprintf('(%d, %d)\t', l, i);        
        
        % Anneal lambda4 (sigmoid)
        r = small.lambda4/this.lambda4s(l,i);
        small.lambda4 = this.lambda4s(l,i);        
        
        if this.anneal_same_s            
            % Scale g accordingly so that sigma(lambda4*g) retains the same
            for t=1:T
                layer_g{t} = layer_g{t}*r;
            end;            
            small.lambda2 = small.lambda2/(r)^2;
            small.lambda3 = small.lambda3/(r)^2;
            small.lambda7 = small.lambda7/(r)^2;
        end;             
       
        %Update hidden fields simultaneously given current flow estimate
        layer_g = estimate_hidden_field_3D_cgd(small, layer_uv, layer_g,...
            layer_uvb, pyramid_images{l}, color_pyramid_images{l});
        %Update soft assignment
        for t=1:T
            layerLabels{t} = g2labels(layer_g{t}, small.lambda4);
        end;       

        for t=1:T-1           
        
            % Forward flow field
            small.images = cat(3, pyramid_images{l}(:,:,t),...
                pyramid_images{l}(:,:,t+1));            
            if size(color_pyramid_images{l}, 4) > 1
                small.labImage    = color_pyramid_images{l}(:,:,:,t);
            else
                small.labImage    = color_pyramid_images{l}(:,:,t);                
            end;
            
            switch small.meanMode
                case 'projective'
                    layer_mean_affine_uv= fit_projective_layer_motion_g(...
                        layer_uv{t}, layer_g{t});
                case 'affineL1'
                    layer_mean_affine_uv= fit_affine_layer_motion_g_L1(...
                        layer_uv{t}, layer_g{t});
                otherwise
%                     layer_mean_affine_uv = fit_affine_motion_robust(...
%                         layer_uv{t}, this.rho_affine);
                    layer_mean_affine_uv = fit_affine_layer_motion_g(...
                        layer_uv{t}, layer_g{t}, this.rho_affine);                    
                    
            end;            
                      
            for i1 = 1:this.nLayers;
                
                if ~small.affine_mean 
                    % no affine mean
                    layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
                end;
                small.mean_uv     = layer_mean_affine_uv{i1};
                
                % Multiply warped labels at next frame for dataterm
                tmp = imwarp_grad_im(layer_uv{t}{i1}, layerLabels{t+1}(:,:,i1));
                small.layer_s     = cat(3, layerLabels{t}(:,:,i1), tmp);                
                tmp(isnan(tmp))   = 0;
                small.layerLabel  = layerLabels{t}(:,:,i1).*tmp;
                                
                % Run flow method on subsampled images
                if i1 < this.nLayers
                    % hidden field g is used in data term
                    layer_uv{t}{i1} = compute_flow_base(small,...
                        layer_uv{t}{i1}, layer_g{t}(:,:,i1), layer_g{t+1}(:,:,i1));
                else
                    layer_uv{t}{i1} = compute_flow_base(small, layer_uv{t}{i1});
                end;
                
            end;
            
            if this.is_symmetric
            
                % Backward flow field
                switch small.meanMode
                    case 'projective'
                        layer_mean_affine_uv= fit_projective_layer_motion_g(...
                            layer_uvb{t}, layer_g{t+1});
                    case 'affineL1'
                        layer_mean_affine_uv= fit_affine_layer_motion_g_L1(...
                            layer_uvb{t}, layer_g{t+1});
                    otherwise                        
%                         layer_mean_affine_uv = fit_affine_motion_robust(...
%                             layer_uvb{t}, this.rho_affine);
                    layer_mean_affine_uv = fit_affine_layer_motion_g(...
                        layer_uvb{t}, layer_g{t+1}, this.rho_affine);                    
                    
                end;
                
                small.images = cat(3, pyramid_images{l}(:,:,t+1),...
                    pyramid_images{l}(:,:,t));
                if size(color_pyramid_images{l}, 4) > 1
                    small.labImage    = color_pyramid_images{l}(:,:,:,t+1);
                else
                    small.labImage    = color_pyramid_images{l}(:,:,t+1);
                end;
                
                for i1 = 1:this.nLayers;
                    
                    if ~small.affine_mean
                        % no affine mean
                        layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
                    end;
                    small.mean_uv   = layer_mean_affine_uv{i1};
                    
                    %multiply warped labels
                    tmp = imwarp_grad_im(layer_uvb{t}{i1}, layerLabels{t}(:,:,i1));
                    small.layer_s     = cat(3, layerLabels{t+1}(:,:,i1), tmp);
                    
                    tmp(isnan(tmp)) = 0;
                    small.layerLabel= layerLabels{t+1}(:,:,i1).*tmp;
                    
                    % Run flow method on subsampled images
                    if i1 < this.nLayers
                        layer_uvb{t}{i1} = compute_flow_base(small,...
                            layer_uvb{t}{i1}, layer_g{t+1}(:,:,i1),...
                            layer_g{t}(:,:,i1));
                    else
                        layer_uvb{t}{i1} = compute_flow_base(small,...
                            layer_uvb{t}{i1});
                    end;
                end;
                
            end;

        end;
        
        %% save intermediate results for debugging
        for t=1:T-1
            tmp_uv = composite_final_flow_g(layer_uv{t}, layer_g{t});
            %fn = [this.fileinfo 'cont' filesep this.extParams.info ...
            fn = [this.fileinfo filesep this.extParams.info ...
              sprintf('flow_l%02d_i%02d_t%d.png', l, i, t)];        
            imwrite(uint8(flowToColor(tmp_uv)), fn);
        end;
        for t=1:T
            tmp_seg = threshold_g(layer_g{t});
            %fn = [this.fileinfo 'cont' filesep this.extParams.info ...
            fn = [this.fileinfo filesep this.extParams.info ...
              sprintf('seg_l%02d_i%02d_t%d.png', l, i, t)];        
            imwrite(uint8(seg2color(tmp_seg)), fn);
        end;
          
          
    end % of max_iter
    
end; % pyramid level
