function [uv]= compute_flow_sym_one_layer(this, init, gt)
%%
%COMPUTE_FLOW   Compute flow field
%   UV = COMPUTE_FLOW(THIS[, INIT]) computes the flow field UV with
%   algorithm THIS and the optional initialization INIT.
%  

%   Compute forward and backward flow fields at the same time, with
%   occlusion reasoning

%   This is a member function of the class 'layer_optical_flow'. 

%   Author: Deqing Sun, Department of Computer Science, Brown University
%   Contact: dqsun@cs.brown.edu
%   $Date: 2007-- $

% Copyright 2007-2008, Brown University, Providence, RI.
%
%                         All Rights Reserved
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of the author and Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
%
% THE AUTHOR AND BROWN UNIVERSITY DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR OR BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  

tic
% disp('');

this.lambda4 = this.lambda4s(this.pyramid_levels, 1);

uv = init{1};
T   =  length(uv)+1;

if size(this.images,4) > 1
    grayImages = [];
    labImages = [];
    for i = 1:size(this.images,4)
        grayImages = cat(3, grayImages, double(rgb2gray(uint8(this.images(:,:,:,i)))) );
        
        im1 = RGB2Lab(this.images(:,:,:,i));
        for j = 1:size(im1, 3);
            im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
        end;
        labImages = cat(4, labImages, im1);
    end;    
else
    grayImages = double(this.images);
    labImages  = double(this.images);
end;

% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof(grayImages, 1/8, 100, this.alp);
else
    images  = scale_image(grayImages, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(labImages, f, this.pyramid_levels, 1/this.pyramid_spacing);


 for l = this.pyramid_levels:-1:1
    
    if this.display
        disp(['-Pyramid level: ', num2str(l)])
    end      

    nsz  = [size(pyramid_images{l}, 1) size(pyramid_images{l}, 2)];
    
    % Rescale the flow field 
    for t=1:T-1;        
        uv{t}  = resample_flow(uv{t}, nsz);
    end;    
    
    for i = 1:this.max_iters
        
        for t=1:T-1
            
            % Forward flow field
            this.images = cat(3, pyramid_images{l}(:,:,t), pyramid_images{l}(:,:,t+1));
            if size(color_pyramid_images{l}, 4) > 1
                this.labImage    = color_pyramid_images{l}(:,:,:,t);
            else
                this.labImage    = color_pyramid_images{l}(:,:,t);
            end;
            
%             switch this.meanMode
%                 case 'projective'
%                     layer_mean_affine_uv = fit_projective_layer_motion_g(layer_uv{t}, layer_g{t});
%                 otherwise
%                     layer_mean_affine_uv = fit_affine_layer_motion_one_layer(uv{t});
%             end;

            
            
            layer_mean_affine_uv = fit_affine_layer_motion_one_layer(uv{t});            
            if ~this.affine_mean
                % no affine mean
                layer_mean_affine_uv = 0*layer_mean_affine_uv;
            end;
            this.mean_uv     = layer_mean_affine_uv;
            
            % Multiply warped labels at next frame for dataterm
            this.layerLabel  = ones(size(uv{t}, 1),size(uv{t}, 2));
            this.layer_s     = ones(size(uv{t}, 1),size(uv{t}, 2), 2);
            
            uv{t} = compute_flow_base(this, uv{t});
        end;
    end;

end;

fprintf('flow estimation finished, %3.2f minutes passed \n', toc/60);