function [layerLabels, layer_uvf, layer_uvb, pyramid_images, color_pyramid_images, layer_g, seg, incon] = compute_init(this, init)
%%
% Frame size
sz = [size(this.images, 1), size(this.images, 2)];


uvf = init{1};
uvb = init{2};
T   =  length(uvf)+1;
nL  = this.nLayers;

% Images
if size(this.images,4) > 1
    grayImages = [];
    labImages = [];
    for i = 1:size(this.images,4)
        grayImages = cat(3, grayImages, double(rgb2gray(uint8(this.images(:,:,:,i)))) );
        
        im1 = RGB2Lab(this.images(:,:,:,i));
        for j = 1:size(im1, 3);
            im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
        end;
        labImages = cat(4, labImages, im1);
    end;    
else
    grayImages = double(this.images);    
    % Duplicate gray for color-modulated hidden field prior
    labImages = [];        
    for i = 1:size(this.images,3)
        im1 = this.images(:,:,i);
        labImages = cat(4, labImages, repmat(im1, [1 1 3]));
    end;        
end;

% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof(grayImages, 1/8, 100, this.alp);
else
    images  = scale_image(grayImages, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(labImages, f, this.pyramid_levels, 1/this.pyramid_spacing);


% % perform ROF on each image pyramid
% gray_pyramid_images    = compute_image_pyramid(grayImages, f, this.pyramid_levels, 1/this.pyramid_spacing);
% for l = 1:length(gray_pyramid_images)
%     pyramid_images{l}  = structure_texture_decomposition_rof(gray_pyramid_images{l}, 1/8, 100, this.alp);
% end;


% Flow field
layer_uvf = cell(1,T-1); layer_uvb = cell(1,T-1); 
for t=1:T-1
    for i=1:nL
        layer_uvf{t}{i}  = uvf{t};
        layer_uvb{t}{i}  = uvb{t};
    end;
end;
% seg = compute_ini_seg_merge(uvf, uvb, nL, 1, sz);
% seg = compute_ini_seg_merge(uvf, uvb, nL, this.nCandidate, sz);
% seg = compute_ini_seg(uvf, uvb, 8, 25, sz);
% Compute initial segmentation
seg = compute_ini_seg(uvf, uvb, nL, this.nCandidate, sz);

% seg with auxiliary seg information (graph-based seg with flow and images)
%imf = cat(3, grayImages(:,:,1), scale_image(uvf{1}, 0, 255));  %1-2 runs

% flow + intenisty

% imf = cat(3, uint8(grayImages(:,:,1)), scale_image(uvf{1}, 0, 255));    % 3-after runs
% % sigma = 0.5;k= 500; min_size = 500;   % 1run
% % [r g b] = segment_matlab_wrapper(imf(:,:,1),imf(:,:,2),imf(:,:,3), sigma, k,min_size);
% imf = double(imf);
% sigma = 0.57; k= 500; min_size = 300; % 2-3 runs
% % sigma = 0.6; k= 500; min_size = 300; % 4th run
% [r g b] = segment_matlab_wrapper(imf(:,:,1)*0.7,imf(:,:,2),imf(:,:,3), sigma, k,min_size);


% by flow only
% sigma = 0.65; k= 500; min_size = 300;
% imf = double(flowToColor(uvf{1}));
% [r g b] = segment_matlab_wrapper(imf(:,:,1),imf(:,:,2),imf(:,:,3), sigma, k,min_size);
% 
% aux_seg = convert_random_seg(cat(3,r,g,b));
% seg = compute_ini_seg_aux_seg(uvf, uvb, nL, this.nCandidate, sz, aux_seg);
% 
% figure; imagesc(seg{1});
% disp('test');
% seg = compute_ini_seg(uvf, uvb, nL, 1, sz);

% human labeled GT for hand for test
% s=load('data/hand-human-gt.mat'); disp('human label');
% seg{1} = s.seg;
% seg{2} = s.seg2;

%Order by speed of each layer
u = uvf{1}(:,:,1); v = uvf{1}(:,:,2);
for i = 1:nL    
    tmp_u = u(seg{1}==i);
    tmp_v = v(seg{1}==i);
    muv(i) = (mean(tmp_u(:)))^2 + (mean(tmp_v(:)))^2;
end;
[tmp ind] = sort(muv, 'descend');
for t=1:T
    tmp = seg{t};
    for i = 1:nL        
        seg{t}(tmp== ind(i)) = i;        
    end;
end;


%%%% Compute hidden fields for each layer
incon = cell(1,T-1);
for t=1:T-1
    tmp = warp_backward_segmentation(seg{t+1}, uvf{t});
    incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);
end;
tmp = warp_backward_segmentation(seg{T-1}, uvb{T-1});
t = T; incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);

[layer_g layerLabels] = convert_ini_seg_to_support_function(this, seg, incon);
 
% visG   = this.visG;
% ratio  = 10; % this.ratioG;
% for t=1:T
%     layer_g{t} = - ones([sz nL-1]) * visG;
%     for l=1:nL-1
%         tmp = layer_g{t}(:,:,l);
%         tmp(seg{t} ==l) =  visG;
%         layer_g{t}(:,:,l) = tmp;      
%     end;   
%     % mark inconsistency
%     layer_g{t}(incon{t} ==1) = layer_g{t}(incon{t} ==1)/ratio;
%     layerLabels{t} = g2labels(layer_g{t}, this.lambda4);
% end;
    
for t=1:T-1
    if this.affine_ini_all
        % Use affine motion for all the pixels
%         layer_uvf{t} = fit_affine_layer_motion_g(layer_uvf{t}, layer_g{t});
%         layer_uvb{t} = fit_affine_layer_motion_g(layer_uvb{t}, layer_g{t+1});
%         fprintf('affine: all\t');

        layer_uvf{t} = affine_possion_interpolate_invisible_g(layer_uvf{t}, layer_g{t});
        layer_uvb{t} = affine_possion_interpolate_invisible_g(layer_uvb{t}, layer_g{t+1});
        fprintf('affine+possion interpolation of invisible\n');
    else
        % Interpolate invisible pixels for each layer
        layer_uvf{t} = affine_interpolate_invisible_g(layer_uvf{t}, layer_g{t}, this.meanMode);
        layer_uvb{t} = affine_interpolate_invisible_g(layer_uvb{t}, layer_g{t+1}, this.meanMode);
        fprintf('affine: only invisible\n');

%         layer_uvf{t} = possion_interpolate_invisible_g(layer_uvf{t}, layer_g{t});
%         layer_uvb{t} = possion_interpolate_invisible_g(layer_uvb{t}, layer_g{t+1});
%         fprintf('possion interpolation of invisible\t');

%         layer_uvf{t} = affine_possion_interpolate_invisible_g(layer_uvf{t}, layer_g{t});
%         layer_uvb{t} = affine_possion_interpolate_invisible_g(layer_uvb{t}, layer_g{t+1});

        % interpolate only large discrepancy
%         disp('no interp');
        
        % no interpolation
    end;
end;


% fprintf('\n');