function [layerLabels layer_uvf, layer_uvb, layer_g, seg, incon] =...
            compute_init_layer_support(this, init, init_seg)
%%
% compute affine clustering of flow field at center key frame, propagate
% the resutls to other frames

uvf = init{1};
uvb = init{2};
T   =  length(uvf)+1;
nL  = this.nLayers;

% compute initial segmentation
if exist('init_seg', 'var');
    % use provide initialization
    seg = compute_ini_seg(uvf, uvb, nL, this.nCandidate,...
        this.rho_affine, init_seg);
elseif this.is_hand_seg
    % load hand segmented initialization
    seg = compute_ini_hand_seg(uvf, uvb, this.extParams.seqName);    
else
    % robust affine clustering
    seg = compute_ini_seg(uvf, uvb, nL, this.nCandidate, this.rho_affine);    
end;
nL = max(seg{1}(:));

% compute initial flow field for each layer
%   visible pixels: use initial flow field
%   invisible ones: use affine motion to interpolate
layer_uvf = cell(1,T-1); 
layer_uvb = cell(1,T-1); 
for t=1:T-1
    for i=1:nL
        layer_uvf{t}{i}  = uvf{t};
        layer_uvb{t}{i}  = uvb{t};
    end;
end;

% order layer by fast to slow order
if ~this.is_hand_seg    
    % order by average speed magnitude of each layer
    u = uvf{1}(:,:,1); v = uvf{1}(:,:,2);
    for i = 1:nL
        tmp_u = u(seg{1}==i);
        tmp_v = v(seg{1}==i);
        muv(i) = (mean(tmp_u(:)))^2 + (mean(tmp_v(:)))^2;
    end;
    [tmp ind] = sort(muv, 'descend');
    for t=1:T
        tmp = seg{t};
        for i = 1:nL
            seg{t}(tmp== ind(i)) = i;
        end;
    end;
end;

%%%% Compute hidden fields (support function) for each layer
incon = cell(1,T-1);
for t=1:T-1
    tmp      = warp_backward_segmentation(seg{t+1}, uvf{t});
    incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);
end;
tmp = warp_backward_segmentation(seg{T-1}, uvb{T-1});
t   = T; 
incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);

[layer_g layerLabels] = convert_ini_seg_to_support_function(this, seg, incon); 
  

for t=1:T-1
    if this.affine_ini_all
        % Use affine motion for all the pixels
        layer_uvf{t} = fit_affine_layer_motion_g(layer_uvf{t}, layer_g{t});
        layer_uvb{t} = fit_affine_layer_motion_g(layer_uvb{t}, layer_g{t+1});
        fprintf('affine: all\t');        
    else
        % Interpolate invisible pixels for each layer
        layer_uvf{t} = affine_interpolate_invisible_g(layer_uvf{t},...
            layer_g{t}, this.meanMode, this.rho_affine);
        layer_uvb{t} = affine_interpolate_invisible_g(layer_uvb{t},...
            layer_g{t+1}, this.meanMode, this.rho_affine);
        fprintf('affine: only invisible\n');
    end;
end;
