function [layerLabels, layerLabels2, layer_uv, layer_uv2, pyramid_images,...
         color_pyramid_images, layer_g, layer_g2] = compute_init_nframes(this, init)

% Frame size
sz = [size(this.images, 1), size(this.images, 2)];

% If we have no initialization argument, initialize with all zeros
if (nargin < 2)
    uv = zeros([sz, 2]);
    uv2 = uv;
else
    uv = init(:,:,1:2);
    uv2 = init(:,:,3:4);
end

layer_uv = cell(this.nLayers,1);
layer_uv2 = cell(this.nLayers,1);
%%
for i = 1:this.nLayers;
    layer_uv{i} = uv;
    layer_uv2{i} = uv2;
end;
%%

% Automatic determine pyramid level
% this.pyramid_levels  =  1 + floor( log(min(size(images, 1), size(images,2))/16) / log(this.pyramid_spacing) );

% Initialize segmentation from K-means flow vectors + intensity as features
% this.pyramid_levels  = 2;

nL = this.nLayers;

nCandidate = 25;

% [seg seg2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz);

% human labeled GT for hand
% load('data/hand-human-gt.mat'); disp('human label');


%%
%Order by speed of each layer
%
% figure; subplot(1,2,1); imagesc(seg);
u = uv(:,:,1); v = uv(:,:,2);
for i = 1:nL    
    tmp_u = u(seg==i);
    tmp_v = v(seg==i);
    muv(i) = (mean(tmp_u(:)))^2 + (mean(tmp_v(:)))^2;
    %muv(i) = mean(abs(tmp_u(:)))^2 + mean(abs(tmp_v(:)))^2;
    %muv(i) = median(tmp_u(:))^2 + median(tmp_v(:))^2;
%     muv(i) = max(tmp_u(:))^2 + max(tmp_v(:))^2;
     %muv(i) = max(abs(tmp_u(:)))^2 + max(abs(tmp_v(:)))^2;
end;

[tmp ind] = sort(muv, 'descend');
seg3 = seg; seg4 = seg2;

for i = 1:nL   
    seg(seg3 == ind(i)) = i;
    seg2(seg4 == ind(i)) = i;    
end;

% subplot(1,2,2); imagesc(seg);
% [seg seg2 layer_uv layer_uv2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz);

% % UI to get correct depth order
% figure; imshow(uint8(this.colorImages(:,:,:,1)));
% figure; imagesc(seg);
% 
% seg3 = seg; seg4 = seg2;
% for i = 1:nL
%     rect = floor(getrect());    
%     tmp = seg(rect(2):rect(2)+rect(4), rect(1):rect(1)+rect(3));    
%     
%     c  = median(tmp(:));  
%     
%     seg(seg3 == c) = i;
%     seg2(seg4 == c) = i;    
% end;


% mark inconsistent pixels
% tmp = warp_backward(uv, seg2, 'nearest');
tmp = warp_backward_segmentation(seg2, uv);
in1 = repmat(tmp~=seg, [1 1 nL]);
% tmp = warp_backward(uv2, seg, 'nearest');
tmp = warp_backward_segmentation(seg, uv2);
in2 = repmat(tmp~=seg2, [1 1 nL]);
% figure; subplot(1,2,1); imagesc(in1(:,:,1)); subplot(1,2,2); imagesc(in2(:,:,1));
%%
maxL = 1-1e-1;
minL = (1-maxL)/(nL-1);
layerLabels = ones([sz this.nLayers]) * minL;
layerLabels2 = layerLabels;

[indr indc] = ind2sub(sz, reshape(1:prod(sz), sz) );

indx = sub2ind([sz this.nLayers], indr, indc, seg);
layerLabels(indx) = maxL;

indx = sub2ind([sz this.nLayers], indr, indc, seg2);
layerLabels2(indx) = maxL;

%Mark inconsistent pixels, equal s always assign them to first layer
layerLabels(in1) = 1/nL;
layerLabels2(in2) = 1/nL;
%disp('equal confidence for inconsistent segmentation');

% slightly tilted so that max operator returns the same segmentation
sf = 1e-3;
for i = 1:nL
    tmp = in1(:,:,1) ==1 & seg ==i; 
    s = layerLabels(:,:,i);
    s(tmp) = s(tmp)+ sf;
    layerLabels(:,:,i) = s;
    
    tmp = in2(:,:,1) ==1 & seg2 ==i;
    s = layerLabels2(:,:,i);
    s(tmp) = s(tmp)+ sf;
    layerLabels2(:,:,i) = s;
end;

layerLabels   = layerLabels./repmat(sum(layerLabels,3), [1 1 this.nLayers]);
layerLabels2  = layerLabels2./repmat(sum(layerLabels2,3), [1 1 this.nLayers]);
    


layer_g = label2g(layerLabels, this.lambda4);
layer_g2 = label2g(layerLabels2, this.lambda4);

% layer_g(in1(:,:,1:nL-1)) = 0;
% layer_g2(in2(:,:,1:nL-1)) = 0;

% for i = 1:nL
%     layerLabels(in1(:,:,1) == i)  = 0.5^i;
%     layerLabels2(in2(:,:,1) == i) = 0.5^i;    
% end;
% disp('equal g for inconsistent segmentation');




% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof( this.images, 1/8, 100, this.alp);
    fprintf('.....\n');
else
    images  = scale_image(this.images, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(this.colorImages, f, this.pyramid_levels, 1/this.pyramid_spacing);



function [seg seg2 layer_uv layer_uv2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz)

for i = 1:nCandidate
    [A{i} score(i) segs{i}] =robust_fit_affine_motion(uv, nL);
    %figure; imagesc(segs{i}); title(num2str(score(i)));
end;
[tmp ind] = min(score);
seg = segs{ind};
seg2 = warp_backward_segmentation(seg, uv2);

% if nargout >=4
%     layer_uv = affine2layer(A{ind}, [size(uv,1) size(uv,2)]);
% end;
% A1 = A{ind};
% 
% for i = 1:nCandidate
%     [A{i} score(i) segs{i}] =robust_fit_affine_motion(uv2, nL);
% end;
% [tmp ind] = min(score);
% seg2 = segs{ind};
% A2 = A{ind};
% 
% %Make two segmentation consistent to each other
% c = ones(nL,1);
% for i =1:nL
%     dis = sum( sum( (A1+ repmat(A2(:, i, :), [1 nL 1])).^2, 3), 1);
%     [tmp ind] = min(dis);
%     c(i) = ind;
% end;
% seg3 = seg2;
% A3 = A2;
% for i =1:nL
%     seg2(seg3==i) = c(i);
%     A2(:,c(i), :) = A3(:,i, :);
% end;
% 
% if nargout >=4
%     layer_uv2 = affine2layer(A2, [size(uv,1) size(uv,2)]);
% end;


% % Test temporal consistency
% tmp = warp_backward(uv, seg2, 'nearest');
% tmp(isnan(tmp)) = seg(isnan(tmp));
% % figure;imagesc(tmp~=seg)
% incon = sum(tmp(:)~=seg(:))/length(seg(:));
% 
% % for debug
% % figure; subplot(1,2,1); imagesc(seg);axis square;
% % subplot(1,2,2);imagesc(seg2); axis square; title('affine');
% 
% if incon > 0.1
% %     fprintf('robust affine fitting fails (%3.3f), uses seg of first for both\n', incon);
% %     seg2 = seg;
%     fprintf('robust affine fitting fails (%3.3f), use warped seg of first\n', incon);
%     seg2 = warp_backward_segmentation(seg, uv2);
% end;

% if incon > 0.05
%     
%     fprintf('robust affine fitting fails (%3.3f), uses k-means to fit constant flow\n', incon);
%     % use k-means for complex scenes
%     u = uv(:,:,1); v = uv(:,:,2);
%     for i = 1:nCandidate
%         [y_cluster{i} centers{i} tmp] = kmeans([u(:) v(:)], nL);
%         score(i) = sum(tmp);
%     end;
%     [tmp ind] = min(score);
%     seg = reshape(y_cluster{ind(1)}, sz);
%     center1 = centers{ind(1)};
%     
%     u = uv2(:,:,1); v = uv2(:,:,2);
%     for i = 1:nCandidate
%         [y_cluster{i} centers{i} tmp] = kmeans([u(:) v(:)], nL);
%         score(i) = sum(tmp);
%     end;
%     
%     [tmp ind] = min(score);
%     seg2 = reshape(y_cluster{ind(1)}, sz);
%     center2 = centers{ind(1)};   
%         
%     for i =1:nL
%         dis = sum((center1 + repmat(center2(i,:), [nL 1])).^2, 2);
%         [tmp ind] = min(dis);
%         c(i) = ind;
%     end;
%     
%     seg3 = seg2;
%     center3 = center2;
%     for i =1:nL
%         seg2(seg3==i) = c(i);
%         center2(i,:) = center3(c(i),:);
%     end;    
% 
function [layerLabels, layerLabels2, layer_uv, layer_uv2, pyramid_images, color_pyramid_images, layer_g, layer_g2] = compute_init(this, init)

% Frame size
sz = [size(this.images, 1), size(this.images, 2)];

% If we have no initialization argument, initialize with all zeros
if (nargin < 2)
    uv = zeros([sz, 2]);
    uv2 = uv;
else
    uv = init(:,:,1:2);
    uv2 = init(:,:,3:4);
end

layer_uv = cell(this.nLayers,1);
layer_uv2 = cell(this.nLayers,1);
%%
for i = 1:this.nLayers;
    layer_uv{i} = uv;
    layer_uv2{i} = uv2;
end;
%%

% Automatic determine pyramid level
% this.pyramid_levels  =  1 + floor( log(min(size(images, 1), size(images,2))/16) / log(this.pyramid_spacing) );

% Initialize segmentation from K-means flow vectors + intensity as features
% this.pyramid_levels  = 2;

nL = this.nLayers;

% % cheating for venus
% load('data/venus-layer-gt.mat');
% %load('data/venus-layer-gt-wrong-order.mat'); % background is in front
% % end of cheating for venus
% load('data/venus-gt-seg-new.mat');

% Fit to several affine motion layers

% % cheating for venus
% load('data/venus-layer-good-ini.mat');  %seg2 = seg; disp('same seg'); 
% seg2 = warp_backward_segmentation(seg, uv2);
% load('data/venus-layer-flow-good-ini.mat');
% % load('data/venus-layer-good-ini2.mat');
% load('data/venus-layer-bad-ini.mat'); % background is in front
% % end of cheating for venus

% kemans for urban2
% load('data/urban2-layer-kmeans-ini.mat');


% load('data/rubberwhale-affine-ini.mat'); fprintf('affine'); 
% load('data/rubberwhale-affine-flow-ini.mat');

% load('data/rubberwhale-hand-seg.mat');  % hand segmented 12 layers

% load('data/rubberwhale-hand-seg-4layers.mat'); fprintf('hand cheat');
% load('data/rubberwhale-hand-seg-12layer-correct-depth-order.mat');  % hand segmented 12 layers

% load('data/rubberwhale-hand-seg-9layer-cropped.mat');  % hand segmented cropped region with 9 layers

% load('data/venus-3layer-affine-warp-ini-seg.mat-wrong-order.mat');disp('venus wrong order')
% load('data/rubberwhale-3layer-affine-warp-ini-seg.mat-wrong-order.mat');disp('rubberwhale wrong order')
% load('data/grove3-3layer-affine-warp-ini-seg.mat-wrong-order.mat'); disp('urban3 wrong order')
% load('data/urban2-3layer-affine-warp-ini-seg.mat-wrong-order.mat'); disp('urban2 wrong order')
% load('data/urban3-3layer-affine-warp-ini-seg.mat-wrong-order.mat'); disp('urban3 wrong order')


% load('data/venus-3layer-affine-warp-ini-seg.mat-correct-order.mat');disp('venus correct order')
% load('data/rubberwhale-3layer-affine-warp-ini-seg.mat-correct-order.mat');disp('rubberwhale correct order')
% load('data/grove3-3layer-affine-warp-ini-seg.mat-correct-order.mat'); disp('grove3 correct order')
% load('data/urban2-3layer-affine-warp-ini-seg.mat-correct-order.mat'); disp('urban2 correct order')
% load('data/urban3-3layer-affine-warp-ini-seg.mat-correct-order.mat'); disp('urban3 correct order')

% load('data/seg/train_4_018.mat');
% load('data/seg/train_4_019.mat');

% seg3 = seg; seg4 = seg2;
% i = 3;  j = 1; seg(seg3==i) = j; seg2(seg4==i) =j;
% i = 2;  j = 3; seg(seg3==i) = j; seg2(seg4==i) =j;
% i = 1;  j = 2; seg(seg3==i) = j; seg2(seg4==i) =j;

nCandidate = 25;

% [seg seg2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz);

% human labeled GT for hand
load('data/hand-human-gt.mat'); disp('human label');

%% Irani's trick, subtract fitted affine and use residual to infer relative
%   motion and depth

% maxL = 1-1e-1;
% minL = (1-maxL)/(nL-1);
% tmpLabels = ones([sz this.nLayers]) * minL;
% [indr indc] = ind2sub(sz, reshape(1:prod(sz), sz) );
% indx = sub2ind([sz this.nLayers], indr, indc, seg);
% tmpLabels(indx) = maxL;
% 
% tmp_uv = fit_affine_layer_motion(layer_uv, tmpLabels);
% 
% %fuv = composite_final_flow(tmp_uv, tmpLabels);
% 
% for i = 1:nL    
%     res = layer_uv{i} - tmp_uv{i};
%     res = sqrt(sum(res.^2, 3)); 
%     tmp = res(seg==i); 
%     muv(i) = mean(tmp(:));    
%     
% end;




%%
%Order by speed of each layer
%
% figure; subplot(1,2,1); imagesc(seg);
u = uv(:,:,1); v = uv(:,:,2);
for i = 1:nL    
    tmp_u = u(seg==i);
    tmp_v = v(seg==i);
    muv(i) = (mean(tmp_u(:)))^2 + (mean(tmp_v(:)))^2;
    %muv(i) = mean(abs(tmp_u(:)))^2 + mean(abs(tmp_v(:)))^2;
    %muv(i) = median(tmp_u(:))^2 + median(tmp_v(:))^2;
%     muv(i) = max(tmp_u(:))^2 + max(tmp_v(:))^2;
     %muv(i) = max(abs(tmp_u(:)))^2 + max(abs(tmp_v(:)))^2;
end;

[tmp ind] = sort(muv, 'descend');
seg3 = seg; seg4 = seg2;

for i = 1:nL   
    seg(seg3 == ind(i)) = i;
    seg2(seg4 == ind(i)) = i;    
end;

% subplot(1,2,2); imagesc(seg);
% [seg seg2 layer_uv layer_uv2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz);

% % UI to get correct depth order
% figure; imshow(uint8(this.colorImages(:,:,:,1)));
% figure; imagesc(seg);
% 
% seg3 = seg; seg4 = seg2;
% for i = 1:nL
%     rect = floor(getrect());    
%     tmp = seg(rect(2):rect(2)+rect(4), rect(1):rect(1)+rect(3));    
%     
%     c  = median(tmp(:));  
%     
%     seg(seg3 == c) = i;
%     seg2(seg4 == c) = i;    
% end;

%% Modulate flow field for each layer
% sigma_uv = 10;
% G = fspecial('gaussian', 2*round(sigma_uv*3)+1, sigma_uv);
% for i = 1:nL
%     tmp = seg == i;    
%     mask = imfilter(double(tmp), G, 'symmetric');
%     mask(tmp) = 1;    
%     layer_uv{i} = layer_uv{i}.*repmat(mask, [ 1 1 2]);
% 
%     tmp = seg2 == i;
%     mask = imfilter(double(tmp), G, 'symmetric');
%     mask(tmp) = 1;
%     layer_uv2{i} = layer_uv2{i}.*repmat(mask, [ 1 1 2]);
% end;
% fprintf('no mask');
%%

% mark inconsistent pixels
% tmp = warp_backward(uv, seg2, 'nearest');
tmp = warp_backward_segmentation(seg2, uv);
in1 = repmat(tmp~=seg, [1 1 nL]);
% tmp = warp_backward(uv2, seg, 'nearest');
tmp = warp_backward_segmentation(seg, uv2);
in2 = repmat(tmp~=seg2, [1 1 nL]);
% figure; subplot(1,2,1); imagesc(in1(:,:,1)); subplot(1,2,2); imagesc(in2(:,:,1));
%%
maxL = 1-1e-1;
minL = (1-maxL)/(nL-1);
layerLabels = ones([sz this.nLayers]) * minL;
layerLabels2 = layerLabels;

[indr indc] = ind2sub(sz, reshape(1:prod(sz), sz) );

indx = sub2ind([sz this.nLayers], indr, indc, seg);
layerLabels(indx) = maxL;

indx = sub2ind([sz this.nLayers], indr, indc, seg2);
layerLabels2(indx) = maxL;

%Mark inconsistent pixels, equal s always assign them to first layer
layerLabels(in1) = 1/nL;
layerLabels2(in2) = 1/nL;
%disp('equal confidence for inconsistent segmentation');

% slightly tilted so that max operator returns the same segmentation
sf = 1e-3;
for i = 1:nL
    tmp = in1(:,:,1) ==1 & seg ==i; 
    s = layerLabels(:,:,i);
    s(tmp) = s(tmp)+ sf;
    layerLabels(:,:,i) = s;
    
    tmp = in2(:,:,1) ==1 & seg2 ==i;
    s = layerLabels2(:,:,i);
    s(tmp) = s(tmp)+ sf;
    layerLabels2(:,:,i) = s;
end;

layerLabels   = layerLabels./repmat(sum(layerLabels,3), [1 1 this.nLayers]);
layerLabels2  = layerLabels2./repmat(sum(layerLabels2,3), [1 1 this.nLayers]);
    


layer_g = label2g(layerLabels, this.lambda4);
layer_g2 = label2g(layerLabels2, this.lambda4);

% layer_g(in1(:,:,1:nL-1)) = 0;
% layer_g2(in2(:,:,1:nL-1)) = 0;

% for i = 1:nL
%     layerLabels(in1(:,:,1) == i)  = 0.5^i;
%     layerLabels2(in2(:,:,1) == i) = 0.5^i;    
% end;
% disp('equal g for inconsistent segmentation');




% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof( this.images, 1/8, 100, this.alp);
    fprintf('.....\n');
else
    images  = scale_image(this.images, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(this.colorImages, f, this.pyramid_levels, 1/this.pyramid_spacing);



function [seg seg2 layer_uv layer_uv2] = compute_ini_seg(uv, uv2, nL, nCandidate, sz)

for i = 1:nCandidate
    [A{i} score(i) segs{i}] =robust_fit_affine_motion(uv, nL);
    %figure; imagesc(segs{i}); title(num2str(score(i)));
end;
[tmp ind] = min(score);
seg = segs{ind};
seg2 = warp_backward_segmentation(seg, uv2);

% if nargout >=4
%     layer_uv = affine2layer(A{ind}, [size(uv,1) size(uv,2)]);
% end;
% A1 = A{ind};
% 
% for i = 1:nCandidate
%     [A{i} score(i) segs{i}] =robust_fit_affine_motion(uv2, nL);
% end;
% [tmp ind] = min(score);
% seg2 = segs{ind};
% A2 = A{ind};
% 
% %Make two segmentation consistent to each other
% c = ones(nL,1);
% for i =1:nL
%     dis = sum( sum( (A1+ repmat(A2(:, i, :), [1 nL 1])).^2, 3), 1);
%     [tmp ind] = min(dis);
%     c(i) = ind;
% end;
% seg3 = seg2;
% A3 = A2;
% for i =1:nL
%     seg2(seg3==i) = c(i);
%     A2(:,c(i), :) = A3(:,i, :);
% end;
% 
% if nargout >=4
%     layer_uv2 = affine2layer(A2, [size(uv,1) size(uv,2)]);
% end;


% % Test temporal consistency
% tmp = warp_backward(uv, seg2, 'nearest');
% tmp(isnan(tmp)) = seg(isnan(tmp));
% % figure;imagesc(tmp~=seg)
% incon = sum(tmp(:)~=seg(:))/length(seg(:));
% 
% % for debug
% % figure; subplot(1,2,1); imagesc(seg);axis square;
% % subplot(1,2,2);imagesc(seg2); axis square; title('affine');
% 
% if incon > 0.1
% %     fprintf('robust affine fitting fails (%3.3f), uses seg of first for both\n', incon);
% %     seg2 = seg;
%     fprintf('robust affine fitting fails (%3.3f), use warped seg of first\n', incon);
%     seg2 = warp_backward_segmentation(seg, uv2);
% end;

% if incon > 0.05
%     
%     fprintf('robust affine fitting fails (%3.3f), uses k-means to fit constant flow\n', incon);
%     % use k-means for complex scenes
%     u = uv(:,:,1); v = uv(:,:,2);
%     for i = 1:nCandidate
%         [y_cluster{i} centers{i} tmp] = kmeans([u(:) v(:)], nL);
%         score(i) = sum(tmp);
%     end;
%     [tmp ind] = min(score);
%     seg = reshape(y_cluster{ind(1)}, sz);
%     center1 = centers{ind(1)};
%     
%     u = uv2(:,:,1); v = uv2(:,:,2);
%     for i = 1:nCandidate
%         [y_cluster{i} centers{i} tmp] = kmeans([u(:) v(:)], nL);
%         score(i) = sum(tmp);
%     end;
%     
%     [tmp ind] = min(score);
%     seg2 = reshape(y_cluster{ind(1)}, sz);
%     center2 = centers{ind(1)};   
%         
%     for i =1:nL
%         dis = sum((center1 + repmat(center2(i,:), [nL 1])).^2, 2);
%         [tmp ind] = min(dis);
%         c(i) = ind;
%     end;
%     
%     seg3 = seg2;
%     center3 = center2;
%     for i =1:nL
%         seg2(seg3==i) = c(i);
%         center2(i,:) = center3(c(i),:);
%     end;    
% 
%     tmp = warp_backward(uv, seg2, 'nearest');
%     tmp(isnan(tmp)) = seg(isnan(tmp));
%     incon = sum(tmp(:)~=seg(:))/length(seg(:));
%     fprintf('k-means consistency %3.3f\n', incon);
%     
%     % % for debug
%     figure; subplot(1,2,1); imagesc(seg);axis square;
%     subplot(1,2,2);imagesc(seg2); axis square; title('k-means');
% 
%     if nargout >=4
%         u = ones(sz);
%         for l = 1:nL
%             layer_uv{l}  = cat(3, u*center1(l,1), u*center1(l,2));
%             layer_uv2{l} = cat(3, u*center2(l,1), u*center2(l,2));
%         end;
%     end;
% 
% end;

function seg2 = warp_backward_segmentation(seg, uvb)

% Replicate so that out of boundary pixels are assigned

rep = ceil(max(abs(uvb(:))));

pad_seg = padarray(seg, rep*[1 1], 'replicate', 'both');        
pad_uvb(:,:,1) = padarray(uvb(:,:,1), rep*[1 1]);        
pad_uvb(:,:,2) = padarray(uvb(:,:,2), rep*[1 1]);        

pad_seg2 = warp_backward(pad_uvb, pad_seg, 'nearest');

seg2 = pad_seg2(rep+1:end-rep, rep+1:end-rep); 

function layer_uv = affine2layer(A, sz)
% convert the affine motion parameters of each layer into dense flow field

[X Y] = meshgrid(1:sz(2), 1:sz(1));
for j = 1:size(A,2)
    u2 = X*A(1,j,1)+Y*A(2,j,1)+A(3,j,1);
    v2 = X*A(1,j,2)+Y*A(2,j,2)+A(3,j,2);
    layer_uv{j} = cat(3, u2, v2);
end;

%     tmp = warp_backward(uv, seg2, 'nearest');
%     tmp(isnan(tmp)) = seg(isnan(tmp));
%     incon = sum(tmp(:)~=seg(:))/length(seg(:));
%     fprintf('k-means consistency %3.3f\n', incon);
%     
%     % % for debug
%     figure; subplot(1,2,1); imagesc(seg);axis square;
%     subplot(1,2,2);imagesc(seg2); axis square; title('k-means');
% 
%     if nargout >=4
%         u = ones(sz);
%         for l = 1:nL
%             layer_uv{l}  = cat(3, u*center1(l,1), u*center1(l,2));
%             layer_uv2{l} = cat(3, u*center2(l,1), u*center2(l,2));
%         end;
%     end;
% 
% end;

function seg2 = warp_backward_segmentation(seg, uvb)

% Replicate so that out of boundary pixels are assigned

rep = ceil(max(abs(uvb(:))));

pad_seg = padarray(seg, rep*[1 1], 'replicate', 'both');        
pad_uvb(:,:,1) = padarray(uvb(:,:,1), rep*[1 1]);        
pad_uvb(:,:,2) = padarray(uvb(:,:,2), rep*[1 1]);        

pad_seg2 = warp_backward(pad_uvb, pad_seg, 'nearest');

seg2 = pad_seg2(rep+1:end-rep, rep+1:end-rep); 

function layer_uv = affine2layer(A, sz)
% convert the affine motion parameters of each layer into dense flow field

[X Y] = meshgrid(1:sz(2), 1:sz(1));
for j = 1:size(A,2)
    u2 = X*A(1,j,1)+Y*A(2,j,1)+A(3,j,1);
    v2 = X*A(1,j,2)+Y*A(2,j,2)+A(3,j,2);
    layer_uv{j} = cat(3, u2, v2);
end;