function layerLabels = estimate_layer_labels_basic_gd(this, layer_uv, prev_layerLabels)

% Only two layers

% Energy function BCE(uv1) sigmoid(g) + BCE(uv2) sigmoid(-g) + lambda2smoothness(g)

% prior is modulated by static image segmentation

nGDiters = 50;
lambda = this.lambda2; % weight on smoothness term for labels

% nL = this.nLayers;
nL = 2;

% Compute BCE for each layer
for i = 1:nL       
   
    It = partial_deriv(this.images, layer_uv{i}, this.interpolation_method, this.deriv_filter);   
    %It = partial_deriv(this.images, real(layer_uv{i}), this.interpolation_method, this.deriv_filter);
    
    a = this.sigmaNs(i);
    % layerLabels(:,:,i) = exp(- It.^2/2/a^2) / (sqrt(2*pi)*a) * this.pis(i);
    aIt2(:,:,i)         = It.^2/2/a^2;    
end;

nL = this.nLayers;
maxL = 1-1e-3;
minL = (1-maxL)/(nL-1);

% convert initial value from sigmoid(x) to x

%BUG: lambda2 is not taken into account

prev_layerLabels(prev_layerLabels>1) = 1;
prev_layerLabels(prev_layerLabels<minL) = minL;
prev_layerLabels = prev_layerLabels./repmat(sum(prev_layerLabels,3), [1 1 this.nLayers]);

g = -log( (1-prev_layerLabels(:,:,1))./prev_layerLabels(:,:,1));
g = g(:);

% Check gradient
% d = checkgrad('compute_g_energy', g, 1e-3, aIt2, lambda);

% Call minimizer
g = minimize(g, @compute_g_energy, nGDiters, aIt2, lambda, this.seg, this.kappa);
% g = minimize(g, @compute_g_energy, nGDiters, aIt2, lambda);

% g = minFunc();

% Convert solution to sigmoid 
tmp = sigmoid(g, 1);
layerLabels = reshape([tmp; 1-tmp], size(prev_layerLabels));



% layerLabels(layerLabels>1) = 1;
% layerLabels(layerLabels<minL) = minL;
% layerLabels = layerLabels./repmat(sum(layerLabels,3), [1 1 this.nLayers]);



function [e d] = compute_g_energy(g, aIt2, lambda, seg, kappa)

% sz = [30 30];
% g  = randn(sz); g= g(:);
% aIt2 = rand([sz 2])*10;
% seg = round(rand(sz)*25);
% lambda = 1;
% kappa = 0.5;
% d = checkgrad('compute_g_energy', g, 1e-3, aIt2, lambda, seg, kappa);

% Compute energy

% data term
l = sigmoid(g);
e = sum(aIt2(:).*[l; 1-l]);

% entropy term
e = e - sum(l.*log(l)) - sum((1-l).*log(1-l));

% spatial term
F =  {[1 -1], [1 -1]'};

g = reshape(g, [size(aIt2,1) size(aIt2,2)]);

if nargin <4
    seg = ones(size(g));        
end;
if nargin <5
    kappa = 1;        
end;
seg = double(seg);

for i = 1:length(F)
    g_ = conv2(g, F{i}, 'valid');    
    seg_ = conv2(seg, F{i}, 'valid');    
    seg_ = 1 - kappa*(seg_==0);
    e   = e + lambda * sum(g_(:).^2.*seg_(:));
end;

if nargout > 1
    
    % Compute derivative     
    
    % data term
    d = (aIt2(1:length(l)) - aIt2(length(l)+1:end) )'.*sigmoid(g(:),2); % .*l.*(1-l);
    
    % entropy term
    %[size(d) size(l) size(g) size(sigmoid(g,2))]
    d = d - log(l./(1-l)).*sigmoid(g(:),2);
    
    % spatial term

    for i = 1:length(F)
        g_  = conv2(g, F{i}, 'valid');

        seg_ = conv2(seg, F{i}, 'valid');
        seg_ = 1 - kappa*(seg_==0);
        
        Fi  = reshape(F{i}(end:-1:1), size(F{i}));                
        tmp = conv2(2*g_.*seg_, Fi, 'full');
        
        d   = d + lambda * tmp(:);
    end;
    
end;

function y = sigmoid(x, i)

% sigmoid function y = 1/(1+exp(-x))

if nargin == 1
    i = 1;
end;

lambda = 2;

switch i
    case 1
        y = 1./(1+exp(-lambda*x));
        
    case 2
        % derivative (sigmoid(x)*sigmoid(-x))
        y = lambda./(1+exp(-lambda*x))./(1+exp(lambda*x));
        %y = lambda./(1+exp(-lambda*x)).*(1- 1./(1+exp(-lambda*x)));
end;