function layerLabels = estimate_layer_labels_gc_occ_sym(this, layer_uv, prev_layerLabels, occ)
%%

% Estimate the expected layer labels by graph cut (hard segmentation)

% Occluded pixels are disabled for the data term

% lambda = 5; % weight on smoothness term for 0.1-1 Ising
%lambda = 4.5*25; % weight on smoothness term for 0-1   Ising
% lambda = 1e8* 4.5*25; % weight on smoothness term for intensity modulated 0-1 Ising
lambda = 3*4.5*25; % weight on smoothness term for segmentation modulated 0-1   Ising

nL = this.nLayers;
maxL = 1-1e-3;
minL = (1-maxL)/(1-nL);

% Compute BCE for each layer
for i = 1:nL   
   
    It = partial_deriv(this.images, layer_uv{i}, this.interpolation_method, this.deriv_filter);
%     It = partial_deriv(this.images, real(layer_uv{i}), this.interpolation_method, this.deriv_filter);
    
    % Simple version: no spatial interaction
    a = this.sigmaNs(i);
    % layerLabels(:,:,i) = exp(- It.^2/2/a^2) / (sqrt(2*pi)*a) * this.pis(i);
    
    % Disable data term for occluded pixels
    It  = It.*occ;
    
%     aIt2(:,:,i)         = It.^2/2/a^2;
    %aIt2(:,:,i)         = It.^2/2/a^2;
    %aIt2(:,:,i)         = abs(It)/a; % L1/Charbonnier
    
    this.rho_data.param = this.sigmaNs(i);
    aIt2(:,:,i)          = reshape(evaluate(this.rho_data, It(:)), size(It));

end;

% Give out-of-boundary pixels some penalty, otherwise some boundary regions
% will choose to move out of boundary for a zero data penalty  
aIt2(aIt2==0) = mean(aIt2(:))/5;

% Threshold large "outlier"
% maxIt2 = 100;
% aIt2(aIt2>maxIt2) = maxIt2;


%%% Solve hard segmentation by alpha expansion
% function seg = compute_seg_alpha_expansion(aIt2, )

% seg = [];
labelset    = 1:nL;
m   = size(aIt2, 1);
n   = size(aIt2, 2);

% Index for row and column, needed for evluating data term for a label field
%[indr indc] = meshgrid(1:n, 1:m);
[indr indc] = ind2sub([m n], reshape(1:m*n, [m n]));

prior_func = @(u) pairwise_potential_table(u, this.pTable);

% Define cliques
vert = im2col((reshape(1:m*n,m,n)),[2 1])';%vertical
hor  = im2col((reshape(1:m*n,m,n)),[1 2])';%horizontal

% Initialize energy
current_energy = 1e32;
success = 0;

% Initialize labeling
%fp = ones(m,n)*labelset(end);
[tmp fp] = max(prev_layerLabels, [], 3);

% Computer intensity difference
% sigma_i = 1;
% im1  = this.seg;
% dif1 = im1(vert(:,1)) - im1(vert(:,2));
% dif2 = im1(hor(:,1))  - im1(hor(:, 2));
% dif  = [dif1(:)' dif2(:)'];
% dif  = exp(-dif.^2/sigma_i^2/2);
% %dif  = dif /max(dif);  % normalization
% dif  = dif /mean(dif);  % normalization
% dif  = repmat(dif, [4 1]);

% Computer segmentation difference
im1  = this.seg;
dif1 = im1(vert(:,1)) ~= im1(vert(:,2));
dif2 = im1(hor(:,1))  ~= im1(hor(:, 2));
dif  = [dif1(:)' dif2(:)'];
% dif  = exp(-dif.^2);
dif  = exp(-dif); % within segment e^0 =1, across segment e^-1 
dif  = repmat(dif, [4 1]);


%%% start alpha_expansion
while true
    for loop=1 : size(labelset,2)
        alpha = labelset(loop);
        current_proposal = repmat(alpha,m,n);
        %in case of pairwise mrfs, we do not split the potentials to unary
        %and pairwise terms, we simply use the energytable
        %[E00,E01,E10,E11], which can be represented by single pairwise
        %term
        
        pt1 = layer_generateEnergyTable(vert, prior_func, current_proposal, fp);
        pt2 = layer_generateEnergyTable(hor,  prior_func, current_proposal, fp);

        pairwise_terms  = [pt1 pt2];          
%         pairwise_terms(3:end,:) = lambda*pairwise_terms(3:end,:); 
        
        % Modulated by intensity difference
        pairwise_terms(3:end,:) = lambda*pairwise_terms(3:end,:).*dif;
        
        %Seteup dataterm, these are only unary terms                
        indx = sub2ind([m n nL], indr, indc, fp);        
        Dp_fp       = aIt2(indx);                  
        indx = sub2ind([m n nL], indr, indc, current_proposal);
        Dp_alpha    = aIt2(indx);                 
        
        %Arrange dataterm to a 3xn matrix
        data_unary = [(1:m*n)' Dp_alpha(:) Dp_fp(:)]';        
      
        [new_fp,new_energy] = QPBO_all_in_one(m*n, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation

%             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
%                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
%         disp(strcat('alpha=',num2str(alpha),', energy=',num2str(current_energy)));
        
        new_fp = reshape(new_fp(1:m*n),m,n);%reshape new_fp to given image size
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
        
    end
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end

%%% End of alpha expansion

%Inpaint occluded regions again
fp(occ==0) = nL + 1;
fp = assign_occlusion_layer(this, fp);

% Convert hard segmentation to "soft" ownership
layerLabels = ones(size(aIt2)) * minL;

indx = sub2ind([m n nL], indr, indc, fp); 
layerLabels(indx) = maxL;

% For Debug
% tmp = aIt2(indx);
% [max(tmp(:)) mean(tmp(:)) min(tmp(:)) ]

% for i = 1:nL
%     tmp = layerLabels(:,:,i);
%     tmp(fp ==i) = maxL;
%     layerLabels(:,:,i) = tmp;
% end;
% 
% 