function g = estimate_layer_labels_multi_layers_sym_gd(this, layer_uv, prev_g, g2)

% Multi-layers

% Energy function BCE(uv1) sigmoid(lambda4*g) + BCE(uv2) sigmoid(-g) + lambda2smoothness(g)

% prior is modulated by static image segmentation

nGDiters = 20; % 50 little improvement

nL = this.nLayers;

s2 = g2labels(g2, this.lambda4);
for i = 1:nL-1       
    warps2(:,:,i) = warp_backward(layer_uv{i}, s2(:,:,i));
    warpg2(:,:,i) = warp_backward(layer_uv{i}, g2(:,:,i));
end;
i = nL;
warps2(:,:,i) = warp_backward(layer_uv{i}, s2(:,:,i));
warps2(isnan(warps2)) = 0;
warpg2(isnan(warpg2)) = prev_g(isnan(warpg2));  % or zero?

% Compute BCE for each layer
for i = 1:nL       
   
    It = partial_deriv(this.images, layer_uv{i}, this.interpolation_method, this.deriv_filter);   
    %It = partial_deriv(this.images, real(layer_uv{i}), this.interpolation_method, this.deriv_filter);
    
    %a = this.sigmaNs(i);
    %aIt2(:,:,i)         = It.^2/2/a^2;    
    %aIt2(:,:,i)         = It.^2/2;    
    
    this.rho_data.param = this.sigmaNs(i);
    aIt2(:,:,i)          = reshape(evaluate(this.rho_data, It(:)), size(It));
end;
aIt2 = aIt2.*warps2;



% Call minimizer
g = minimize(prev_g(:), @compute_g_con_energy, nGDiters, warpg2, aIt2, ...
    this.seg, this.g_filters, this.kappa, this.lambda2, this.lambda3, this.lambda4);

% g = minFunc();

g = reshape(g, size(prev_g));
% % for debug
% for i = 1:nL-1 
%     tmp = g(:,:,i);
%     fprintf('%3.3e\t %3.3e\t', min(tmp(:)), max(tmp(:)));    
% end;
% fprintf('\n');

% function [e d] = compute_g_con_energy(g, warpg2, aIt2, this)
% [e d] = compute_g_con_energy(this, g, warpg2, aIt2)

function [e d] = compute_g_con_energy(g, warpg2, aIt2, seg, F, kappa, lambda, lambda_g, lambda_s)

% Data + smoothness prior + temporal consistency term

% nL = 4;
% sz = [30 30];
% g  = randn([sz nL-1]); g= g(:);
% warpg2 = randn([sz nL-1]);
% aIt2 = rand([sz nL])*10;
% seg = round(rand(sz)*25);
% lambda = 1;
% kappa = 0.5;
% lambda_g = 1;  lambda_s = 1;
% d = checkgrad('compute_g_energy', g, 1e-3, aIt2, lambda, seg, kappa);

sz = size(aIt2);
nL = sz(3);
sz = sz(1:2);

g = reshape(g, [sz nL-1]);

% if nargin <4
%     seg = ones(size(g));        
% end;
% if nargin <5
%     kappa = 1;        
% end;
seg = double(seg);


% Compute energy

% data term
layerLabels = g2labels(g, lambda_s);
e = sum(aIt2(:).*layerLabels(:));

% spatial term
% F =  {[1 -1], [1 -1]'};

for j = 1:nL-1
    for i = 1:length(F)
        g_ = conv2(g(:,:,j), F{i}, 'valid');
        seg_ = conv2(seg, F{i}, 'valid');
        seg_ = 1 - kappa*(seg_==0);
        e   = e + lambda * sum(g_(:).^2.*seg_(:));
    end;
end;

% consistency term
e = e + lambda_g*sum((g(:)-warpg2(:)).^2);

if nargout > 1
    
    % Compute derivative     
    
    % data term
    d = zeros(size(g));

    for j = 1:nL-1
        d(:,:,j) = aIt2(:,:,j).*layerLabels(:,:,j)...
                   .*sigmoid(g(:,:,j),2,lambda_s)./sigmoid(g(:,:,j),1,lambda_s); % = lambda_4 sigmoid(-g(:,:,j),1)
        
        d(:,:,j) = d(:,:,j) - sum(aIt2(:,:,j+1:nL).*layerLabels(:,:,j+1:nL), 3)...
                                 .*sigmoid(g(:,:,j),2,lambda_s)./sigmoid(-g(:,:,j),1,lambda_s);  % = lambda_4 sigmoid(g(:,:,j),1)
    end;
    
    % spatial term
    for j = 1:nL-1
        for i = 1:length(F)
            g_  = conv2(g(:,:,j), F{i}, 'valid');
            
            seg_ = conv2(seg, F{i}, 'valid');
            seg_ = 1 - kappa*(seg_==0);
            
            Fi  = reshape(F{i}(end:-1:1), size(F{i}));
            tmp = conv2(2*g_.*seg_, Fi, 'full');
            
            d(:,:,j)   = d(:,:,j) + lambda * tmp;
        end;
    end;
    
    % consistency term
    d = d + 2*lambda_g*(g-warpg2);
    
    d = d(:);
end;