% function e = evaluate_energy(this, layer_uv, g, g2)
function [e eind] = evaluate_energy(this, g, layer_uv, layer_uvb, images, colorImages)
%%
% Data + smoothness prior + temporal consistency term

% for 2 frames symmetric case, solve g g2 together
% $ Date: 2010-7-26$


sigma_i = this.sigma_i;
Ti2     = this.Ti2; %20^2;
F       = this.g_filters;
kappa   = this.kappa;
lambda  = this.lambda2;
lambda_g = this.lambda3;
lambda_s = this.lambda4;
lambda_a = this.lambda;

sz = size(layer_uv{1}{1});
sz = sz(1:2);
nL = length(layer_uv{1});

T  = length(layer_uv)+1;

if ~iscell(colorImages)
    colorImages2 = cell(1,T);
    for t=1:T
        if size(colorImages,4)>1
            colorImages2{t} = colorImages(:,:,:,t);
        else
            colorImages2{t} = colorImages(:,:,t);
        end;
    end;
colorImages = colorImages2;
clear('colorImages2');    
end;


for t=1:T
    s{t} = g2labels(g{t}, lambda_s);
end;

% Compute mean affine flow field or projective flow field
%%
for t=1:T-1
    
    switch this.meanMode
        case 'projective'            
            layer_uv_affine{t}  = fit_projective_layer_motion_g(...
                                  layer_uv{t}, g{t});
            if this.is_symmetric
                layer_uvb_affine{t} = fit_projective_layer_motion_g(...
                    layer_uvb{t}, g{t+1});
            end;
            
        case 'affineL1'
            layer_uv_affine{t}  = fit_affine_layer_motion_g_L1(...
                                  layer_uv{t}, g{t});
            if this.is_symmetric
                layer_uvb_affine{t} = fit_affine_layer_motion_g_L1(...
                    layer_uvb{t}, g{t+1});
            end;
        otherwise          
%             layer_mean_affine_uv = fit_affine_motion_robust(...
%                 layer_uvb{t}, this.rho_affine);

            layer_uv_affine{t}  = fit_affine_layer_motion_g(...
                layer_uv{t}, g{t}, this.rho_affine);
            
            if this.is_symmetric
                layer_uvb_affine{t} = fit_affine_layer_motion_g(...
                    layer_uvb{t}, g{t+1}, this.rho_affine);
            end;
%             layer_uv_affine{t}  = fit_affine_motion_robust(...
%                                   layer_uv{t}, this.rho_affine);
%             if this.is_symmetric                              
%                 layer_uvb_affine{t} = fit_affine_motion_robust(...
%                     layer_uvb{t}, this.rho_affine);
%             end;
    end;    

end
%%
% compute data constancy error
for t=1:T-1
    for i=1:this.nLayers;
        
        % Compute BCE for each forward flow field
        It = partial_deriv(images(:,:,t:t+1), layer_uv{t}{i},...
             this.interpolation_method, this.deriv_filter);
        aIt2{t}(:,:,i) = reshape(evaluate(this.rho_data, It(:)),...
             size(It)) - this.lambda5;
        
        if this.is_symmetric       
            % Compute BCE for each backward flow field
            It = partial_deriv(images(:,:,t+1:-1:t), layer_uvb{t}{i},...
                this.interpolation_method, this.deriv_filter);
            aIt2_b{t}(:,:,i) = reshape(evaluate(this.rho_data, It(:)),...
                size(It)) - this.lambda5;
        end;

    end;
end;

for t=1:T-1
    
    % warp hidden field and layer using FORWARD flow field    
    for i = 1:nL-1
        warpg_f{t}(:,:,i) = imwarp_grad_im(layer_uv{t}{i}, g{t+1}(:,:,i));
    end;    
    warpg_f{t}(isnan(warpg_f{t})) = g{t}(isnan(warpg_f{t}));
    
    % warp toward g{t}, out-of-boundary replicate g{t}
    for i = 1:nL
        warps_f{t}(:,:,i) = imwarp_grad_im(layer_uv{t}{i}, s{t+1}(:,:,i));        
    end;
    warps_f{t}(isnan(warps_f{t})) = 0;    
    
    if this.is_symmetric    
        % warp hidden field and layer using BACKWARD flow field
        for i = 1:nL-1
            warpg_b{t}(:,:,i) = imwarp_grad_im(layer_uvb{t}{i}, g{t}(:,:,i));
        end;
        warpg_b{t}(isnan(warpg_b{t})) = g{t+1}(isnan(warpg_b{t}));
        
        for i = 1:nL
            warps_b{t}(:,:,i) = imwarp_grad_im(layer_uvb{t}{i}, s{t}(:,:,i));
        end;
        warps_b{t}(isnan(warps_b{t})) = 0;
    end;
end;


% Compute energy


e = 0; 
ed= e;
% data term
for t=1:T-1
    % forward flow field
    e = e+ sum(aIt2{t}(:).*s{t}(:).*warps_f{t}(:));
    
    if this.is_symmetric
        % backward flow field
        e = e+ sum(aIt2_b{t}(:).*s{t+1}(:).*warps_b{t}(:));
    end;
    
end;
ed = e-ed;

% fprintf('\n');
% return;

esg = e;
% spatial term for hidden field
% rho = robust_function('geman_mcclure', 12);
seg = colorImages;
for t=1:T
    for i = 1:length(F)
        
        if size(seg{t},3) ==1
            % weighte by segmentation difference
%             seg_ = conv2(seg{t}, F{i}, 'valid');
%             seg_ = 1 - kappa*(seg_==0);
            % Gray scale
            seg_ = conv2(seg{t}, F{i}, 'valid').^2;
            seg_ = seg_*3;  % normalize            
            seg_(seg_>Ti2) = Ti2;
            seg_ = exp(-seg_/2/sigma_i^2);
        else
            % weighted by lab color difference
            fsz = size(F{i});
            seg_ = zeros(sz(1)-fsz(1)+1, sz(2)-fsz(2)+1);
            for i1 = 1:size(seg{t},3)
                seg_ = seg_ + conv2(seg{t}(:,:,i1), F{i}, 'valid').^2;
            end;
            seg_ = seg_*size(seg{t},3)/3; % normalize
            
            seg_(seg_>Ti2) = Ti2;
            seg_ = exp(-seg_/2/sigma_i^2);

            % geman-mcclure 
            %seg_ = exp(-evaluate(rho, seg_));
            % asb
            %seg_ = exp(-abs(seg_)/12);
        end;        
        
        if nargout > 1
            % back up for gradient computation
            seg__{t}{i} = seg_;
        end;
        
        for j = 1:nL-1
            g_ = conv2(g{t}(:,:,j), F{i}, 'valid');
            e   = e + lambda * sum(g_(:).^2.*seg_(:));
            %fprintf('%3.3e\t', lambda * sum(g_(:).^2.*seg_(:)));
            %figure; imshow(g_.^2.*seg_, []);
        end;
    end;
end;
esg = e-esg;
% fprintf('\n');

ec = e;
% consistency term
for t=1:T-1
    % forward flow field
    e = e + lambda_g*sum((g{t}(:)-warpg_f{t}(:)).^2);
    
    if this.is_symmetric
        % backward flow field
        e = e + lambda_g*sum((g{t+1}(:)-warpg_b{t}(:)).^2);
    end;
end;
ec = e-ec;


% non-local term for flow field
nlm = 0;

if ~isempty(this.median_filter_size)
    
    for t=1:T-1
        for l=1:nL
            % forward flow field
            occ = detect_occlusion(layer_uv{t}{l}, images(:,:,t:t+1));            
            nlm = nlm+ evaluate_weighted_energy(layer_uv{t}{l},...
                colorImages{t},occ, this.area_hsz,...
                this.median_filter_size, this.sigma_mi);           
            
            if this.is_symmetric
                % backward flow field
                occ = detect_occlusion(layer_uvb{t}{l}, images(:,:,t+1:-1:t));
                nlm = nlm+ evaluate_weighted_energy(layer_uvb{t}{l},...
                    colorImages{t+1},occ, this.area_hsz,...
                    this.median_filter_size, this.sigma_mi);
            end;

        end;
    end;    

end;

nlm = nlm*this.lambdaNLM;

e = e+nlm;
%%
esf = e;
% spatial term for forward flow field
S = this.spatial_filters;
for t=1:T-1
    for l=1:nL
        for i = 1:length(S)            
            uv = layer_uv{t}{l}-layer_uv_affine{t}{l};                            
%             if ~isempty(this.mean_uv)
%                 uv = layer_uv{t}{l}-layer_uv_affine{t}{l};                
%                 % deviation from affine
%                 e = e + this.lambdaAff*sum(evaluate(this.rho_affine, uv(:)));                                
%             else
%                 uv = layer_uv{t}{l};
%             end;
            
            u_ = conv2(uv(:,:,1), S{i}, 'valid');
            v_ = conv2(uv(:,:,2), S{i}, 'valid');
                
            e  = e + lambda_a*sum(evaluate(this.rho_spatial_u{i}, u_(:)))...
                   + lambda_a*sum(evaluate(this.rho_spatial_v{i}, v_(:)));                
        end;       
    end;
end;


if this.is_symmetric
    % spatial term for backward flow field
    for t=1:T-1
        for l=1:nL
            for i = 1:length(S)
                uv = layer_uvb{t}{l}-layer_uvb_affine{t}{l};
%                 if ~isempty(this.mean_uv)
%                     uv = layer_uvb{t}{l}-layer_uvb_affine{t}{l};
%                     % deviation from affine
%                     e = e + this.lambdaAff*sum(evaluate(this.rho_affine, uv(:)));
%                 else
%                     uv = layer_uvb{t}{l};
%                 end;
                
                u_ = conv2(uv(:,:,1), S{i}, 'valid');
                v_ = conv2(uv(:,:,2), S{i}, 'valid');
                
                e  = e + lambda_a*sum(evaluate(this.rho_spatial_u{i}, u_(:)))...
                    + lambda_a*sum(evaluate(this.rho_spatial_v{i}, v_(:)));
            end;
        end;
    end;
end;

esf = e-esf;
%%
% % % variance term for hidden field
% % %e = e + this.lambda7*sum(g12(:).^2);
% % e = e + this.lambda7*sum( (g12(:)+this.Tg).^2);

%fprintf('data\t spa g\t con\t spa f\t total\n');
fprintf('%3.3e\t%3.3e\t%3.3e\t%3.3e\t%3.3e\t%3.3e\n', ed, esg, ec, esf, nlm, e);
if nargout >=2
    eind = [ed, esg, ec, esf, nlm];
end;
