function ouv = fuse_motion_given_seg(this, layer_id, z, z2, can_uv)
%%
% choose for every pixel in each layer, choose its motion vector among a candidate
% set of motion vectors (l1/l2)

% ouv(i,j, :) = can_uv{l(i,j)}(i,j, :)

% if nargin<6
%     lambdaL = 1; 
% end;
lambdaL = this.lambda;
lambdaD = this.lambda5;


[H W]= size(z);
validInd = find(z==layer_id); 
nvalid  = numel(validInd);

l1 = ones(H, W); %*(2*layer_id-1);

nC = size(can_uv,4);
%% This cell is same for all the layers
% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]);
for i = 1:nC   
    It          = partial_deriv(this.images, can_uv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;
aIt2 = aIt2-lambdaD;
% Set data term of occluded pixels to be zero
for i = 1:nC   
    occ = compute_occluded_pixels(can_uv(:,:,:,i), z, z2);
    aIt2(:,:,i) = aIt2(:,:,i).*(1-occ);
end;
%%
% cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

prior_funcL1V = @(u) pairwise_potential_tableL(u, can_uv, 0, this.rho_spatial_u{1});
prior_funcL1H = @(u) pairwise_potential_tableL(u, can_uv, 0, this.rho_spatial_u{1});

%data_unary = [1:H*W; zeros(2, H*W)]; 
fp = l1(:);
current_energy = 1e32;
success = 0;
% start alpha expansion
while true
    for il1 = 1:nC
        current_proposal = il1 * ones(size(fp));
        
        %pairwise terms of l1
        pl1v = layer_generateEnergyTable(vert, prior_funcL1V, current_proposal, fp);
        pl1h = layer_generateEnergyTable(hor,  prior_funcL1H, current_proposal, fp);
        pl1v(3:end, :) = lambdaL*pl1v(3:end, :);
        pl1h(3:end, :) = lambdaL*pl1h(3:end, :);
        
        pairwise_terms  = [pl1v pl1h];
        
        % Data term penalize deviation from affine flow field
        %                 ind_curr = sub2ind([4 nvalid+2*H*W],
        %                 current_proposal', 1:nvalid+2*H*W);
        %                 ind_fp   = sub2ind([4 nvalid+2*H*W], fp', 1:nvalid+2*H*W);
        %                 data_unary = [1:nvalid+2*H*W; dataD(ind_curr); dataD(ind_fp)];
        %
        %     fprintf('%3.3e\t%3.3e\n', max(dataD(ind_curr)), max(dataD(ind_fp))); % for debug
        
        % Data term : mathcing cost
        currInd = validInd + (current_proposal(validInd)-1)*H*W;
        fpInd   = validInd + (fp(validInd)-1)*H*W;
        
        data_unary = [validInd'; aIt2(currInd)'; aIt2(fpInd)'];
        
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
%         fprintf('%3.3f ', current_energy);        
        %Generate new labeling
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;
        
    end;
    
%     fprintf('\n');
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
fp = reshape(fp, [H W]);
fp2 = repmat(fp, [ 1 1 2]);
% ind = 1:2*H*W + (fp2(:)'-1)*2*H*W; % ? != below
ind = 1:2*H*W;
ind = ind + (fp2(:)'-1)*2*H*W;

ouv = reshape(can_uv(ind), [H W 2]);
%%
% figure; 
% subplot(3,3,1); imagesc(flowToColor(ouv));
% subplot(3,3,2); imagesc(flowToColor(can_uv(:,:,:,1)));
% subplot(3,3,3); imagesc(fp);
% z(ind) = fp(1:nvalid);
% l1     = fp(nvalid+1:nvalid+H*W);
% tind   = [1:H*W]' + (l1-1)*H*W*2;
% layer_uv{1}(:,:,1) = reshape(can_uv(tind), [H W]);
% layer_uv{1}(:,:,2) = reshape(can_uv(tind+H*W), [H W]);
% l2     = fp(nvalid+1+H*W:nvalid+H*W*2);
% tind   = [1:H*W]' + (l2-1)*H*W*2;
% layer_uv{2}(:,:,1) = reshape(can_uv(tind), [H W]);
% layer_uv{2}(:,:,2) = reshape(can_uv(tind+H*W), [H W]);

% figure; imagesc(reshape(l1, [H W]));
% figure; imagesc(reshape(l2, [H W]));
% Arrange solution into normal format

function occ = compute_occluded_pixels(uv, z, z2)

% given flow field and segmentaiton at two times
% occluded pixels are pixels whose segmentation change over time

% out-of-boundary pixles are treated as occluded too

H = size(uv,1);
W = size(uv,2);
[X Y] = meshgrid(1:W, 1:H);
x = round(X + uv(:,:,1));
y = round(Y + uv(:,:,2));
valid = (x>0) & (x<=W) & (y>0) & (y<=H);

x(~valid) = X(~valid);
y(~valid) = Y(~valid);  
ind2 = sub2ind([H W], y, x);
zw = z2(ind2);
occ = zw~=z;

% set out of boundary to be occluded
occ(~valid) = 1;
