function [oz] = fuse_seg_libDAI_sep_candidate_set(this, zs, layer_id, luvs, lambdaZ, lambdaT)


% Given the flow fields for each layer, decide which layer is visible at
% every pixel at frames t and t+1

% The data term will consider the occlusion relationship by checking the
% segementation consistency over time

% layer_id specifis the two layers to switch

if nargin<6
    lambdaZ = 1;    %
end;

if nargin<7
    lambdaT = 1;        % weight on the temporal term
end;

lambdaD     = this.lambda5;    %
lambdaD     = lambdaD*lambdaT;


%%
z0 = zs;
z0(zs==layer_id(1)) = 0;
z0(zs==layer_id(2)) = 0;
validZ = (zs==layer_id(1)) | (zs==layer_id(2));
z0(~validZ) = nan;

z   = z0(:,:,1);
[H W]= size(z);

% indV = find(validZ);
% indV1 = indV(indV<= H*W);
% indV2 = indV(indV> H*W);

luv  = luvs{1}(:,:,:,layer_id);
luv2 = luvs{2}(:,:,:,layer_id);

nC = size(luv, 4); 
nZ = nC;

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]); aIt22 = aIt2;
for i = 1:nC  
    % Data term at time 1
    It          = partial_deriv(this.images, luv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));    
    % Data term at time 2
    It          = partial_deriv(this.images(:,:,2:-1:1), luv2(:,:,:,i), this.interpolation_method, this.deriv_filter);
    aIt22(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;

aIt2 = aIt2*lambdaT;
aIt22 = aIt22*lambdaT;

% z1 z2 penalty
% 0  0     0 
% 0  1     0
% 1  0     It^2
% 1 1/oob  lambdaD
% aIt2 = aIt2-lambdaD; aIt22 = aIt22-lambdaD;
% need change construct_temporal_neighbor,
% construct_libDAI_structure_L_Z_temporal

% cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal


% % Temporal cliques for time t
% [cliqueT1 aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv, H, W, aIt2, validZ, lambdaD);
% [cliqueT2 aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2, H, W, aIt22, validZ(:,:,2:-1:1), lambdaD);
% % index of cliqueT2 should be flipped
% cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;

%
% compute weights for Ising model of z
wv1 = compute_color_weight(this.labImage{1}, vert, this.Ti2, this.sigma_i); wv1 = [reshape(wv1, [H-1 W]); ones(1, W)];
wh1 = compute_color_weight(this.labImage{1}, hor, this.Ti2, this.sigma_i);  wh1 = [reshape(wh1, [H W-1]) ones(H, 1)];
wv2 = compute_color_weight(this.labImage{2}, vert, this.Ti2, this.sigma_i); wv2 = [reshape(wv2, [H-1 W]); ones(1, W)];
wh2 = compute_color_weight(this.labImage{2}, hor, this.Ti2, this.sigma_i);  wh2 = [reshape(wh2, [H W-1]) ones(H, 1)];

%Construct strucutre for libDAI
%
isLogDomain = false;

%Spatial term for layer segmentation
indZ10 = 0;
psiZ1  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor, indZ10, wv1, wh1, isLogDomain);
indZ20 = H*W;
psiZ2  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor, indZ20, wv2, wh2, isLogDomain);
%
nZ1 = H*W; nZ2 = H*W;


% Temporal cliques for time t
[cliqueT1 tmp_aIt2  oob1 onInd1]= construct_temporal_neighbor_z1z2(can_uv,  H, W, aIt2, validZ, lambdaD);
psiZ1Z2 = construct_libDAI_structure_Z1_L_Z2_temporal(tmp_aIt2,  cliqueT1, indZ10, indZ20, indL110, isLogDomain, 1, lambdaD, oob1, onInd1); % 1 selects L11

[cliqueT2 tmp_aIt22 oob2 onInd2]= construct_temporal_neighbor_z1z2(can_uv2(:,:,:,layer_id(1)), H, W, aIt22(:,:,layer_id(1)), validZ(:,:,2:-1:1), lambdaD);
cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;
psiZ2Z1 = construct_libDAI_structure_Z2_L_Z1_temporal(tmp_aIt22, cliqueT2, indZ10, indZ20, indL210, isLogDomain, 1, lambdaD, oob2, onInd2);


psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2 ];
% psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2  psiL11Aff psiL12Aff psiL21Aff psiL22Aff];

% psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2 psiL11  psiL12  psiL21  psiL22 psiL11Aff psiL12Aff psiL21Aff psiL22Aff];
%%
% Call DAI solver
% [logZ,q,md,qv,qf, qmap] = dai(psi, 'BP', '[updates=SEQMAX,tol=1e-9,maxiter=10000,logdomain=0]');

% [qmap] = dai_bp_map(psi, 'BP', '[updates=SEQMAX,tol=1e-6,maxiter=1,logdomain=1]'); % SEQFIX, SEQRND, PARALL 
%%
[qmap] = dai_bp_map(psi, 'BP', '[updates=PARALL,tol=1e-6,maxiter=25,logdomain=0]');

%%
score = evaluate_BP_log_score(psi, qmap+1)

%%
% Interpret the output of DAI
% Update segmentation
tmp = reshape(qmap+1, [H W 6]);

oz = tmp(:,:,1:2);
tmp_oz = oz;
oz(tmp_oz==1) = layer_id(1);
oz(tmp_oz==2) = layer_id(2);
oz(~validZ) = zs(~validZ);
% Update flow of each layer
l11 = tmp(:,:,3);
l12 = tmp(:,:,4);
uv1 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(1) 4]), l11);
uv2 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(2) 4]), l12);
l21 = tmp(:,:,5);
l22 = tmp(:,:,6);
uvb1 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(1) 4]), l21);
uvb2 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(2) 4]), l22);

oluvs = luvs;
oluvs{1}(:,:,:,layer_id(1)) = uv1;
oluvs{1}(:,:,:,layer_id(2)) = uv2;
oluvs{2}(:,:,:,layer_id(1)) = uvb1;
oluvs{2}(:,:,:,layer_id(2)) = uvb2;
%  figure; imagesc(oz(:,:,1));
%%
% figure; imagesc([l11 l12]); % for debug
%%
%  figure; imagesc(flowToColor(uv2));
%%
%  figure; imagesc(oz(:,:,1));
%%
% figure; imagesc(validZ(:,:,1));
%%
function seg = warp_integer_seg(z, uv, H, W)
[X Y] = meshgrid(1:W, 1:H);
x = round(X+uv(:,:,1));
x(x>W) = W; x(x<1) = 1;
y = round(Y+uv(:,:,2));
y(y>H) = H; y(y<1) = 1;
ind = sub2ind([H W], y, x);

seg = reshape(z(ind), [H W]);


function weight = compute_color_weight(labImage, cliques, Ti2, sigma_i)

H = size(labImage,1);
W = size(labImage,2);

if size(labImage, 3) >1
    dif2 = zeros(size(cliques,1),1);
    for j=1:size(labImage, 3)
        tmp = labImage(cliques+(j-1)*H*W);
        dif2 = dif2 + (tmp(:,1)-tmp(:,2)).^2;
    end;
    dif2 = dif2*size(labImage, 3)/3;    
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
else
    tmp = labImage(cliques);
    dif2 = (tmp(:,1)-tmp(:,2)).^2 *3;
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
end;

function [cliqueT aIt2 oob onInd]= construct_temporal_neighbor(can_uv, H, W, aIt2, validZ, lambdaD)
% two corresponding neighbors are valid if they are both in the valid
% regions, as specified by validZ
% onInd(i) records the flow candidate that connect clique(i,1) and clique(i,2)
% oob records the selected pixels in image 1 that are projected out of boundary

[X Y] = meshgrid(1:W, 1:H);
cliqueT = [];
oob     = [];
ind1  = reshape(1:W*H, [H W]);

sel  = validZ(:,:,1);
nSel = sum(sel(:));
onInd = repmat(1:size(can_uv,4), [nSel 1]);
onInd = onInd(:);

for i=1:size(can_uv,4);
    x = round(X + can_uv(:,:,1,i));
    y = round(Y + can_uv(:,:,2,i));
    valid = (x>0) & (x<=W) & (y>0) & (y<=H);
    
    % record the selected pixels that are projected out of image boundary
    OB = (~valid)&validZ(:,:,1);
    oob = [oob; ind1(OB)];   
    
    
    x(~valid) = X(~valid);
    y(~valid) = Y(~valid);    
       
    ind2 = sub2ind([H W], y, x);
           
    wz2  = validZ(ind2+H*W);
    valid = valid(:)&wz2(:);        % is &wz2 necessary?
    %these valid pixels are projected to within the selected region in the
    %other image
    
    
    % Set data term of invalid flow candidate to be lambdaD
    tmp = aIt2(:,:,i);
    tmp(~valid) = lambdaD;
    aIt2(:,:,i) = tmp;
    
    % clique only contains the selected pixels in the first image    
    cliqueT = [cliqueT; [ind1(sel) ind2(sel)+H*W]];    
        
end;

