function [oluvs oz] = fuse_seg_motion_libDAI_sep_candidate_set(this, zs, can_uvs, layer_id, luvs, lambdaZ, lambdaT)

% choose from several flow candidates, which the pixel should take at time
% t and t+1

% The data term will consider the occlusion relationship by checking the
% segementation consistency over time

% layer_id specifis the two layers to switch

if nargin<6
    lambdaZ = 1;    %
end;

if nargin<7
    lambdaT = 1;        % weight on the temporal term
end;

lambdaAff   = 10;        % weight on deviation from the mean affine flow field

lambdaL     = this.lambda;
lambdaD     = this.lambda5;    %
lambdaD     = lambdaD*lambdaT;

lambdaFlip  = lambdaD*10;       % penalty for front layer to change to back layers at next frame 
%%
z0 = zs;
z0(zs==layer_id(1)) = 0;
z0(zs==layer_id(2)) = 0;
validZ = (zs==layer_id(1)) | (zs==layer_id(2));
z0(~validZ) = nan;

z   = z0(:,:,1);
[H W]= size(z);

% indV = find(validZ);
% indV1 = indV(indV<= H*W);
% indV2 = indV(indV> H*W);

can_uv  = can_uvs{1};
can_uv2 = can_uvs{2};

nC = size(can_uv, 4); 

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]); aIt22 = aIt2;
for i = 1:nC  
    % Data term at time 1
    It          = partial_deriv(this.images, can_uv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));    
    % Data term at time 2
    It          = partial_deriv(this.images(:,:,2:-1:1), can_uv2(:,:,:,i), this.interpolation_method, this.deriv_filter);
    aIt22(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;

aIt2 = aIt2*lambdaT;
aIt22 = aIt22*lambdaT;

% z1 z2 penalty
% 0  0     0 
% 0  1     0
% 1  0     It^2
% 1 1/oob  lambdaD
% aIt2 = aIt2-lambdaD; aIt22 = aIt22-lambdaD;
% need change construct_temporal_neighbor,
% construct_libDAI_structure_L_Z_temporal

% cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal


% % Temporal cliques for time t
% [cliqueT1 aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv, H, W, aIt2, validZ, lambdaD);
% [cliqueT2 aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2, H, W, aIt22, validZ(:,:,2:-1:1), lambdaD);
% % index of cliqueT2 should be flipped
% cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;

%
% compute weights for Ising model of z
wv1 = compute_color_weight(this.labImage{1}, vert, this.Ti2, this.sigma_i); wv1 = [reshape(wv1, [H-1 W]); ones(1, W)];
wh1 = compute_color_weight(this.labImage{1}, hor, this.Ti2, this.sigma_i);  wh1 = [reshape(wh1, [H W-1]) ones(H, 1)];
wv2 = compute_color_weight(this.labImage{2}, vert, this.Ti2, this.sigma_i); wv2 = [reshape(wv2, [H-1 W]); ones(1, W)];
wh2 = compute_color_weight(this.labImage{2}, hor, this.Ti2, this.sigma_i);  wh2 = [reshape(wh2, [H W-1]) ones(H, 1)];

%Construct strucutre for libDAI
%%
isLogDomain = false; %false;

%Spatial term for layer segmentation
indZ10 = 0;
psiZ1  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor, indZ10, wv1, wh1, isLogDomain);
indZ20 = H*W;
psiZ2  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor, indZ20, wv2, wh2, isLogDomain);
%
nZ1 = H*W; nZ2 = H*W;

compID = size(can_uv,4);     % ID for composite flow field

%Spatial term for flow field
indL110 = nZ1+nZ2;
flowPTables = compute_can_flow_spatial_potential(can_uv(:,:,:,[layer_id(1) 4]), this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
psiL11 = construct_libDAI_structure_L_spatial(flowPTables, indL110, vert, hor);

flowPTables = compute_can_flow_spatial_potential(can_uv(:,:,:,[layer_id(2) 4]), this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
indL120 = nZ1+nZ2+H*W;
psiL12 = construct_libDAI_structure_L_spatial(flowPTables, indL120, vert, hor);

flowPTables = compute_can_flow_spatial_potential(can_uv2(:,:,:,[layer_id(1) 4]), this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
indL210 = nZ1+nZ2+H*W*2;
psiL21 = construct_libDAI_structure_L_spatial(flowPTables, indL210, vert, hor);

flowPTables = compute_can_flow_spatial_potential(can_uv2(:,:,:,[layer_id(2) 4]), this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
indL220 = nZ1+nZ2+H*W*3;
psiL22 = construct_libDAI_structure_L_spatial(flowPTables, indL220, vert, hor);

% Affine deviation term for flow field
% psiL11Aff = construct_libDAI_structure_L_affine_dev(indL110, lambdaAff, can_uv(:,:,:,[layer_id(1) 4]), isLogDomain);
% psiL12Aff = construct_libDAI_structure_L_affine_dev(indL120, lambdaAff, can_uv(:,:,:,[layer_id(2) 4]), isLogDomain);
% psiL21Aff = construct_libDAI_structure_L_affine_dev(indL210, lambdaAff, can_uv2(:,:,:,[layer_id(1) 4]), isLogDomain);
% psiL22Aff = construct_libDAI_structure_L_affine_dev(indL220, lambdaAff, can_uv2(:,:,:,[layer_id(2) 4]), isLogDomain);

% 
%%
% Temporal cliques for time t
[cliqueT1 tmp_aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv(:,:,:,[layer_id(1) 4]),  H, W, aIt2(:,:,[layer_id(1) 4]), validZ, lambdaD);
psiZ1L11Z2 = construct_libDAI_structure_Z1_L_Z2_temporal(tmp_aIt2,  cliqueT1, indZ10, indZ20, indL110, isLogDomain, 1, lambdaD, oob1, onInd1); % 1 selects L11

[cliqueT1 tmp_aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv(:,:,:,[layer_id(2) 4]),  H, W, aIt2(:,:,[layer_id(2) 4]), validZ, lambdaD);
psiZ1L12Z2 = construct_libDAI_structure_Z1_L_Z2_temporal(tmp_aIt2,  cliqueT1, indZ10, indZ20, indL120, isLogDomain, 2, lambdaD, oob1, onInd1); % 2 selects L22

[cliqueT2 tmp_aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2(:,:,:,[layer_id(1) 4]), H, W, aIt22(:,:,[layer_id(1) 4]), validZ(:,:,2:-1:1), lambdaD);
cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;
psiZ1L21Z2 = construct_libDAI_structure_Z2_L_Z1_temporal(tmp_aIt22, cliqueT2, indZ10, indZ20, indL210, isLogDomain, 1, lambdaD, oob2, onInd2);

[cliqueT2 tmp_aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2(:,:,:,[layer_id(2) 4]), H, W, aIt22(:,:,[layer_id(2) 4]), validZ(:,:,2:-1:1), lambdaD);
cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;
psiZ1L22Z2 = construct_libDAI_structure_Z2_L_Z1_temporal(tmp_aIt22, cliqueT2, indZ10, indZ20, indL220, isLogDomain, 2, lambdaD, oob2, onInd2);
%
psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2 psiL11  psiL12  psiL21  psiL22];
% psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2  psiL11Aff psiL12Aff psiL21Aff psiL22Aff];

% psi = [psiZ1  psiZ2  psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2 psiL11  psiL12  psiL21  psiL22 psiL11Aff psiL12Aff psiL21Aff psiL22Aff];
%%
% Call DAI solver
% [logZ,q,md,qv,qf, qmap] = dai(psi, 'BP', '[updates=SEQMAX,tol=1e-9,maxiter=10000,logdomain=0]');

% [qmap] = dai_bp_map(psi, 'BP', '[updates=SEQMAX,tol=1e-6,maxiter=1,logdomain=1]'); % SEQFIX, SEQRND, PARALL 
%%
% [qmap] = dai_bp_map(psi, 'BP', '[updates=PARALL,tol=1e-6,maxiter=5,logdomain=0,inference=MAXPROD]');


%%
% tic
isLogDomain = true;
bpMethod = 'PARALL'; %'PARALL';
bpiter   = 50;
params   = sprintf('[updates=%s,tol=1e-9,maxiter=%d,logdomain=%d,inference=MAXPROD]', bpMethod, bpiter-1, isLogDomain);
%%
[qmap]   = dai_bp_map(psi, 'BP', params);
isLogDomain = false;
evaluate_BP_log_score(psi, qmap+1, isLogDomain)
% toc
%%
% bpMethod = 'PARALL'; %'SEQRND'; %;
% for bpiter = 1:10;
%     params   = sprintf('[updates=%s,tol=1e-6,maxiter=%d,logdomain=%d,inference=MAXPROD]', bpMethod, bpiter-1, isLogDomain);
%     [qmap] = dai_bp_map(psi, 'BP', params);
%     score(bpiter) = evaluate_BP_log_score(psi, qmap+1, isLogDomain);
% end;
% 
% figure; plot(score); title(bpMethod);


%%
% score = evaluate_BP_log_score(psi, qmap+1)

% Replace final with initial, compare log score
% tmp = reshape(qmap+1, [H W 6]);
% oz = tmp(:,:,1:2);
% tmp_oz = oz;
% oz(tmp_oz==1) = layer_id(1);
% oz(tmp_oz==2) = layer_id(2);
% tmp2 = tmp;
% oz(zs==layer_id(1))=1;
% oz(zs==layer_id(2))=2;
% tmp2(:,:,1:2) = oz;
% evaluate_BP_log_score(psi, tmp2(:), isLogDomain)

%%
% q_init = round(rand(H, W, 6));
% tmp    = q_init(:,:,1:2);
% % q_init = zeros(H, W, 6);
% % tmp    =round(rand(H, W, 2));
% tmp(validZ) = zs(validZ);
% tmp_z = tmp;
% tmp(tmp_z==layer_id(1)&validZ) = 0;
% tmp(tmp_z==layer_id(2)&validZ) = 1;
% q_init(:,:,1:2) = tmp;
% evaluate_BP_log_score(psi, q_init(:)+1)
% %%
% bpiter   = 20; isLogDomain = true; bpMethod = 'PARALL'; %'PARALL';
% params   = sprintf('[updates=%s,tol=1e-9,maxiter=%d,logdomain=%d,inference=MAXPROD]', bpMethod, bpiter-1, isLogDomain);
% [qmap3]   = dai_bp_map_with_init(psi, 'BP', params, q_init(:), (0:H*W*6-1)');
% evaluate_BP_log_score(psi, qmap3+1)
%%
% ind = reshape(1:H*W*2, [H W 2]);
% sel_vars = ind(validZ)-1;
% q_init   = zs(validZ);
% tmp = q_init;
% q_init(tmp==layer_id(1)) =0;
% q_init(tmp==layer_id(2)) =1;
% %%
% % %% from previous finished state
% % ind = reshape(1:H*W*2, [H W 2]);
% % sel_vars = ind(validZ);
% % q_init   = qmap300(sel_vars);
% % sel_vars = sel_vars -1;
% % %%
% % bpiter   = 200;
% % params   = sprintf('[updates=%s,tol=1e-9,maxiter=%d,logdomain=%d,inference=MAXPROD]', bpMethod, bpiter-1, isLogDomain);
% [qmap2]   = dai_bp_map_with_init(psi, 'BP', params, double(q_init(:)), sel_vars(:));
% evaluate_BP_log_score(psi, qmap2+1)
% % 
% % tmp = reshape(qmap2+1, [H W 6]);figure; imagesc(tmp(:,:,1));

%% observe log score vs iterations

% bpiters = [1 5 10 30 50]; % 100 200 500];
% % bpiters = [1 5];
% % bpscore = zeros(size(bpiters));
% 
% for i =1:length(bpiters);
%     bpiter = bpiters(i);
%     params   = sprintf('[updates=%s,tol=1e-6,maxiter=%d,logdomain=%d,inference=MAXPROD]', bpMethod, bpiter, isLogDomain);    
% %     [qmap] = dai_bp_map_with_init(psi, 'BP', params);
%     [qmap] = dai_bp_map_with_init(psi, 'BP', params, q_init(:), sel_vars); disp('use init');
%     bpscore(i) = evaluate_BP_log_score(psi, qmap+1);
%     tmp = reshape(qmap+1, [H W 6]);
%     figure; imagesc(tmp(:,:,1));    title(sprintf('log score %3.3e', bpscore(i)));
% end;
% 
% figure; semilogx(bpiters, bpscore); title(bpMethod);


% [qmap2]   = dai_bp_map_with_init(psi, 'BP', params, q_init(:));
% evaluate_BP_log_score(psi, qmap2+1, isLogDomain)
%%
% Interpret the output of DAI
% Update segmentation
tmp = reshape(qmap+1, [H W 6]);

oz = tmp(:,:,1:2);
tmp_oz = oz;
oz(tmp_oz==1) = layer_id(1);
oz(tmp_oz==2) = layer_id(2);
oz(~validZ) = zs(~validZ);
% figure; imagesc(oz(:,:,1));
%%
% Update flow of each layer
l11 = tmp(:,:,3);
l12 = tmp(:,:,4);
uv1 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(1) 4]), l11);
uv2 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(2) 4]), l12);
l21 = tmp(:,:,5);
l22 = tmp(:,:,6);
uvb1 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(1) 4]), l21);
uvb2 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(2) 4]), l22);

oluvs = luvs;
oluvs{1}(:,:,:,layer_id(1)) = uv1;
oluvs{1}(:,:,:,layer_id(2)) = uv2;
oluvs{2}(:,:,:,layer_id(1)) = uvb1;
oluvs{2}(:,:,:,layer_id(2)) = uvb2;

%%
% figure; imagesc([l11 l12]); % for debug
%%
% figure; imagesc(flowToColor(uv1));
%%
% figure; imagesc(oz(:,:,1));
%%
% figure; imagesc(tmp(:,:,1));
%%
% figure; imagesc(validZ(:,:,1));
%%
function seg = warp_integer_seg(z, uv, H, W)
[X Y] = meshgrid(1:W, 1:H);
x = round(X+uv(:,:,1));
x(x>W) = W; x(x<1) = 1;
y = round(Y+uv(:,:,2));
y(y>H) = H; y(y<1) = 1;
ind = sub2ind([H W], y, x);

seg = reshape(z(ind), [H W]);


function weight = compute_color_weight(labImage, cliques, Ti2, sigma_i)

H = size(labImage,1);
W = size(labImage,2);

if size(labImage, 3) >1
    dif2 = zeros(size(cliques,1),1);
    for j=1:size(labImage, 3)
        tmp = labImage(cliques+(j-1)*H*W);
        dif2 = dif2 + (tmp(:,1)-tmp(:,2)).^2;
    end;
    dif2 = dif2*size(labImage, 3)/3;    
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
else
    tmp = labImage(cliques);
    dif2 = (tmp(:,1)-tmp(:,2)).^2 *3;
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
end;

function [cliqueT aIt2 oob onInd]= construct_temporal_neighbor(can_uv, H, W, aIt2, validZ, lambdaD)
% two corresponding neighbors are valid if they are both in the valid
% regions, as specified by validZ
% onInd(i) records the flow candidate that connect clique(i,1) and clique(i,2)
% oob records the selected pixels in image 1 that are projected out of boundary

[X Y] = meshgrid(1:W, 1:H);
cliqueT = [];
oob     = [];
ind1  = reshape(1:W*H, [H W]);

sel  = validZ(:,:,1);
nSel = sum(sel(:));
onInd = repmat(1:size(can_uv,4), [nSel 1]);
onInd = onInd(:);

for i=1:size(can_uv,4);
    x = round(X + can_uv(:,:,1,i));
    y = round(Y + can_uv(:,:,2,i));
    valid = (x>0) & (x<=W) & (y>0) & (y<=H);
    
    % record the selected pixels that are projected out of image boundary
    OB = (~valid)&validZ(:,:,1);
    oob = [oob; ind1(OB)];   
    
    
    x(~valid) = X(~valid);
    y(~valid) = Y(~valid);    
       
    ind2 = sub2ind([H W], y, x);
           
    wz2  = validZ(ind2+H*W);
    valid = valid(:)&wz2(:);        % is &wz2 necessary?
    %these valid pixels are projected to within the selected region in the
    %other image
    
    
    % Set data term of invalid flow candidate to be lambdaD
    tmp = aIt2(:,:,i);
    tmp(~valid) = lambdaD;
    aIt2(:,:,i) = tmp;
    
    % clique only contains the selected pixels in the first image    
    cliqueT = [cliqueT; [ind1(sel) ind2(sel)+H*W]];    
        
end;

