function [oluvs oz] = fuse_seg_motion_libDAI_sep_candidate_set_complete_clique(this, zs, can_uvs, layer_id, luvs, lambdaZ, lambdaT)

% choose from several flow candidates, which the pixel should take at time
% t and t+1

% The data term will consider the occlusion relationship by checking the
% segementation consistency over time

% layer_id specifis the two layers to switch

if nargin<6
    lambdaZ = 1;    %
end;

if nargin<7
    lambdaT = 1;        % weight on the temporal term
end;

lambdaL     = this.lambda;
lambdaD     = this.lambda5;    %
lambdaD     = lambdaD*lambdaT;
lambdaFB    = lambdaD*10;       % penalty for front layer to change to back layers at next frame 

%%
[H W]   = size(zs(:,:,1));
can_uv  = can_uvs{1};
can_uv2 = can_uvs{2};
nC      = size(can_uv, 4); 

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]); aIt22 = aIt2;
for i = 1:nC  
    % Data term at time 1
    It          = partial_deriv(this.images, can_uv(:,:,:,i),...
                    this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));    
    % Data term at time 2
    It          = partial_deriv(this.images(:,:,2:-1:1), can_uv2(:,:,:,i),...
                    this.interpolation_method, this.deriv_filter);
    aIt22(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;

aIt2  = aIt2*lambdaT;
aIt22 = aIt22*lambdaT;

% spatial cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal

% compute weights for Ising model of z
wv1 = compute_color_weight(this.labImage{1}, vert, this.Ti2, this.sigma_i); 
wv1 = [reshape(wv1, [H-1 W]); ones(1, W)];
wh1 = compute_color_weight(this.labImage{1}, hor, this.Ti2, this.sigma_i);  
wh1 = [reshape(wh1, [H W-1]) ones(H, 1)];
wv2 = compute_color_weight(this.labImage{2}, vert, this.Ti2, this.sigma_i); 
wv2 = [reshape(wv2, [H-1 W]); ones(1, W)];
wh2 = compute_color_weight(this.labImage{2}, hor, this.Ti2, this.sigma_i);  
wh2 = [reshape(wh2, [H W-1]) ones(H, 1)];


% % Temporal cliques for time t
% [cliqueT1 aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv, H, W, aIt2, validZ, lambdaD);
% [cliqueT2 aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2, H, W, aIt22, validZ(:,:,2:-1:1), lambdaD);
% % index of cliqueT2 should be flipped
% cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;

%Construct strucutre for libDAI
%%
isLogDomain = false; 

%Spatial term for layer segmentation
indZ10 = 0;
psiZ1  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor,...
            indZ10, wv1, wh1, isLogDomain);
indZ20 = H*W;
psiZ2  = construct_libDAI_structure_Z_spatial_all(lambdaZ, vert, hor, ...
            indZ20, wv2, wh2, isLogDomain);
nZ1 = H*W; nZ2 = H*W;

%Spatial term for flow field
indL110 = nZ1+nZ2;
flowPTables = compute_can_flow_spatial_potential(can_uv, ...
            this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
psiL11 = construct_libDAI_structure_L_spatial(flowPTables, indL110, vert, hor);

indL120 = nZ1+nZ2+H*W;
psiL12 = construct_libDAI_structure_L_spatial(flowPTables, indL120, vert, hor);

flowPTables = compute_can_flow_spatial_potential(can_uv2,...
            this.rho_spatial_u{1}, vert, hor, lambdaL, isLogDomain);
indL210 = nZ1+nZ2+H*W*2;
psiL21 = construct_libDAI_structure_L_spatial(flowPTables, indL210, vert, hor);

indL220 = nZ1+nZ2+H*W*3;
psiL22 = construct_libDAI_structure_L_spatial(flowPTables, indL220, vert, hor);

%%
% Temporal cliques for time t
[cliqueT1 tmp_aIt2  oob1 onInd1] = construct_temporal_neighbor_forward(...
    can_uv,  H, W, aIt2, zs, layer_id, lambdaD, lambdaFB);
psiZ1Z2 = construct_libDAI_structure_Z1_Z2_temporal(tmp_aIt2,  cliqueT1,...
    indZ10, indZ20, indL110, isLogDomain, 1, lambdaD, oob1, onInd1); % 1 selects L11

% [cliqueT1 tmp_aIt2  oob1 onInd1]= construct_temporal_neighbor(can_uv(:,:,:,[layer_id(2) 4]),  H, W, aIt2(:,:,[layer_id(2) 4]), validZ, lambdaD);
% psiZ1L12Z2 = construct_libDAI_structure_Z1_L_Z2_temporal(tmp_aIt2,  cliqueT1, indZ10, indZ20, indL120, isLogDomain, 2, lambdaD, oob1, onInd1); % 2 selects L22

% [cliqueT2 tmp_aIt22 oob2 onInd2]= construct_temporal_neighbor_backward(...
%     can_uv2(:,:,:,[layer_id(1) 4]), H, W, aIt22(:,:,[layer_id(1) 4]), validZ(:,:,2:-1:1), lambdaD);
% cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;
% psiZ1L21Z2 = construct_libDAI_structure_Z2_L_Z1_temporal(tmp_aIt22, ...
%     cliqueT2, indZ10, indZ20, indL210, isLogDomain, 1, lambdaD, oob2, onInd2);
% 
% [cliqueT2 tmp_aIt22 oob2 onInd2]= construct_temporal_neighbor(can_uv2(:,:,:,...
%     [layer_id(2) 4]), H, W, aIt22(:,:,[layer_id(2) 4]), validZ(:,:,2:-1:1), lambdaD);
% cliqueT2(:,1) = cliqueT2(:,1)+H*W;cliqueT2(:,2) = cliqueT2(:,2)-H*W;
% psiZ1L22Z2 = construct_libDAI_structure_Z2_L_Z1_temporal(tmp_aIt22,...
%     cliqueT2, indZ10, indZ20, indL220, isLogDomain, 2, lambdaD, oob2, onInd2);
%
% psi = [psiZ1  psiZ2  psiL11  psiL12  psiL21  psiL22 psiZ1L11Z2  psiZ1L12Z2  psiZ1L21Z2  psiZ1L22Z2 ];

psi = [psiZ1  psiZ2  psiL11  psiL12  psiL21  psiL22 psiZ1Z2  psiZ2Z1];
%%
% tic
isLogDomain = true;
bpMethod = 'PARALL'; %'PARALL';
bpiter   = 5;   % 5e3 or 5e4
params   = sprintf('[updates=%s,tol=1e-9,maxiter=%d,logdomain=%d,inference=MAXPROD]',...
                bpMethod, bpiter-1, isLogDomain);
% damping=, 
            
[qmap]   = dai_bp_map(psi, 'BP', params);
isLogDomain = false;
evaluate_BP_log_score(psi, qmap+1, isLogDomain)

% Interpret the output of DAI
% Update segmentation
tmp = reshape(qmap+1, [H W 6]);

oz = tmp(:,:,1:2);
tmp_oz = oz;
oz(tmp_oz==1) = layer_id(1);
oz(tmp_oz==2) = layer_id(2);
oz(~validZ) = zs(~validZ);
% figure; imagesc(oz(:,:,1));
%%
% Update flow of each layer
l11 = tmp(:,:,3);
l12 = tmp(:,:,4);
uv1 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(1) 4]), l11);
uv2 = composite_final_flow_seg(can_uv(:,:,:,[layer_id(2) 4]), l12);
l21 = tmp(:,:,5);
l22 = tmp(:,:,6);
uvb1 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(1) 4]), l21);
uvb2 = composite_final_flow_seg(can_uv2(:,:,:,[layer_id(2) 4]), l22);

oluvs = luvs;
oluvs{1}(:,:,:,layer_id(1)) = uv1;
oluvs{1}(:,:,:,layer_id(2)) = uv2;
oluvs{2}(:,:,:,layer_id(1)) = uvb1;
oluvs{2}(:,:,:,layer_id(2)) = uvb2;

%%
% figure; imagesc([l11 l12]); % for debug
%%
% figure; imagesc(flowToColor(uv1));
%%
% figure; imagesc(oz(:,:,1));
%%
% figure; imagesc(tmp(:,:,1));
%%
% figure; imagesc(validZ(:,:,1));
%%
function seg = warp_integer_seg(z, uv, H, W)
[X Y] = meshgrid(1:W, 1:H);
x = round(X+uv(:,:,1));
x(x>W) = W; x(x<1) = 1;
y = round(Y+uv(:,:,2));
y(y>H) = H; y(y<1) = 1;
ind = sub2ind([H W], y, x);

seg = reshape(z(ind), [H W]);




