function this = layer_optical_flow_3D(varargin)
%%
%LAYER_OPTICAL_FLOW_3D   Optical flow computation with


%   LAYER_OPTICAL_FLOW_3D([IMGS]) constructs a LAYER_OPTICAL_FLOW_3D optical flow object
%   with the optional image sequence IMGS ([n x m x 2] array). 
%   LAYER_OPTICAL_FLOW_3D(O) constructs LAYER_OPTICAL_FLOW_3D optical flow object by copying O.
%  
%   This is a member function of the class 'layer_optical_flow_3D'. 

%   Author: Deqing Sun, Department of Computer Science, Brown University
%   Contact: dqsun@cs.brown.edu
%   $Date: 2007-11-30 $

% Copyright 2007-2008, Brown University, Providence, RI.
%
%                         All Rights Reserved
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of the author and Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
%
% THE AUTHOR AND BROWN UNIVERSITY DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR OR BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  

error(nargchk(0, 1, length(varargin)));
  
  switch (length(varargin))
    case 0

      this.dataset_name = [];
      this.prior           = [];
      this.isstuff         = [];
      this.class_mask      = [];	        
      this.images          = [];              
      this.lambda          = 3;
      this.lambda_q        = 3;    % Quadratic formulation of the objective function
      
      this.sor_max_iters   = 1e4;       % 100 seems sufficient

      this.limit_update    = true;      % limit the flow incrment to be less than 1 per linearization step
      this.display         = false;      
      
      
      this.solver          = 'backslash';   % 'sor' 'pcg' for machines with limited moemory 
      this.warping_mode    = 'backward';
      this.deriv_filter         = [1 -8 0 8 -1]/12; % 5-point 7 point [-1 9 -45 0 45 -9 1]/60; 
      
      this.texture             = false;     % use texture component as input
      this.fc                  = false;     % use filter constancy

      this.median_filter_size   = []; %[5 5];
      this.denoise_robust       = false; % can't be used simulatneous with MF
      this.lambda_dr            = 1; % can't be used simulatneous with MF
      this.denoise_method       = 'charbonnier';
      this.denoise_niters       = 50;
      
      this.affine               = false;
      
      this.interpolation_method = 'bi-cubic';  % 'bi-cubic', 'cubic', 'bi-linear'
    
      % For Graduated Non-Convexity (GNC) optimization
      this.gnc_iters       = 1;
      this.alpha           = 0;             % change linearly from 1 to 0 through the GNC stages

      this.max_iters       = 3;            % number of warping per pyramid level
      this.max_linear      = 1;             % maximum number of linearization performed per warping, 1 OK for HS
      
      % For GNC stage 1
      this.pyramid_levels  = 4;           
      this.pyramid_spacing = 2;

      % For GNC stage 2 to end
      this.gnc_pyramid_levels     = 2;
      this.gnc_pyramid_spacing    = 1.25;                
      

      this.spatial_filters = {[1 -1], [1; -1]};  
      this.g_filters       = {[1 -1], [1; -1]};  %, [1 0; 0 -1], [0 1; -1 0]
      %this.g_filters       = {[1 -1], [1; -1], [1 0; 0 -1], [0 1; -1 0]};  %
      %this.spatial_filters = {[1 -2 1], [1; -2; 1]};  % 2nd-order smoothnes prior
    
      method = 'generalized_charbonnier'; %'lorentzian'
      a   = 0.45;
      sig = 1e-3;
      for i = 1:length(this.spatial_filters);
          this.rho_spatial_u{i}   = robust_function(method, sig, a);
          this.rho_spatial_v{i}   = robust_function(method, sig, a);
      end;
      a   = 0.45;
      this.rho_data        = robust_function(method, sig, a);
      this.rho_affine      = robust_function(method, sig, a);
      this.lambdaAff       = 1;
      this.lambdaAffCont   = 0.01;      % for continuous optimization
      
      this.seg             = [];    % sore segementatio result
      this.mfT             = 15;    % threshold for intensity-median-filter
      this.imfsz           = [7 7]; % for intensity-median-filter
      this.filter_weight   = [];    % only for the new obj. charbonnier
      this.alp             = 0.95;  % for rof texture decomposition
      
      this.hybrid          = false;  
      this.area_hsz        = 10;    % half window size for the weighted area term
      this.affine_hsz      = 4;     % half window size for robust affine 
      
      this.nLayers         = 3;
      %[this.nLayers]
      this.layerLabel      = [];
      this.sigmaNs         = ones(this.nLayers, 1);    % sigma for each layer distributino
      this.pis             = ones(1,this.nLayers)/this.nLayers;    % prior distribution on layers
      %this.pTable          = ones(this.nLayers) - 0.9*eye(this.nLayers);    % potential table
      this.pTable          = ones(this.nLayers) - eye(this.nLayers);    % potential table
      
      this.lambda_sym      = 1e3;       % symmetric term weight
      %[this.lambda_sym]
      this.lambda_lab      = 350; %3*4.5*25;  % label smoothness term weight
      this.lambda_occ      = this.lambda_sym;       % occlusion smoothness term weight
      this.Tocc            = this.lambda_sym;
      this.colorImages     = [];
      
      this.lambda2         = 5;     % weight on smoothness term for hidden fields
      this.lambda3         = 2;     % weight on consistency term for hidden g field
      this.lambda4         = 2;     % for the sigmoid function
      this.lambda5         = 2;     % for data penalty term
      this.kappa           = 0.5;   % 1-kappa is penalty for edge in g to occur at static segment image edges
      %[this.lambda2]
      this.lambda4s        = [];    % for annealing
      this.lambda2s        = [];    % for annealing
      this.lambda6         = 100;%1e-1;  % for mean affine motion field, not used?
      this.lambda7         = 0;     % for variance term of hidden field g
      this.lambdaNLM       = 1;     % NLM term for flow field, for evaluating energy
      this.mean_uv         = []; 
      
      this.labImage        = [];    % lab image of the first color image
      
      this.nGDiters        = 10;    % # cgd iterations for g
      
      this.area_hsz        = 2;    % half window size for the weighted area term
      this.sigma_mi        = 7;     % for non-local flow prior term
      this.area_hsz_final  = 7;     % for composite flow field
      this.sigma_i         = 7;    % for g smoothness term
      this.Ti2             = 20^2;

      this.nCandidate      = 25;    % number of runs to obtain ini segmentation
      this.layer_s         =[];
      
      this.depth_order     = 'f2s'; %'lower';
%       [this.depth_order]
      this.affine_mean     = true;
      this.affine_ini_all  = false;  % true: use affine flow field as initialization
      this.Tg              = 0;     % threshold for the logistic/sigmoid function
      
      %this.auto_level       = true;      
      this.fullVersion      = false;
      this.meanMode         = 'affine'; % projective;
      this.mf_mode          = 'wmf'; % 'mf' otherwise no
      
      this.fileinfo        = [];      
      this.visG            = 1.5/32;
      this.ratioG          = 1;    % initial g for inocnsistent/occ reduce by ratioG
      this.anneal_same_s   = true; % keep s fixed during annealing
      this.comb_hard       = true;  % hard/soft combining layer flow field
      this.init_mode       = 'c+nl'; % c++
      this.gc_init         = true;  % true: use graph cuts to cluster
      
      this.is_symmetric     = true;  % true: bi-direction flow field, false:uni-
      this.disDownRatio    = 2;
      this.extParams       = [];    % external parameters from input
      this.intParams       = [];    % internal parameters from load_of_method     
      
      this.labChoice       = 'correct'; % for debug
      
      this.cont_ref        = true;  % use continuous refinement
      this.discParams      = [];
      this.is_hand_seg     = false;
      this.MRFOnFlow       = false;
      this.is_save_mat     = true; 
      this.is_post_sup     = false;
      %this.addParams       = [];
      this.is_use_disc     = true;
      this.discSolver      = 'GC'; %MF

      this = class(this, 'layer_optical_flow_3D');         
      
    case 1
      if isa(varargin{1}, 'layer_optical_flow_3D')
        this = varargin{1};        
      else    
          this = layer_optical_flow_3D;
          this.images = varargin{1};  
      end
      
    otherwise
      error('Incompatible arguments!');
      
  end
  
