function layer_uv = fit_affine_layer_motion_g_L1(layer_uv, layer_g)
% Fit affine motion to the visible pixels of each layer
% The objective is L1 and the solution can be computed as the median
% for other penalty function, energy minimization should be used.
% L1 is very close to the generalized charbonnier penalty (a =0.45)

% \sum_{j\in N_i} | [u_(i)-u_aff(i)]-[u_(j)-u_aff(j)]|
% this objective has no constraint on the translational components and we
% compute them by the median of the flow vectors

nL = length(layer_uv);
sz = size(layer_uv{1}); sz = sz(1:2);

[X Y] = meshgrid(1:sz(2), 1:sz(1));

% Obtain hard segmentation by thresholding the hidden fields
ind = threshold_g(layer_g);

for i = 1:nL
    
    uv = layer_uv{i};
    u = uv(:,:,1); 
    v = uv(:,:,2); 

    % fit affine motion to visible pixels
    % horizontal flow
    tmp_u = u(ind==i);    
    A(3,1) = median(tmp_u(:));    
    
    ux = u(2:end,:) - u(1:end-1,:);
    tmp_ux = ux(ind(1:end-1,:)==i);
    A(1,1) = median(tmp_ux(:));    
    uy = u(:,2:end) - u(:,1:end-1);    
    tmp_uy = uy(ind(:,1:end-1)==i);
    A(2,1) = median(tmp_uy(:));
    
    % vertical flow
    tmp_v  = v(ind==i);
    A(3,2) = median(tmp_v(:));

    vx = v(2:end,:) - v(1:end-1,:);
    tmp_vx = vx(ind(1:end-1,:)==i);
    A(1,2) = median(tmp_vx(:));    
    vy = v(:,2:end) - v(:,1:end-1);    
    tmp_vy = vy(ind(:,1:end-1)==i);
    A(2,2) = median(tmp_vy(:));

    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    layer_uv{i} = cat(3, u, v);
end;


