function layer_uv = fit_affine_motion_robust(layer_uv, rho)
%% Fit affine motion to the motion of each layer

nL = length(layer_uv);
sz = size(layer_uv{1}); sz = sz(1:2);

nData = prod(sz);
[X Y] = meshgrid(1:sz(2), 1:sz(1));
x   = X(:); 
y   = Y(:);
xy1 = [x y ones(nData, 1)];

for i = 1:nL
    
    uv = layer_uv{i};
    u = uv(:,:,1); u = u(:);
    v = uv(:,:,2); v = v(:);    
   
    A(:,1) = RLS_fit(u, xy1, rho);
    A(:,2) = RLS_fit(v, xy1, rho);
    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    layer_uv{i} = cat(3, u, v);
end;


