function params = load_parse_parameters(model)
%%

% default parameter setting

params.cont_refine       = true; 
params.cont_refine_iters = 3;
params.softDiscVis       = 0.999;


% according to continuous model
params.T2_lab           = 25^2;  %40
params.lambdaOcc        = 8;
params.sigma_lab        = 12;
params.lambdaFlow       = 3;
params.inputDownRatio   = 2;

% according to data
params.T                = 2;  

% according to input parameters
params.lambdaSeg        = 7.5e1;
params.lambdaIncon      = 3; % temporal inconsisntecy between support functions
params.lambdaAff        = 10;

params.lambdaForbid     = 8; % temporarily
params.alt_niters       = 3;
params.nIgnore          = 100; % layers with less visible pixels than this number ignored

params.deriv_filter         = [1 -8 0 8 -1]/12;
params.interpolation_method = 'bi-cubic';

a       = 0.45;
sig     = 1e-3;
method  = 'generalized_charbonnier'; %'lorentzian'


params.rho_data     = robust_function(method, sig, a);
params.rho_color    = robust_function('quadratic', 1);   % weight for support function/segmentation
params.rho_affine   = robust_function(method, sig, a);
params.rho_spatial  = robust_function(method, sig, a);


params.t            = 4;
params.seqName      = 'bird_apple'; 


params.iBest        = 1;   % selects the one with ith lowest fitting error
params.nLayers      = 3;
params.homeDir      = '/data/vision/dqsun/program/iccv11_flow/';
params.nGCiters     = 2;
params.nClusterRun  = 15;
params.nCan         = params.nLayers+1;

params.nSwapIters   = 12;     % max # iters for swapping depth order
params.nSwapT       = 5;     % regions with neighoring pixels larger than this are adjacent
params.nMaxSwapPairs= 10;  % maximum # of pairs to compare

params.display      = false;
params.fdir         = '/data/vision/dqsun/program/iccv11_flow/result/output/disc_temp/';

params.downOption   = []; %'hires_0.5'; % subsample on high-res grid 1/2 size
params.spaCliqueType= '4NN';

params.maxMoveIters = 3; % maximum number of QPBO iteration per individual move

params.PruneToClosest = false; 

params.supMFType    = 'no_mf';
params.flowMFType   = 'none';
params.lambdaMF     = 25; %params.lambdaSeg/3; % for non-local term if used
params.flowFusionFeature  = false; 
% %fprintf('use feature matching %d\n', params.flowFusionFeature);

params.flowPrior     = 'mrfOnFlow';
params.isSupportTempOn = false;
params.isFlowL1Norm    = true;

% input from the calling functions

if nargin > 0
    %%%%%%%%%%%% Parse parameters
    

    % from continuous model
    % params.rho_affine   = model.rho_affine;
    % Temporarily
    % params.cont_refine = false;
    
    
    if isfield(model.discParams, 'isMex')
        params.isMex    = model.discParams.isMex;
        %fprintf('isMex = %d\n', params.isMex);
    else
        %fprintf('isMex undefined for model.extParams, set to false\n');
        params.isMex    = false;
    end;
    
    params.lambdaFlow       = model.lambda;
    % params.lambdaAff      = model.lambdaAff;
    params.lambdaOcc        = model.lambda5;
    params.T2_lab           = model.Ti2;  % 20^2 vs 40^2
    params.sigma_lab        = model.sigma_i; % 7 vs 12
    params.is_symmetric     = model.is_symmetric;
    params.inputDownRatio   = model.disDownRatio;
    
    if isfield(model.intParams, 'nGCiters')
        params.nGCiters= model.intParams.nGCiters;
        %fprintf('nGCiters %d \n' , params.nGCiters);
    else
        params.nGCiters=1;
        %fprintf('nGCiters unchanged \n');
    end;
    
    if isfield(model.intParams, 'nMaxSwapPairs')
        params.nMaxSwapPairs= model.intParams.nMaxSwapPairs;
        %fprintf('nMaxSwapPairs %d \n' , params.nMaxSwapPairs);
    else
        %fprintf('nMaxSwapPairs %d unchanged \n', params.nMaxSwapPairs);
    end;
    
    if isfield(model.intParams, 'nSwapIters')
        params.nSwapIters= model.intParams.nSwapIters;
        %fprintf('nSwapIters %d \n' , params.nSwapIters);
    else
        %fprintf('nSwapIters %d unchanged \n', params.nSwapIters);
    end;
    
    if isfield(model.discParams, 'interpolation_method')
        params.interpolation_method= model.discParams.interpolation_method;
        %fprintf('interpolation_method %s \n' , params.interpolation_method);
    end;
    
    if isfield(model.discParams, 'deriv_filter')
        params.deriv_filter= model.discParams.deriv_filter;
        %fprintf('deriv_filter changed \n');
    end;
    
    if isfield(model.discParams, 'rho_data')
        params.rho_data= model.discParams.rho_data;
        %fprintf('rho_data changed \n');
    end;
    
    if isfield(model.discParams, 'rho_spatial')
        params.rho_spatial= model.discParams.rho_spatial;
        %fprintf('rho_spatial changed \n');
    end;
    
    if isfield(model.discParams, 'rho_affine')
        params.rho_affine= model.discParams.rho_affine;
        %fprintf('rho_affine changed \n');
    end;
    
    if isfield(model.discParams, 'isBasicMove')
        params.isBasicMove= model.discParams.isBasicMove;
        %fprintf('isBasicMove changed to %d \n', params.isBasicMove);
    else
        params.isBasicMove = false;
        %fprintf('isBasicMove remains  %d \n', params.isBasicMove);
    end;
    
    % whether use temporal term of support function during cont flow estimation
    if isfield(model.discParams, 'isSupportTempOn')
        params.isSupportTempOn= model.discParams.isSupportTempOn;
        %fprintf('isSupportTempOn changed to %d \n', params.isSupportTempOn);
    end;
    
%     if isfield(model.discParams, 'isSupportTempOn')
%         params.isSupportTempOn= model.discParams.isSupportTempOn;
%         %fprintf('isSupportTempOn changed to %d \n', params.isSupportTempOn);
%     else
%         params.isSupportTempOn = false;
%     end;
    
    if isfield(model.discParams, 'spaCliqueType')
        params.spaCliqueType= model.discParams.spaCliqueType;
        %fprintf('spaCliqueType changed to %s \n', params.spaCliqueType);
    end;

    if isfield(model.discParams, 'flowMFType')
        params.flowMFType= model.discParams.flowMFType;
        %fprintf('flowMFType changed to %s \n', params.flowMFType);
    end;

    if isfield(model.discParams, 'supMFType')
        params.supMFType= model.discParams.supMFType;
        %fprintf('supMFType changed to %s \n', params.supMFType);
    end;
    
    if isfield(model.discParams, 'PruneToClosest')
        params.PruneToClosest= model.discParams.PruneToClosest;
        %fprintf('PruneToClosest changed to %d \n', params.PruneToClosest);
    end;

    if isfield(model.discParams, 'flowFusionFeature')
        params.flowFusionFeature= model.discParams.flowFusionFeature;
        %fprintf('flowFusionFeature changed to %d \n', params.flowFusionFeature);
    end;
    
    if isfield(model.discParams, 'flowPrior')
        params.flowPrior= model.discParams.flowPrior;
        %fprintf('flowPrior changed to %s \n', params.flowPrior);
    end;

    if isfield(model.discParams, 'isFlowL1Norm')
        params.isFlowL1Norm= model.discParams.isFlowL1Norm;
        %fprintf('isFlowL1Norm changed to %d \n', params.isFlowL1Norm);
    end;    

    
    % from input parameters
    params.lambdaSeg    = model.extParams.lambdaSeg;
    params.lambdaIncon  = model.extParams.lambdaIncon;
    params.lambdaAff    = model.extParams.lambdaAff;
    params.T2_lab       = model.extParams.T2_lab;
    %fprintf('T2_lab %3.3f \n', params.T2_lab);
    
    if strcmp(params.flowPrior, 'mrfOnDev')
        params.lambdaAff    = model.lambdaAff;
    end;
%     params.fdir = [model.fileinfo 'dis' filesep];
%     if ~exist(params.fdir, 'file')
%         mkdir(params.fdir);
%     end;
    
end;