function out = compute_pw_t_potential_state_si_asym(vals, ...
                matchCost, params, OOB, prevP, currP, state, t1, isForward)
%%
% Compute the temporal pairwise term for layer segmentation with the
% sequential Ising model. the input states, vals(3:4,:), select which group
% of layer support (3D array) to use, because the selection variable
% changes the flow and there for the temporal clique structure

% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

% the temporal pairwise terms only take effect when the first variables are
% in the chosen STATE

% test code (for code optimization)
% params.lambdaOcc    =1;
% params.lambdaIncon  =1;
% 
% H = 50; W = 50; nP = 10; T = 2;
% prevP = round(rand(H, W, nP, T));
% currP = round(rand(H, W, nP, T));

if ~exist('isForward', 'var');
    isForward = true;
end;

[H W nP T] = size(prevP);
nLayers    = nP+1;

% each support function is extended (usualy by 4) to generate proposals
HP    = size(vals,2)/nLayers;

prevV = support2visibility(prevP);
currV = support2visibility(currP);

% ind1  = vals(1,:);

% ind1  = vals(1,:) - (t1-1)*H*W;
% % for indexing visibility masks
% indV1 = ind1;
% % for indexing support
% indP1 = ind1(1:HP*(nLayers-1));
% 
% ind2  = vals(2,:) - t1*H*W ;
% % for indexing visibility masks
% indV2 = ind2 + nLayers*H*W;
% % for indexing support
% indP2 = ind2(1:HP*(nLayers-1)) + (nLayers-1)*H*W;

if isForward
    ind1  = vals(1,:) - (t1-1)*H*W;
    % for indexing visibility masks, layer_ID will be added later
    indV1 = ind1;
    % for indexing support
    indP1 = ind1(1:HP*(nLayers-1));
    
    ind2  = vals(2,:) - t1*H*W ;
    % for indexing visibility masks
    indV2 = ind2 + nLayers*H*W;
    % for indexing support
    indP2 = ind2(1:HP*(nLayers-1)) + (nLayers-1)*H*W;
else
    ind1  = vals(1,:) - t1*H*W;
    %ind1  = vals(1,:) - (t1-1)*H*W;
    % for indexing visibility masks
    indV1 = ind1 + nLayers*H*W;
    % for indexing support
    indP1 = ind1(1:HP*(nLayers-1)) + (nLayers-1)*H*W;
    
    ind2  = vals(2,:) - (t1-1)*H*W ;
    %ind2  = vals(2,:) - (t1-2)*H*W ;
    % for indexing visibility masks
    indV2 = ind2;
    % for indexing support
    indP2 = ind2(1:HP*(nLayers-1));
end;   

% obtain layer IDs for each state
layerID = repmat([1:nLayers], [HP 1]);
layerID = layerID(:)';
  
% data matching cost
tmp_ind = ind1 + (layerID-1)*H*W;  
out     = matchCost(tmp_ind);
oob     = OOB(tmp_ind);

% out of boundary
out(oob==1) = params.lambdaOcc;

tmp_ind = indV2 + (layerID-1)*H*W;  
vis2    = prevV(tmp_ind).*vals(4,:) + ...
          currV(tmp_ind).*(1-vals(4,:));            
% invisible at other frame
out(vis2==0) = params.lambdaOcc;

% invisible at current frame
tmp_ind = indV1 + (layerID-1)*H*W;  
vis1    = prevV(tmp_ind).*vals(3,:) + ...
          currV(tmp_ind).*(1-vals(3,:));
out(vis1==0) = 0;      

% corresponding nodes at frame t+1
% tmp_ind= indP2;
% for iL = 1:nLayers-1
%     tmp_ind( (iL-1)*HP+1:iL*HP ) =...
%         tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
% end;
tmp_ind = indP2 + (layerID(1:HP*(nLayers-1))-1)*H*W;
sup2    = prevP(tmp_ind).*vals(4,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(4,1:HP*(nLayers-1)));
      
% corresponding nodes at frame t            
% tmp_ind= indP1;
% for iL = 1:nLayers-1
%     tmp_ind( (iL-1)*HP+1:iL*HP ) =...
%         tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
% end;
tmp_ind = indP1 + (layerID(1:HP*(nLayers-1))-1)*H*W;
sup1    = prevP(tmp_ind).*vals(3,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(3,1:HP*(nLayers-1)));      
      
% add temporal consistency term for the nLayers-1 support functions
out(1:HP*(nLayers-1)) = out(1:HP*(nLayers-1)) +...
    params.lambdaIncon*(sup1~=sup2);

% turn off temporal cliques that are off at the center frame
isOff = vals(3,:) ~= state;
out(isOff) = 0;