function [data solution] = move_front_layer_to_back_asym(data, solution, params, layer_ids)
%%
% move the front layer behind the back layer while retaining the
% visibility mask and making the minimum change to the support functions

nLayers = size(solution.layer_support,3)+1;

layer_ids = sort(layer_ids, 'ascend');

if layer_ids(1) == layer_ids(2)
    return;
end;

preV   = support2visibility(solution.layer_support);

if max(layer_ids) < nLayers
    
    new_orders = [1:layer_ids(1)-1 layer_ids(1)+1:layer_ids(2)...
        layer_ids(1) layer_ids(2)+1:nLayers-1];        
    sup = solution.layer_support(:,:,new_orders,:);
else
    new_orders = [1:layer_ids(1)-1 layer_ids(1)+1:nLayers-1 nLayers-1];
    
    sup = solution.layer_support(:,:,new_orders,:);    
    sup(:,:,nLayers-1,:) = preV(:,:,layer_ids(2),:) == 1;    
end;

newV = support2visibility(sup);

% set visiblity of the new back layer to be same as before by changing the
% support function of those in front of it to be zero at where its
% previously visible pixels are invisible now

mask = newV(:,:,layer_ids(2),:) == 0 & preV(:,:,layer_ids(1),:) == 1;

for iL = 1:layer_ids(2)-1
    tmp = sup(:,:,iL,:);
    tmp(mask) = 0;
    sup(:,:,iL,:) = tmp;
end;

% set visiblity of back layer to be same as before
% mask = newV(:,:,layer_ids(2),:) == 0 & preV(:,:,layer_ids(1),:) == 1;
% tmp = sup(:,:,1:layer_ids(2)-1,:);
% tmp(repmat(mask, [1 1 layer_ids(2)-1 1])) = 0;
% sup(:,:,1:layer_ids(2)-1,:) = tmp;

solution.layer_support = sup;

T = size(solution.layer_support,4);

%%% move the flow fields
for t=1:T-1;

    new_orders = [1:layer_ids(1)-1 layer_ids(1)+1:layer_ids(2) ...
        layer_ids(1), layer_ids(2)+1:nLayers];        
    
    data.aff_uv{t} =...
        data.aff_uv{t}(:,:,:, new_orders);
    data.layer_uv{t} =...
        data.layer_uv{t}(:,:,:, new_orders);
    
    if params.is_symmetric
        data.aff_uvb{t}   =...
            data.aff_uvb{t}(:,:,:, new_orders);
        data.layer_uvb{t} =...
            data.layer_uvb{t}(:,:,:, new_orders);
    end;
    

end;
