function [oData solution params]= parse_data_seq_si_asym(...
    layer_uv, layer_uvb, pyramid_images, color_pyramid_images,...
    layer_g, seg, this)
% parse the data from continuous optimization format to the discrete
% optimization format 

%%


params          = load_parse_parameters(this);

%%%%%%%%%       convert continuous data to discrete formats

inputDownRatio  = params.inputDownRatio;
inputSZ         = size(seg{1});
downSZ          = round(inputSZ/inputDownRatio);
T               = length(layer_g);
nLayers         = length(layer_uv{1});



%%%%% forward flow field

% fit affine mean flow field (robustly)
for t=1:T-1   
    layer_mean_affine_uv{t}  = fit_affine_layer_motion_g(...
        layer_uv{t}, layer_g{t}, params.rho_affine);
end;                    

% down sample segmentation and support functions
% convert continuous support function to binary support functions


layer_seg       = zeros([downSZ T]);
layer_support   = zeros([downSZ nLayers-1 T]);

for t=1:T
%     sseg{t} = imresize(seg{t}, downSZ, 'nearest');    
%     layer_g_s{t} = imresize(layer_g{t}, downSZ);    
%     layer_support(:,:,:,t) = layer_g_s{t} > 0;    
%     layer_seg(:,:,t) = sseg{t};    
    layer_seg(:,:,t)        = imresize(seg{t}, downSZ, 'nearest');    
    layer_support(:,:,:,t)  = imresize(layer_g{t}, downSZ) > 0;    
end;

% down sample flow field,
for t=1:T-1        
    oData.layer_uv{t} = zeros([downSZ 2 nLayers]);
    oData.aff_uv{t}   = zeros([downSZ 2 nLayers]);
    
    for i=1:nLayers
%         tmp_uv = imresize(layer_uv{t}{i}, downSZ)/inputDownRatio;
%         oData.layer_uv{t}(:,:,:,i)  = tmp_uv;
%         tmp_uv = imresize(layer_mean_affine_uv{t}{i}, downSZ)/inputDownRatio;
%         oData.aff_uv{t}(:,:,:,i)    = tmp_uv;
        oData.layer_uv{t}(:,:,:,i)  = ...
            imresize(layer_uv{t}{i}, downSZ)/inputDownRatio;
        oData.aff_uv{t}(:,:,:,i)    = ...
            imresize(layer_mean_affine_uv{t}{i}, downSZ)/inputDownRatio;
    end;    

    % for old nLayer+1 candidate flow fields, may be deleted
    uvf = composite_final_flow_seg(oData.layer_uv{t},  layer_seg(:,:,t));
    oData.can_uv{t} = cat(4, oData.aff_uv{t}, uvf);

    if params.flowFusionFeature
        oData.feature_uv{t} = compute_sift_candidate(...
            this.extParams.grayImages(:,:,t),...
            this.extParams.grayImages(:,:,t+1), uvf);
    end;

end;

%%%%%%%%%%%%%%% TODO (delete): flow_sel not used

% assign initial flow selection variables: pick up single-layer output for
% visible pixels, affine mean flow field for invisible pixels
for t=1:T-1
    flow_sel{t} = ones([downSZ nLayers]);
    for i=1:nLayers
        tmp = ones(downSZ)*i;        
        tmp(layer_seg(:,:,t)==i) = nLayers+1; 
        flow_sel{t}(:,:,i) = tmp;        
    end;    
end;

%%%% image pyramid

% down sample image features and color images

% redundant but correct implementation
% factor            = sqrt(2);  % sqrt(3) worse
% smooth_sigma      = sqrt(params.inputDownRatio)/factor; 
% f          = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
% pimages    = compute_image_pyramid(pyramid_images{1}, f, 2, 1/params.inputDownRatio);
% images     = pimages{2};

if strcmp(params.downOption, 'hires_0.5')
    images = pyramid_images{1};
    for t=1:T
        if size(color_pyramid_images{1}, 4)>1
            labImages{t} = imresize(color_pyramid_images{1}(:,:,:,t), downSZ);
        else
            labImages{t} = imresize(color_pyramid_images{1}(:,:,t), downSZ);
        end;
    end;
else
    images = imresize(pyramid_images{1}, downSZ);
    for t=1:T
        if size(color_pyramid_images{1}, 4)>1
            labImages{t} = imresize(color_pyramid_images{1}(:,:,:,t), downSZ);
        else
            labImages{t} = imresize(color_pyramid_images{1}(:,:,t), downSZ);
        end;
    end;
end;
oData.images    = images; %{images, images(:,:,2:-1:1)};
oData.labImages = labImages;

params.nCan     = size(oData.can_uv{1}, 4);
params.nLayers  = nLayers;
params.T        = T;
H               = downSZ(1);
W               = downSZ(2);
params.startIndex = 1 + (0:T-1)*H*W;
% params.startIndex_b = 1 + (0:T-1)*H*W;

solution.layer_support  = layer_support; 
solution.flow_sel       = flow_sel;


%%%%%%%%%% backward flow field

if params.is_symmetric    
    for t=1:T-1               
        layer_mean_affine_uvb{t}  = fit_affine_layer_motion_g(...
            layer_uvb{t}, layer_g{t+1}, params.rho_affine);
    end;
    
    for t=1:T-1        
        oData.layer_uvb{t} = zeros([downSZ 2 nLayers]);
        oData.aff_uvb{t}   = zeros([downSZ 2 nLayers]);
        
        for i=1:nLayers            
            oData.layer_uvb{t}(:,:,:,i)= ...
                imresize(layer_uvb{t}{i}, downSZ)/inputDownRatio;
            oData.aff_uvb{t}(:,:,:,i)  = ...
                imresize(layer_mean_affine_uvb{t}{i}, downSZ)/inputDownRatio;
        end;             
        
        uvb = composite_final_flow_seg(oData.layer_uvb{t}, layer_seg(:,:,t+1));
        oData.can_uvb{t} = cat(4, oData.aff_uvb{t}, uvb);

        if params.flowFusionFeature
            oData.feature_uvb{t} = compute_sift_candidate(...
                this.extParams.grayImages(:,:,t+1),...
                this.extParams.grayImages(:,:,t), uvb);
        end;
    end;
    
%%%%%%%%%%%%%%% TODO (delete): flow_sel not used        
    % flow selection
    for t=1:T-1        
        flow_sel_b{t} = ones([downSZ nLayers]);
        for i=1:nLayers
            tmp = ones(downSZ)*i;
            tmp(layer_seg(:,:,t+1)==i) = nLayers+1;
            flow_sel_b{t}(:,:,i) = tmp;
        end;
    end;    
    
    solution.flow_sel_b     = flow_sel_b;
end;


%%%% weights for support functions 

% spatial cliques, weights for layer segmentation

% to see clique structure: im2col(reshape(1:16, [4 4]),[2 2])'



vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

if strcmp(params.spaCliqueType, '4NN');
    oData.spaCliquesSup = [vert; hor];
else
    cliques = im2col((reshape(1:H*W,H,W)),[2 2])';
    dia1    = cliques(:, [1 4]);
    dia2    = cliques(:, [2 3]);
    oData.spaCliquesSup = [vert; hor; dia1; dia2];
end;

for t=1:params.T
    wv = compute_color_weight(oData.labImages{t}, vert, params.T2_lab,...
        params.sigma_lab, params.rho_color);
    wh = compute_color_weight(oData.labImages{t}, hor,  params.T2_lab,...
        params.sigma_lab, params.rho_color);
    
    if  strcmp(params.spaCliqueType, '4NN');
        weight = repmat([wv(:)' wh(:)'], [4 1]);        
    else
        wd1 = compute_color_weight(oData.labImages{t}, dia1, params.T2_lab,...
            params.sigma_lab, params.rho_color);
        wd2 = compute_color_weight(oData.labImages{t}, dia2, params.T2_lab,...
            params.sigma_lab, params.rho_color);

        weight = repmat([wv(:)' wh(:)' wd1(:)' wd2(:)' ], [4 1]);

    end

    weight = weight(:)';
    
    oData.weight{t} = weight;
    
end;

