function  [data solution params] = refine_flow_move(data, solution, params, this)
%%
%  FlowFusion to make big changes to the flow field of each layer
%  followed by continuous flow refinement

T       = size(solution.layer_support, 4);
nLayers = size(solution.layer_support, 3)+1;
params.nLayers = nLayers;
params.T       = T;

data    = estimate_affine_mean_asym(data, solution, params);
% data = precompute_asym(data, params);
% en = evaluate_overall_energy_si_asym(solution, data, params, params.display);
% fprintf('re-estimate affine energy %3.3e\n', en);

[H W ~] = size(data.images);

% Preallocate QPBO optimizer
QPBO_all_in_one_prealloc('a',H*W);

% Occlusion-aware FlowFusion move
for t=1:T-1;
    for iL=1:nLayers
        data.layer_uv{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
            solution, t, iL, params, data);
        if params.is_symmetric
            data.layer_uvb{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
                solution, t, iL, params, data, false);
        end;
    end;
end;

% Clean up QPBO optimizer
QPBO_all_in_one_prealloc('d');

% data    = precompute_asym(data, params);
% en      = evaluate_overall_energy_si_asym(solution, data, params, params.display);
% fprintf('fusion flow energy %3.3e\n', en); 


% Perform continuous flow refinement

if params.cont_refine
    
    small = this;
    
    if strcmp(params.flowPrior, 'mrfOnDev')
        small.MRFOnFlow     = false;        
        small.lambdaAffCont = this.lambdaAffCont;
    else
        small.MRFOnFlow     = true;        
        small.lambdaAffCont = params.lambdaAff;
    end;
    
    small.mf_mode = params.flowMFType; 
    
    for iF = 1:length(small.rho_spatial_v)
        small.rho_spatial_u{iF} = params.rho_spatial;
        small.rho_spatial_v{iF} = params.rho_spatial;
    end;
    small.rho_affine = params.rho_affine;
    small.rho_data   = params.rho_data;
    small.interpolation_method = params.interpolation_method;    
       
    vis = support2visibility(solution.layer_support);
    
    % soften visibility mask
    vis(vis==1) = params.softDiscVis;
    vis(vis==0) = 1-params.softDiscVis;
        
    layer_seg   = support2seg(solution.layer_support);
    
    
    for t=1:T-1;
        for iL=1:nLayers            
            small.images  = data.images(:,:,t:t+1);
            small.mean_uv = data.aff_uv{t}(:,:,:,iL);
            small.labImage= data.labImages{t};
            uv            = data.layer_uv{t}(:,:,:,iL);
            
            if sum(layer_seg(:)==iL) < params.nIgnore
                continue;
            else                
                for i1 = 1:params.cont_refine_iters                    
                    % Multiply warped labels at next frame for data term
                    tmp = imwarp_grad_im(uv, vis(:,:,iL, t+1));
                    small.layer_s     = cat(3, vis(:,:,iL, t), tmp);
                    tmp(isnan(tmp))   = 0;
                    small.layerLabel  = vis(:,:,iL, t).*tmp;
                    
                    if params.isSupportTempOn && (iL < nLayers)
                        small.lambda3 = params.lambdaIncon;
                        g  = solution.layer_support(:,:,iL, t);
                        g2 = solution.layer_support(:,:,iL, t+1);
                        uv = compute_flow_base(small, uv, g, g2);
                    else
                        uv = compute_flow_base(small, uv);
                    end;
                end;
                data.layer_uv{t}(:,:,:,iL) = uv;
            end;
            
            if params.is_symmetric
                small.images  = data.images(:,:,t+1:-1:t);
                small.mean_uv = data.aff_uvb{t}(:,:,:,iL);
                small.labImage= data.labImages{t+1};
                uv            = data.layer_uvb{t}(:,:,:,iL);
                
                if sum(layer_seg(:)==iL) <params.nIgnore
                    continue;
                else
                    for i1 = 1:params.cont_refine_iters;                        
                        % Multiply warped labels at next frame for data term
                        tmp = imwarp_grad_im(uv, vis(:,:,iL, t));
                        small.layer_s     = cat(3, vis(:,:,iL, t+1), tmp);
                        tmp(isnan(tmp))   = 0;
                        small.layerLabel  = vis(:,:,iL, t+1).*tmp;
                        if params.isSupportTempOn && (iL < nLayers)
                            small.lambda3 = params.lambdaIncon;
                            g  = solution.layer_support(:,:,iL, t+1);
                            g2 = solution.layer_support(:,:,iL, t);
                            uv = compute_flow_base(small, uv, g, g2);                            
                        else                            
                            uv = compute_flow_base(small, uv);
                        end;
                    end;
                    data.layer_uvb{t}(:,:,:,iL) = uv;
                end;                
            end;
            
        end;
    end;    
end;
