function can_uv = compute_sift_candidate(image1, image2, uv)
%%
% compute candidate flow field using the SIFT matching algorithm

% See the web page at 
%   http://www.cs.ubc.ca/~lowe/keypoints/ 
% for references to the relevant papers describing this approach.

[im1, des1, loc1] = sift(image1);
[im2, des2, loc2] = sift(image2);

% For efficiency in Matlab, it is cheaper to compute dot products between
%  unit vectors rather than Euclidean distances.  Note that the ratio of 
%  angles (acos of dot products of unit vectors) is a close approximation
%  to the ratio of Euclidean distances for small angles.
%
% distRatio: Only keep matches in which the ratio of vector angles from the
%   nearest to second nearest neighbor is less than distRatio.
distRatio = 0.6;   

% For each descriptor in the first image, select its match to second image.
des2t = des2';                          % Precompute matrix transpose
for i = 1 : size(des1,1)
   dotprods = des1(i,:) * des2t;        % Computes vector of dot products
   [vals,indx] = sort(acos(dotprods));  % Take inverse cosine and sort results

   % Check if nearest neighbor has angle less than distRatio times 2nd.
   if (vals(1) < distRatio * vals(2))
      match(i) = indx(1);
   else
      match(i) = 0;
   end
end

%%
H       = size(im1,1);
W       = size(im1,2);
tmp     = ones([H W]);

if nargin >= 3
    T_flow  = max(abs(uv(:)))*2;
else
    T_flow  = max(H, W)/2;
end;

T_thres = 2;
iCan    = 1;

neighbor=-2:2;

% TODO: add a new flow candidate if similar matching hasn't been added

add_uv  = [];
isAdded = false;

can_uv  = [];
for i = 1: size(des1,1)
  if (match(i) > 0)
      
      if ~isempty(add_uv)
          dif = sqrt( (add_uv(:,1)-u).^2 + (add_uv(:,2)-v).^2 );
          
          if min(dif) < 1
              isAdded = true;
          else
              isAdded = false;
          end;
          
      end
      
      u = loc2(match(i),2)- loc1(i,2);
      v = loc2(match(i),1)- loc1(i,1);      
      
      yind = round(loc1(i,1))+neighbor;
      xind = round(loc1(i,2))+neighbor;
      
      yind(yind<1) = 1;
      yind(yind>H) = H;
      xind(xind<1) = 1;
      xind(xind>W) = W;      
      
      up = uv(yind, xind, 1);
      vp = uv(yind, xind, 2);
      
      dif = sqrt( (u-up(:)).^2 + (v-vp(:)).^2 );
            
      if  (min(dif) > T_thres) && ~isAdded &&... 
              (abs(u) < T_flow) && (abs(v) < T_flow)
          
          %can_uv{iCan} = cat(3, u*tmp, v*tmp);

          can_uv = cat(4, can_uv, cat(3, u*tmp, v*tmp) );

          iCan = iCan + 1;
          
          add_uv = [add_uv; u v];
      end;
      
  end  
end