function out = change_flow_selection(k, sel, seg)
%%
% The effect is to generate the proposal that every pixel in layer K is
% visible but with the current flow selection. For the corresponding
% visible pixels in the other layer, their flow selection weil be the
% affine flow field for that layer

% seg selects layer, not the candidate flow fields

%% test code
% 
% sz = [100 100 3];
% sel = {round(rand(sz)*2)+1, round(rand(sz)*2)+1};
% seg = ones(100,100,2)*2;
% 
% out = change_flow_selection(1, sel, seg); 
% figure; imagesc(out{1}(:,:,1));
% figure; imagesc(out{2}(:,:,1));
% figure; imagesc(out{1}(:,:,2));
% 
% out = change_flow_selection(5, sel, seg); 
% out = change_flow_selection(1, sel{1}, seg); 
%

if iscell(sel);
    T = length(sel);
    [H W nLayers] = size(sel{1});

    if k >nLayers || k<1
        error(['change_flow_selection: cannot assign pixels to layer'...
              num2str(k), '!']);
    end;

    out = sel;
    
    ind = reshape(1:H*W, [H W]);
    
    for t=1:T
        % set flow selection for the kth flow field to be that of the
        % originally visible pixel
        %out{t}(:,:,k) = seg(:,:,t);
        
        tmp_ind = ind + (seg(:,:,t)-1)*H*W;
        out{t}(:,:,k) = sel{t}(tmp_ind);
        
        % set flow selection of pixels that become invisible at other
        % layers to be the corresponding affine flow
        for iL=1:nLayers            
            
            % if a pixel remain in the selected layer, it uses the
            % candidate flow, not the affine flow
            if iL==k
                continue;
            end;
            
            tmp = out{t}(:,:,iL);
            % set flow selection of originally visible pixels to be their
            % affine flow field, this assumes that "i"th candidate flow field
            % is the affine flow for the "i"th layer
            tmp(seg(:,:,t)==iL) = iL;            
            out{t}(:,:,iL) = tmp;            
            
            %out{t}(:,:,iL) = set_to_mask(out{t}(:,:,k), seg(:,:,t)==iL, iL);
            
        end;        
    end;
    
else
    error('change_flow_selection: sel should be cell array');
end;
