function [data layer_support flow_sel] = compute_init_clustering(params)
%%
% load flow data, perform robust affine clustering to obtain initial
% segmentation, and layer support


nL = params.nLayers;
nClusterRun = params.nClusterRun;
seqName = params.seqName;
t0 = params.t;
iBest = params.iBest;
rho = params.rho_affine;
homeDir = params.homeDir;
inputDownRatio = params.inputDownRatio;

initDir = [homeDir 'result/precompute/' seqName filesep];

fn  = [initDir 'forward' sprintf('%03d.flo',t0)];    
if exist(fn, 'file')
    fnb = [initDir 'backward' sprintf('%03d.flo',t0)];    
else
    fn  = [initDir 'forwarde' sprintf('%02d.flo',t0)];    
    fnb = [initDir 'backwarde' sprintf('%02d.flo',t0)];    
end;

uvf = readFlowFile(fn);
uvb = readFlowFile(fnb);

% downsample

[H W n] = size(uvf);
H  = round(H/inputDownRatio);
W  = round(W/inputDownRatio);
uvf = imresize(uvf, [H W])/inputDownRatio;
uvb = imresize(uvb, [H W])/inputDownRatio;


for i = 1:nClusterRun
    [A{i} score(i) segs{i}] =robust_fit_affine_motion_rho(uvf, nL, rho);      
end;

[tmp ind] = sort(score, 'ascend');

layer_seg(:,:,1) = segs{ind(iBest)};
layer_seg(:,:,2) = warp_backward_segmentation(layer_seg(:,:,1), uvb);
layer_vis        = seg2visibility(layer_seg);
layer_support    = visibility2support(layer_vis);

%%
for i=1:nL
    tmp_A = squeeze(A{ind(iBest)}(:,i,:));
    layer_uvf(:,:,:,i) = affine2motion(tmp_A, H, W);    
end;

layer_uvb = fit_affine_layer_motion_robust(uvb, layer_seg(:,:,2), rho);

data.aff_uv = {layer_uvf, layer_uvb};

layer_uvf(:,:,:,nL+1) = uvf;
layer_uvb(:,:,:,nL+1) = uvb;

% layer_uv = affine2layer(A{ind(iBest)}, [size(uvf,1) size(uvf,2)]);
% lyaer_uvb = [];

data.can_uv = {layer_uvf, layer_uvb};


for t=1:params.T
    for i=1:nL
        tmp = ones(H,W)*i;
        tmp(layer_seg(:,:,t)==i) = nL+1;        
        flow_sel{t}(:,:,i) = tmp;
    end;    
end;

% load images
dataDir = [homeDir 'data/' seqName filesep];
images  = [];
for i=1:params.T
    fn = [dataDir sprintf('frame%03d.png', t0+i-1)];
    if ~exist(fn, 'file')
        fn = [dataDir sprintf('frame%02d.png', t0+i-1)];
    end;    
    im = imread(fn);
    
    %downsample
    im = imresize(im, [H W], 'cubic');
    
    if size(im,3) >1
        images = cat(4, images, im);
    else
        images = cat(3, images, im);
    end;
    
end;

if size(images,4) > 1
    grayImages = [];
    labImages = [];
    for i = 1:size(images,4)
        grayImages = cat(3, grayImages, double(rgb2gray(uint8(images(:,:,:,i)))) );
        
        im1 = RGB2Lab(images(:,:,:,i));
        for j = 1:size(im1, 3);
            im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
        end;
        labImages = cat(4, labImages, im1);
    end;    
else
    grayImages = double(images);    
    % Duplicate gray for color-modulated hidden field prior
    labImages = [];        
    for i = 1:size(images,3)
        im1 = images(:,:,i);
        labImages = cat(4, labImages, repmat(im1, [1 1 3]));
    end;        
end;

% Perform structure texture decomposition to get the texture component
texImages  = structure_texture_decomposition_rof(grayImages);

data.images = {texImages, texImages(:,:,2:-1:1)};

data.labImages = {labImages(:,:,:,1), labImages(:,:,:,2)};
