function [data solution params] = compute_init_gc_all_asym(data, solution, params)
%%

%  visiblity per layer
%  visibility and flow per layer
%  support per layer
%  affine and continuous flow per layer



for iter=1:params.nGCiters
    
    % layer segmentation
    solution = segment_layers_gc_si_asym(solution, params, data);
    
    en = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en);       title('after seg');
    
    % remove unnecessary layers
    [data solution params] = prune_layers_asym(data, solution, params);
    data    = precompute_asym(data, params);
    
    % simultaneous layer segmentation + flow selection
    [solution data]= segment_layers_select_flow_gc_si_asym(...
        solution, params, data);
    
    % remove unnecessary layers
    [data solution params] = prune_layers_asym(data, solution, params);
    
    %update data.matchCost since the layer flow fields (layer_uv) change
    %in segment_layers_select_flow_gc_si_asym
    data    = precompute_asym(data, params);
    
    en = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en);  title('after seg & flow');
    
    % update support function for each layer
    solution = segment_layers_change_support_gc_si_asym(solution, params, data);
    en = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en);        title('after support');
    
    % flow selection
    for t=1:T-1;
        for iL=1:params.nLayers
            data.layer_uv{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
                solution, t, iL, params, data);
            if params.is_symmetric
                data.layer_uvb{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
                    solution, t, iL, params, data, false);
            end;
        end;
    end;
    
    en      = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en);  title('after discrete flow');
    
    % re-estimate affine mean
    data=estimate_affine_mean_asym(data, solution, params);
    
    en      = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en);  title('after affine mean');
    
    % continuous refinement of flow field for each layer (3-warping)
    small = this;
    % Use strong weight on affine deviation term
    small.lambdaAffCont = this.lambdaAff;
    small.mf_mode   = 'none'; %'mf' 'wmf'
    vis = support2visibility(solution.layer_support);
    
    % soften visibility mask
    vis(vis==1) = 0.999;
    vis(vis==0) = 0.001;
    
    layer_seg = support2seg(solution.layer_support);
    
    nLinIters = 3;
    for t=1:T-1;
        for iL=1:params.nLayers
            small.images  = data.images(:,:,t:t+1);
            small.mean_uv = data.aff_uv{t}(:,:,:,iL);
            uv            = data.layer_uv{t}(:,:,:,iL);
            
            if sum(layer_seg(:)==iL) <params.nIgnore
                continue;
            else
                for i1 = 1:nLinIters;
                    
                    % Multiply warped labels at next frame for dataterm
                    tmp = imwarp_grad_im(uv, vis(:,:,iL, t+1));
                    small.layer_s     = cat(3, vis(:,:,iL, t), tmp);
                    tmp(isnan(tmp))   = 0;
                    small.layerLabel  = vis(:,:,iL, t).*tmp;
                    
                    uv = compute_flow_base(small, uv);
                end;
                
                data.layer_uv{t}(:,:,:,iL) = uv;
            end;
        end;
    end;
    
    %update data.matchCost since the layer flow fields (layer_uv) change
    data    = precompute_asym(data, params);
    
    en      = evaluate_overall_energy_si_asym(solution, data, params, 1);
    fprintf('energy %3.3e\n', en); title('after continuous flow');
end
