function cost = compute_pw_flow_spatial_potential_si(vals, ...
               can_uv, params, prevSel, currSel)
%% 
% Flow smoothness for each layer according to the flow selection (0: use
% previous selection; 1: use current selection)
% 
% %%
% %%
% params.lambdaFlow    =1;
% H = 10; W = 10; nLayers = 5; 
% prevSel = round(rand(H, W, nLayers)*(nLayers-1))+1;
% currSel = round(rand(H, W, nLayers)*(nLayers-1))+1;
% vals = round(rand(4, H*W*2))+1;
% vals(1:2,:) = round(rand(2, H*W*2)*(H*W-2))+1;
% vals(3:4, :) = double(vals(3:4, :) ==1);
% can_uv = randn(H, W, 2, nLayers);
% 
% tic
% out = compute_pw_flow_spatial_potential_si(vals, can_uv, params, prevSel, currSel);
% toc
% %%
% tic
% out2 = compute_pw_flow_spatial_potential_si_mex(vals, can_uv, params.lambdaFlow, prevSel, currSel);
% toc
% dif = out-out2;
% max(abs(dif)) ans =    3.5527e-15


%%
rho = params.rho_spatial;
lambdaFlow    = params.lambdaFlow;
[H W nLayers] = size(prevSel);

% convert to states of each layer
% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

cost = zeros(size(vals(3,:)));
ind1 = vals(1,:);
ind2 = vals(2,:);

for i=1:nLayers    
    tmp_ind1 = (i-1)*H*W + ind1;
    tmp_ind2 = (i-1)*H*W + ind2;

    s1 = prevSel(tmp_ind1).*vals(3,:) + currSel(tmp_ind1).*(1-vals(3,:));
    s2 = prevSel(tmp_ind2).*vals(4,:) + currSel(tmp_ind2).*(1-vals(4,:));

    % horizontal flow
    tmp_ind1 = (s1-1)*2*H*W + ind1;
    tmp_ind2 = (s2-1)*2*H*W + ind2;    
    f1 = can_uv(tmp_ind1);
    f2 = can_uv(tmp_ind2);        
    cost = cost+ evaluate(rho, f1-f2);   

    % vertical flow
    tmp_ind1 = (s1-1)*2*H*W + H*W + ind1;
    tmp_ind2 = (s2-1)*2*H*W + H*W + ind2;    
    f1 = can_uv(tmp_ind1);
    f2 = can_uv(tmp_ind2);    
    cost = cost+ evaluate(rho, f1-f2);           
end;    

cost = cost*lambdaFlow;