function cost = compute_pw_t_potential_all_can_flow_si(vals, ...
                matchCost, params, OOB, prevP, currP, prevF, currF)
%%
% Compute the temporal pairwise term for layer segmentation with the
% sequential Ising model. the input states, vals(3:4,:), select which group
% of layer support (3D array) to use  

% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

% the temporal terms includes all the candidnate flow fields, whose number
% is usually larger than the number of layers

[H W nLayers T] = size(prevP);
nLayers = nLayers+1;
nCan    = params.nCan;

HP = size(vals,2)/nCan; % 4*H*W

prevV = support2visibility(prevP);
currV = support2visibility(currP);

%%
% t=2; i=1;   figure; imagesc([ prevV(:,:,i,t) currV(:,:,i,t)]); % for debug
% t=1; i=3;   figure; imagesc([ prevF{t}(:,:,i) currF{t}(:,:,i)]); % for debug
 
%%


ind1 = vals(1,:);
if (ind1(1,1)>H*W)
    % from t+1 to t data term
    ind1 = ind1-H*W;
    ind1Can = ind1 + nCan*H*W;
else
    ind1Can = ind1;
end;

ind2 = vals(2,:);
if (ind2(1,1)>H*W)
    % from t to t+1 data term
    ind2 = ind2-H*W;            
    ind2Can = ind2 + nCan*H*W;
else
    ind2Can = ind2;
end;

flowCanID = repmat([1:nCan], [H*W*4 1]); % ID for candidate flow field, OOB(:,:,ID)
flowCanID = flowCanID(:)';

tmp_ind = ind1 + (flowCanID-1)*H*W;  
cost    = matchCost(tmp_ind);
oob     = OOB(tmp_ind);

cost(oob==1) = params.lambdaOcc;

% convert visibility to the vals data format, unselected and invisible, all
% set to "invisible"

ind = reshape(1:H*W, [H, W]);

allPrevVis = zeros(H, W, nCan, T);
allCurrVis = zeros(H, W, nCan, T);

for t=1:T    
    prevVis = zeros(H, W, nCan);
    tmp_ind = repmat(ind, [1 1 nLayers]) + (prevF{t}-1)*H*W;
    prevVis(tmp_ind(:)') = reshape(prevV(:,:,:,t), 1, []);
    
    currVis = zeros(H, W, nCan);
    tmp_ind = repmat(ind, [1 1 nLayers]) + (currF{t}-1)*H*W;
    currVis(tmp_ind(:)') = reshape(currV(:,:,:,t), 1, []);
    
    allPrevVis(:,:,:,t) = prevVis;
    allCurrVis(:,:,:,t) = currVis;
end;

tmp_ind = ind1Can + (flowCanID-1)*H*W;
vis1      = vals(3,:).*allPrevVis(tmp_ind)+...
            (1-vals(3,:)).*allCurrVis(tmp_ind);
tmp_ind = ind2Can + (flowCanID-1)*H*W;
vis2      = vals(4,:).*allPrevVis(tmp_ind)+...
            (1-vals(4,:)).*allCurrVis(tmp_ind);
        
% invisible at other frame
cost(vis2==0) = params.lambdaOcc;
% invisible at current frame
cost(vis1==0) = 0;      

%%
% cost = cost*0; % for debug
%%
% add temporal inconsistency bewteen support functions

selected= zeros(size(cost));

ind = reshape(1:H*W, [H, W]);
allPrevSup = nan(H, W, nCan, T);
allCurrSup = nan(H, W, nCan, T);

for t=1:T    
    prevSup = nan(H, W, nCan);
    tmp_ind = repmat(ind, [1 1 nLayers-1]) + (prevF{t}(:,:,1:nLayers-1)-1)*H*W;
    prevSup(tmp_ind(:)') = reshape(prevP(:,:,:,t), 1, []);
    
    currSup = nan(H, W, nCan);
    tmp_ind = repmat(ind, [1 1 nLayers-1]) + (currF{t}(:,:,1:nLayers-1)-1)*H*W;
    currSup(tmp_ind(:)') = reshape(currP(:,:,:,t), 1, []);
    
    % this doesn't work if there is duplicate entries in tmp_ind, only set
    % to the last one
%     a = 1:3;
%     a([ 2 2])=a([ 2 2]) + [1 2];    
%     a =[    1     4     3];
    
    allPrevSup(:,:,:,t) = prevSup;
    allCurrSup(:,:,:,t) = currSup;
end;

tmp_ind = ind1Can + (flowCanID-1)*H*W;
sup1      = vals(3,:).*allPrevSup(tmp_ind)+...
            (1-vals(3,:)).*allCurrSup(tmp_ind);
tmp_ind = ind2Can + (flowCanID-1)*H*W;
warpedSup2= vals(4,:).*allPrevSup(tmp_ind)+...
            (1-vals(4,:)).*allCurrSup(tmp_ind);
selected = ~isnan(sup1);

% sum(selected)/length(selected) % for debug

costIn = double(sup1~=warpedSup2) * params.lambdaIncon ;
cost(selected==1) = cost(selected==1) + costIn(selected==1);
