function  en = evaluate_overall_energy_si(layer_support, flow_sel, data,...
                params, display)
%% Evaluate energy for sequential Ising layered models
%  some computations are redundant for sanity check reasons

if nargin < 5
    display = false;
end;

layer_seg = support2seg(layer_support);
layer_vis = support2visibility(layer_support);

% parse data                        
[H W n nCan] = size(data.can_uv{1});
nLayers = size(flow_sel{1},3);

ind = reshape(1:H*W, [H W]);
for t=1:params.T    
    for i = 1:nLayers
        
        id = i;        
        flow_seg = flow_sel{t}(:,:,id);
        layer_uv{t}(:,:,:,i) = composite_final_flow_seg(data.can_uv{t},...
            flow_seg);                
    end;        
end;

% flow smoothness
enFlow=0;
Dx = [1 -1];
Dy = Dx';
for t=1:params.T    
    for i = 1:nLayers
        for j=1:2
            % deformation from affine
%             tmp = layer_uv{t}(:,:,j,i) - data.aff_uv{t}(:,:,j,i);
           tmp = layer_uv{t}(:,:,j,i);
            
            % horizontal derivative
            tmpx = conv2(tmp, Dx, 'valid');
            enFlow = enFlow + sum( evaluate(params.rho_spatial, tmpx(:)) );
            
            % vertical derivative
            tmpy = conv2(tmp, Dy, 'valid');
            enFlow = enFlow + sum( evaluate(params.rho_spatial, tmpy(:)) );
            
        end;

    end;
end;

% deviation from mean affine flow fields
enAff=0;
for t=1:params.T    
    tmp = layer_uv{t}-data.aff_uv{t};    
    enAff=enAff+sum( evaluate(params.rho_affine, tmp(:)) );    
end;

        
% support function smoothness term
enSeg = 0;
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';
for t=1:params.T    
    
    % compute color weight
    wv = compute_color_weight(data.labImages{t}, vert, params.T2_lab, params.sigma_lab);
    wh = compute_color_weight(data.labImages{t}, hor,  params.T2_lab, params.sigma_lab);

    for iL=1:nLayers-1
        tmp = layer_support(:,:,iL,t);
        tmpx = tmp(hor(:,1)) ~= tmp(hor(:,2));
        tmpy = tmp(vert(:,1)) ~= tmp(vert(:,2));
        
        enSeg = enSeg + sum(tmpx(:).*wh(:)) + sum(tmpy(:).*wv(:));
    end;
end;
    

% temporal (data) term
enData = 0;
for t=1:params.T        
    for iL=1:nLayers
    
        uv = layer_uv{t}(:,:,:,iL);        
        It = partial_deriv(data.images{t}, uv,...
                params.interpolation_method, params.deriv_filter);       
        [neighbors OOB] = compute_temporal_neigbhors(uv);
        
        It = reshape(evaluate(params.rho_data, It(:)), [H W]);
        
        % out of bounaries
        It(OOB) = params.lambdaOcc;
        
        % occlusion reasoning by checking visibility        
        vis1 = layer_vis(:,:,iL,t);
        t2   = t+1;
        if t2 > params.T
            t2 = 1;
        end;
        vis2 = layer_vis(:,:,iL,t2);
        
        warpV = vis2(neighbors);        
        It(warpV==0) = params.lambdaOcc;
        It(vis1==0)  = 0;        

        enData = enData+ sum(It(:));

        % time term for layer support        
        if iL < nLayers
            sup1 = layer_support(:,:,iL,t);           
            sup2 = layer_support(:,:,iL,t2);            
            warpSup = sup2(neighbors);            
            enData = enData+ params.lambdaIncon*sum(sup1(:)~=warpSup(:));            
        end;
        
    end;
end;

% en = enFlow*params.lambdaFlow + enSeg*params.lambdaSeg + enData;
en = enFlow*params.lambdaFlow + enSeg*params.lambdaSeg + enData...
    +enAff*params.lambdaAff; %for debug
if display
    fprintf('flow smoothness %3.3e\t flow deviation %3.3e\t  sup smoothness %3.3e\t data+time %3.3e\t\n', ...
        enFlow*params.lambdaFlow, enAff*params.lambdaAff, enSeg*params.lambdaSeg,enData);
end;