#include "math.h"
#include "mex.h"
#include <iostream>

// generalized Charbonnier (1e-3, 0.45)
#define EPS2        1e-6
#define EXPA        0.45

// Compile using   mex compute_pw_flow_spatial_potential_si_mex.cpp

#define VALS_OUT    plhs[0]	

#define VALS_IN     prhs[0]
#define CANUV       prhs[1] 
#define LAMBDA_IN   prhs[2] 
#define PREVSEL_IN    prhs[3] 
#define CURRSEL_IN    prhs[4] 
#define RHO_IN     prhs[5] 


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    
    double *can_uv, *vals_in, *prevSel, *currSel, *vals_out, lambda;
    
    int height, width, nLayers, width_in, height_in;
    if (mxGetNumberOfDimensions(PREVSEL_IN) != 3)
        printf("compute_pw_flow_spatial_potential_si_mex: prevSel should be a 3D array!\n");
    
    const int *dim_array = mxGetDimensions(PREVSEL_IN);    
    height  = dim_array[0];
    width   = dim_array[1];
    nLayers = dim_array[2];
    
    width_in  = mxGetN(VALS_IN);
    height_in = mxGetM(VALS_IN);
        
    lambda = mxGetPr(LAMBDA_IN)[0];    
    can_uv  = (double*) mxGetPr(CANUV);
    vals_in = (double*) mxGetPr(VALS_IN);
    prevSel   = (double*) mxGetPr(PREVSEL_IN);
    currSel   = (double*) mxGetPr(CURRSEL_IN);

    if (nlhs > 0)
    {
	int dims[2] = {1, width_in};
    //     dims[0] = 1;
    //     dims[1] = width_in;
	VALS_OUT= mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL); 
	vals_out= (double*) mxGetPr(VALS_OUT);
    }
    else
    {	
	vals_out = new double[width_in];	
    }
    
    // 2D array in C is row-majored
    int i, ind1, ind2;
    double s1, s2, f1, f2;
    
    int HW = height*width;
    int v1, v2, layer;
    
    // Initialize output values
    for ( i=0; i<width_in; i++)
        vals_out[i] = 0.0;
        
    for (i=0; i<width_in; i++)
    {
                   
        v1 = vals_in[i*height_in]-1;
        v2 = vals_in[i*height_in+1]-1;
        
        for (layer=0; layer<nLayers; layer++)
        {            
            
            ind1 = layer*HW + v1;
            ind2 = layer*HW + v2;
            
            s1 = (vals_in[i*height_in+2] == 1 )? prevSel[ind1]:currSel[ind1];
            s2 = (vals_in[i*height_in+3] == 1 )? prevSel[ind2]:currSel[ind2];

            // horizontal flow
            f1 = can_uv[(int) (s1-1)*2*HW + v1];
            f2 = can_uv[(int) (s2-1)*2*HW + v2];            
            //vals_out[i] += pow((f1-f2)*(f1-f2)+EPS2, EXPA);
	    
	    
            vals_out[i] += fabs((f1-f2));

            // vertical flow
            f1 = can_uv[(int) (s1-1)*2*HW + HW + v1];
            f2 = can_uv[(int) (s2-1)*2*HW + HW + v2];            
            //vals_out[i] += pow((f1-f2)*(f1-f2)+EPS2, EXPA);
            vals_out[i] += fabs((f1-f2));
        }
        
        vals_out[i] *= lambda;
    }   
 
   if (nlhs == 0)
    {
	//printf("pass value back by first argument\n");
        for ( int i=0; i<width_in; i++)
	    vals_in[i] = vals_out[i];
	delete [] vals_out;
    }   
  
    return;
}