#include "math.h"
#include "mex.h"
#include <iostream>

#define TOL 1e-6

// Compile using 
//      mex compute_pw_spatial_potential_si_mex.cpp

#define VALS_OUT    plhs[0]	

#define VALS_IN     prhs[0]
#define WEIGHT_IN   prhs[1] 
#define LAMBDA_IN   prhs[2] 
#define PREVP_IN    prhs[3] 
#define CURRP_IN    prhs[4] 
#define NSEL_IN     prhs[5] 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double lambda = mxGetPr(LAMBDA_IN)[0];
    double *weight, *vals_in, *prevP, *currP, *vals_out;
    
    int height, width, nSel, width_in, height_in;
    int number_of_dims = mxGetNumberOfDimensions(PREVP_IN);
    const int *dim_array = mxGetDimensions(PREVP_IN);
    
    height  = dim_array[0];
    width   = dim_array[1];
    nSel    = dim_array[2];
    
    width_in= mxGetN(VALS_IN);
    height_in = mxGetM(VALS_IN);
    
    if (nrhs >5)
        nSel = (int) mxGetPr(NSEL_IN)[0];
    
    if (nSel > dim_array[2])
        nSel = dim_array[2];
    
    weight  = (double*) mxGetPr(WEIGHT_IN);
    vals_in = (double*) mxGetPr(VALS_IN);
    prevP   = (double*) mxGetPr(PREVP_IN);
    currP   = (double*) mxGetPr(CURRP_IN);

//    printf("%d\n", nlhs);
    if (nlhs > 0)
    {
	int dims[2];
	dims[0] = 1;
	dims[1] = width_in;
	VALS_OUT= mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL); 
	vals_out= (double*) mxGetPr(VALS_OUT);
    }
    else
    {	
	vals_out = new double[width_in];	
    }

    
    int i, j;
    
    // Initialize output values
    for ( i=0; i<width_in; i++)
        vals_out[i] = 0.0;
    
    // 2D array in C is row-majored
    int ind1, ind2, ind3, ind4, x1, y1, x2, y2;
    double p1, p2;
    
    int HW = height*width;
    int v1, v2;
    
    for (i=0; i<width_in; i++)
    {
                   
        v1 = vals_in[i*height_in]-1;
        v2 = vals_in[i*height_in+1]-1;
        
        if ( (vals_in[i*height_in+2] == 1 ) && (vals_in[i*height_in+3] == 1 ) )
        {
            for (j=0; j<nSel; j++)
            {            
                ind1 = j*HW + v1;
                ind2 = j*HW + v2;
                p1 = prevP[ind1];
                p2 = prevP[ind2];
                vals_out[i] += (p1 == p2)? 0 : 1;
            }
        }
        else if ( (vals_in[i*height_in+2] == 1 ) && (vals_in[i*height_in+3] != 1 ) )            
        {
            for (j=0; j<nSel; j++)
            {            
                ind1 = j*HW + v1;
                ind2 = j*HW + v2;
                p1 = prevP[ind1];
                p2 = currP[ind2];            
                vals_out[i] += (p1 == p2)? 0 : 1;
            }
        }            
        else if  ( (vals_in[i*height_in+2] != 1 ) && (vals_in[i*height_in+3] == 1 ) )                    
        {
            for (j=0; j<nSel; j++)
            {            
                ind1 = j*HW + v1;
                ind2 = j*HW + v2;
                p1 = currP[ind1];
                p2 = prevP[ind2];
                vals_out[i] += (p1 == p2)? 0 : 1;
            }
        }                    
        else
        {
            for (j=0; j<nSel; j++)
            {            
                ind1 = j*HW + v1;
                ind2 = j*HW + v2;
                p1 = currP[ind1];
                p2 = currP[ind2];            
                vals_out[i] += (p1 == p2)? 0 : 1;
            }
        }               
//         for (j=0; j<nSel; j++)
//         {            
//             
//             ind1 = j*HW + v1;
//             ind2 = j*HW + v2;
//             
//             p1 = (vals_in[i*height_in+2] == 1 )? prevP[ind1]:currP[ind1];
//             p2 = (vals_in[i*height_in+3] == 1 )? prevP[ind2]:currP[ind2];
//             
// //             if (vals_in[i*height_in+2] == 1 )
// //                 p1 = prevP[ind1];
// //             else
// //                 p1 = currP[ind1];
// //             
// //             
// //             if (vals_in[i*height_in+3] == 1 )
// //                 p2 = prevP[ind2];
// //             else
// //                 p2 = currP[ind2];            
// 
//              vals_out[i] += (p1 == p2)? 0 : 1;
//             //vals_out[i] += (double)(p1 != p2);
//         }
        
    }   
    // weight    
    for (i=0; i<width_in; i++)
        vals_out[i] *= weight[i]*lambda;    
    
   if (nlhs == 0)
    {
	//printf("pass value back by first argument\n");
        for ( int i=0; i<width_in; i++)
	    vals_in[i] = vals_out[i];
	delete [] vals_out;
    }   
     
    return;
}

// It is important to note that the data inside the array is in column major order. Instead of reading a matrix's values across and then down, the values are read down and then across. This is contrary to how C indexing works and means that special care must be taken when accessing the array's elements. To access the data inside of mxArrays, use the API functions (see below).