#include "math.h"
#include "mex.h"
// Compile using    mex compute_pw_t_potential_si_asym_mex.cpp
// #include<iostream>
#define VALS_OUT    plhs[0]	

#define VALS_IN         prhs[0]
#define MATCHCOST_IN    prhs[1] 
#define LAMBDAOCC_IN    prhs[2] 
#define LAMBDAINCON_IN  prhs[3] 
#define OOB_IN          prhs[4]
#define PREVP_IN        prhs[5]
#define CURRP_IN        prhs[6] 
#define TCURR_IN        prhs[7] 
#define ISFORWARD_IN    prhs[8] 
#define NUM_INPUT_MINUS_ONE  8

// using namespace std;

void support2visibility(const double *P, double *V, int height, int width,
                       int nLayers, int T)
{
    double *tmp = new double[height*width];
    int HW, i, ind, indP, t, layer;
    HW = height*width;
    
    for (t=0; t<T; t++)
    {
        for (i=0; i<HW; i++)
            tmp[i] = nLayers;
        // sequentially theresholding the support functions
        for (layer=0; layer<nLayers-1; layer++)
            for (i=0; i<HW; i++)
            {
                indP   = (t*(nLayers-1) + layer)*HW + i;
                ind    = (t*nLayers + layer)*HW + i;
                if ((P[indP] == 1) && (tmp[i]== nLayers))
                {
                    V[ind] = 1;
                    tmp[i] = 0;
                }
                else
                    V[ind] = 0;
            }
            
        // background layer
        layer = nLayers-1;
        for (i=0; i<HW; i++)
        {
            ind    = (t*nLayers + layer)*HW + i;
            V[ind] = (tmp[i] == nLayers)? 1:0;
        }
    }
    
    delete [] tmp;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double lambdaOcc, lambdaIncon;
    double *matchCost, *vals_in, *oob, *prevP, *currP,  *vals_out;
    int    t_curr;
    bool   isForward = true;
    
    int height, width, nP, T, nLayers, nSel, width_in, height_in;
    int number_of_dims = mxGetNumberOfDimensions(PREVP_IN);
    const int *dim_array = mxGetDimensions(PREVP_IN);
    
    height  = dim_array[0];
    width   = dim_array[1];
    nP      = dim_array[2];
    T       = dim_array[3];
    
    nLayers = nP+1;
    
    width_in  = mxGetN(VALS_IN);
    height_in = mxGetM(VALS_IN);
    
    // read in parameters from MATLAB
    vals_in = (double*) mxGetPr(VALS_IN);
    lambdaOcc   = mxGetPr(LAMBDAOCC_IN)[0];
    lambdaIncon = mxGetPr(LAMBDAINCON_IN)[0];        
    matchCost = (double*) mxGetPr(MATCHCOST_IN);
    oob     = (double*) mxGetPr(OOB_IN);    
    prevP   = (double*) mxGetPr(PREVP_IN);
    currP   = (double*) mxGetPr(CURRP_IN);
    t_curr  = mxGetPr(TCURR_IN)[0];
    
    if (nrhs >NUM_INPUT_MINUS_ONE)
	{
        //isForward = (bool) mxGetPr(ISFORWARD_IN)[0];
		if( !mxIsLogical(ISFORWARD_IN) )
			mexErrMsgTxt("Input argument 9 must be logical\n");
		isForward = (bool) mxGetLogicals(ISFORWARD_IN)[0];
	}
    double *prevV, *currV;
    prevV = new double[height*width*nLayers*T];
    currV = new double[height*width*nLayers*T];
    
    // obtain visiblity state for each pixel in every layer
    support2visibility(prevP, prevV, height, width, nLayers, T);
    support2visibility(currP, currV, height, width, nLayers, T);        
    
    //std::cout<<nlhs<<std::endl;
    // printf("%d\n", nlhs);
    
    if (nlhs > 0)
    {
	// Initialize output values
	int dims[2];
	dims[0] = 1;
	dims[1] = width_in;
	VALS_OUT= mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL); 
	vals_out= (double*) mxGetPr(VALS_OUT);    
    }
    else
    {	
	vals_out = new double[width_in];	
    }
    
    for ( int i=0; i<width_in; i++)
        vals_out[i] = 0.0;
    
    int HW = height*width;
    int layer, i; 
    
    // the temporal term between selection variables, not support
    // TODO/CHECK : different temporal terms may connect the same nodes together        
    
    int shift1, shift1P, shift2, shift2P, shift1C;
    
    if (isForward)
    {
	shift1C  = (t_curr-1)*height*width+1;
	shift1 = (t_curr-1)*height*width+1; // 1 shift due to difference between C and Matlab
	shift1P= shift1;
	
	shift2 = (t_curr)*height*width+1-(nLayers)*HW;
	shift2P= (t_curr)*height*width+1-(nLayers-1)*HW;
    }
    else
    {
	shift1C  = (t_curr)*height*width+1;
	shift1 = (t_curr)*height*width+1-(nLayers)*HW;
	shift1P= (t_curr)*height*width+1-(nLayers-1)*HW; 
	
	shift2 = (t_curr-1)*height*width+1;	
	shift2P= shift2;
    }
	
    int indin, indCost, indVis2, indSup2, indSup1, indVis1;
    double vis2, sup2, vis1, sup1;    
    int columnsPerLayer = width_in/nLayers;

    for (layer=0; layer<nLayers; layer++)
        for (i=0; i< columnsPerLayer; i++)
        {

            indin = layer*columnsPerLayer+i;

            // data cost 
            // index to data match cost, support function, and visibility mask
            // of the first iamge, same for every varialbes            
// 	    if (isForward)
// 		indCost = vals_in[indin*height_in] - shift1 + layer*HW;
// 	    else
// 		indCost = vals_in[indin*height_in] - shift1 + (nLayers+layer)*HW;
	    indCost = vals_in[indin*height_in] - shift1C + layer*HW;
		
	    indVis1 = vals_in[indin*height_in] - shift1 + layer*HW;
            // out of boundary mask
            vals_out[indin] = (oob[indCost]==1)? lambdaOcc:matchCost[indCost];

            // visibility of corresponding pixel at next frame
            // the shift is nLayers*height*width, while (nLayers-1)*height*width 
            // for support function 
// 	    if (isForward)
// 		indVis2 = vals_in[indin*height_in+1] - shift2 + (nLayers+layer)*HW;            
// 	    else
// 		indVis2 = vals_in[indin*height_in+1] - shift2 + (layer)*HW;

	    indVis2 = vals_in[indin*height_in+1] - shift2 + (layer)*HW;
		
            vis2     = (vals_in[indin*height_in+3]==1) ? prevV[indVis2]:currV[indVis2];            

            if (vis2==0)
                vals_out[indin] = lambdaOcc;
            
            // visibility at current frame
            vis1 = (vals_in[indin*height_in+2]==1) ? prevV[indVis1]:currV[indVis1];
            if (vis1==0)
                vals_out[indin] = 0;

            // temporal consistency term on layer support
            if (layer <nLayers-1)
            {
		indSup1 = vals_in[indin*height_in]   - shift1P + (layer)*HW;
                indSup2 = vals_in[indin*height_in+1] - shift2P + (layer)*HW;
                sup2 = (vals_in[indin*height_in+3]==1) ? prevP[indSup2]:currP[indSup2];
                sup1 = (vals_in[indin*height_in+2]==1) ? prevP[indSup1]:currP[indSup1];

                vals_out[indin] += lambdaIncon*(double)(sup1!=sup2);
            }
        }
        
        // pass value back via first input argument
    if (nlhs == 0)
    {
	//printf("pass value back by first argument\n");
        for ( int i=0; i<width_in; i++)
	    vals_in[i] = vals_out[i];
	delete [] vals_out;
    }   
    delete [] prevV;
    delete [] currV;
    return;
}