function layer_support = segment_layers_gc_seq_ising(layer_support,...
           flow_sel, params, data)
%% Given with-layer flow field, assign pixels to different layers

% use alpha expansion to solve sequential Ising model for layer
% segmentation

[H W nLayers] = size(flow_sel{1});
nStates       = nLayers;

ind = reshape(1:H*W, [H W]);
for t=1:params.T        
    tmp_ind          = repmat(ind, [1 1 nLayers]) + (flow_sel{t}-1)*H*W;
    matchCost{t}     = data.matchCost{t}(tmp_ind);
    OOB{t}           = data.OOB{t}(tmp_ind);
    tempNeighbors{t} = data.tempNeighbors{t}(tmp_ind);
end;

% construct pairwise spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

% construct pairwise temporal data term
indexInc = params.startIndex-1;

for t=1:params.T    
    temporal{t} = [ repmat([1:H*W]', [nLayers, 1]) + indexInc(t)...
        tempNeighbors{t}(:)+indexInc(t+1)];
end;


%%%%%%%%%%%%%%% alpha expansion
% no data unary term
data_unary  = [1:params.T*H*W; zeros(2, params.T*H*W)];
fp          = ones(params.T*H*W,1);

current_energy  = 1e32;
success         = 0;

% start alpha expansion


% the change in state is updated to the prev/curr_layer_support
current_proposal = zeros(2*H*W,1);

while true       
    
    for iState=1:nStates        
        
        curr_layer_support = change_support(iState, layer_support);        
        
        % pairwise spatial term        
        spas  = [];
        temps = [];
        
        for t=1:params.T
            
            spatial_func = @(u) compute_pw_spatial_potential_si(...
                u, data.weight{t}, params,...
                layer_support(:,:,:,t), curr_layer_support(:,:,:,t));                        
            
            spa = generateEnergyTable([vert; hor], spatial_func,...
                current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
            
            spa(1:2,:)  = spa(1:2,:) + H*W*(t-1);

            temp_fun    = @(u) compute_pw_t_potential_si(u, ...
                matchCost{t}, params, OOB{t}, ...
                layer_support, curr_layer_support);
            
            temp        = generateEnergyTable(temporal{t}, temp_fun,...
                current_proposal, fp);
                
            spas        = [spas spa];
            temps       = [temps temp];
            
        end;
        
        pairwise_terms  = [spas temps];
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W*2,...
            data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
%             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
%                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
%         fprintf('%3.3f ', current_energy);
        
        % generate new support functions
        layer_support = update_layer_support(layer_support,...
            curr_layer_support, new_fp);
    end
%     fprintf('\n');
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
% fprintf('alpha expansion finished, energy %3.3e \n', current_energy);




%%
% % for testing energy function evaluation
% new_fp = fp;
% en = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp);
% fprintf('energy from potential table (no flow smoothness term) %3.3e \n', en);