function [layer_support flow_sel] = segment_layers_select_flow_gc_seq_ising(...
           layer_support, flow_sel, params, data)
%% A baisc move includes flip the visible pixel from layer A to layer B
%   together with its flow seleciton variable. The corresponding flow
%   selection variable at A will be set to its affine flow field.

% use alpha expansion to solve sequential Ising model for layer
% segmentation



[H W nLayers]   = size(flow_sel{1});
nStates         = nLayers;
nCan            = params.nCan;

matchCost       = data.matchCost;
OOB             = data.OOB;
tempNeighbors   = data.tempNeighbors;


% construct pairwise spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

% construct pairwise temporal data term
indexInc = params.startIndex-1;

for t=1:params.T    
    temporal{t} = [ repmat([1:H*W]', [nCan, 1]) + indexInc(t)...
                    tempNeighbors{t}(:)+indexInc(t+1)];    
end;


%%%%%%%%%%%%%%% alpha expansion

% no data unary term
data_unary = [1:params.T*H*W; zeros(2, params.T*H*W)];
fp         = ones(params.T*H*W,1);

% the change in state is updated to the curr_layer_support/flow_sel 
current_proposal= zeros(2*H*W,1);

current_energy  = 1e32;
success         = 0;

% start alpha expansion
while true    
    
    for iState= 1:nStates %:-1:1

        curr_layer_support = change_support(iState, layer_support);        
        
        layer_seg     = support2seg(layer_support);        
        curr_flow_sel = change_flow_selection(iState, flow_sel, layer_seg);               
        
        % pairwise spatial term
        spas  = [];
        temps = [];
        for t=1:params.T
            
            % support function smoothness
            spatial_func = @(u) compute_pw_spatial_potential_si(...
                u, data.weight{t}, params,...
                layer_support(:,:,:,t), curr_layer_support(:,:,:,t));                        
            
            spa = generateEnergyTable([vert; hor], spatial_func,...
                current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));

            % flow selection smoothness
            spatial_func = @(u) compute_pw_flow_spatial_potential_si(...
                u, data.can_uv{t}, params,...
                flow_sel{t}, curr_flow_sel{t});
            
            spa_flow = generateEnergyTable([vert; hor], spatial_func,...
                current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
            
            spa(1:2,:) = spa(1:2,:) + H*W*(t-1);
            
            % add together
            spa(3:end,:) = spa(3:end,:) + spa_flow(3:end,:);
            
            spas = [spas spa];

            % state 1 on temporal cliques: on only when sate of node at
            % frame t is 1
            state   = 1;
            ind     = reshape(1:H*W, [H W]);
            tmp_ind = repmat(ind, [1 1 nLayers]) + (flow_sel{t}-1)*H*W;
            tmp_matchCost = data.matchCost{t}(tmp_ind);
            tmp_OOB       = data.OOB{t}(tmp_ind);
            tempNeighbors = data.tempNeighbors{t}(tmp_ind);
            temporalN = [repmat((1:H*W)', [nLayers, 1]) + indexInc(t)...
                         tempNeighbors(:)+indexInc(t+1)];
            
            temp_fun  = @(u)compute_pw_t_potential_state_si(u, ...
                            tmp_matchCost, params, tmp_OOB, ...
                            layer_support, curr_layer_support, state);
            temp  = generateEnergyTable(temporalN, temp_fun,...
                                        current_proposal, fp);
            temps = [temps temp];

            % state 0 on temporal cliques: on only when sate of node at
            % frame t is 0
            state   = 0;
            ind     = reshape(1:H*W, [H W]);
            tmp_ind = repmat(ind, [1 1 nLayers])+(curr_flow_sel{t}-1)*H*W;
            tmp_matchCost = data.matchCost{t}(tmp_ind);
            tmp_OOB       = data.OOB{t}(tmp_ind);
            tempNeighbors = data.tempNeighbors{t}(tmp_ind);
            temporalN = [repmat((1:H*W)', [nLayers, 1]) + indexInc(t)...
                         tempNeighbors(:)+indexInc(t+1)];
            
            temp_fun  = @(u)compute_pw_t_potential_state_si(u, ...
                            tmp_matchCost, params, tmp_OOB, ...
                            layer_support, curr_layer_support, state); 
            temp  = generateEnergyTable(temporalN, temp_fun,...
                                        current_proposal, fp);                
            temps = [temps temp];
            
        end;        
        
        % better than [spa1 temp1 spa2 temp2], TODO: examine reasons
        pairwise_terms  = [spas temps]; 
        
        % data term: deviation of flow field from the affine flow field of
        % each layer

        dev0= zeros(H,W,params.T);
        dev1= dev0;
        
        for t=1:params.T;
            for iL=1:nLayers                
                                
                uv0 = composite_final_flow_seg(...
                    data.can_uv{t}, curr_flow_sel{t}(:,:,iL));
                dev0(:,:,t) = dev0(:,:,t)+sum(evaluate(params.rho_affine,...
                    uv0-data.aff_uv{t}(:,:,:,iL) ), 3);
                
                uv1 = composite_final_flow_seg(...
                    data.can_uv{t}, flow_sel{t}(:,:,iL));
                dev1(:,:,t) = dev1(:,:,t)+sum(evaluate(params.rho_affine,...
                    uv1-data.aff_uv{t}(:,:,:,iL) ), 3);
            end;
        end;
        dev0 = dev0*params.lambdaAff;
        dev1 = dev1*params.lambdaAff;
        
        data_unary = [1:2*H*W; dev0(:)'; dev1(:)'];
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W*2,...
            data_unary, pairwise_terms);              
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %    ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
%         fprintf('%3.3f ', current_energy);
        
        %generate new support function and flow selection
        layer_support = update_layer_support(layer_support,...
            curr_layer_support, new_fp);
        flow_sel      = update_flow_selection(flow_sel,...
            curr_flow_sel, new_fp);
        
        %for debug
        %display_results_si(data.can_uv, flow_sel, layer_support); 
    end
%     fprintf('\n');
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
% fprintf('%3.3e \n', current_energy);

%% evaluate energy according to potential tables
% % for testing energy function evaluation
% new_fp = fp;
% en = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp);
% fprintf('energy from potential table %3.3e \n', en);