function layer_g = perform_mean_field_inference(mfParams, pyramid_images,...
        color_pyramid_images, layer_uv, layer_uvb, init_layer_g)
%%    
% Perform mean field approximation to infer the approximate marginal
% probabilities for the layer support function given the flow field

% convert initial value to confidence
[H W nL T] = size(init_layer_g);
nL = nL+1;
layer_q = repmat(init_layer_g, [1 1 1 1 2]);
layer_q(:,:,:,:,1) = init_layer_g == 0;

layer_q = exp(layer_q-0.5);
layer_q = bsxfun('rdivide', layer_q, sum(layer_q,5));
% sumQ = sum(layer_q, 5);
% for i=1:size(layer_q,5);
%     layer_q(:,:,:,:,i) = layer_q(:,:,:,:,i)/sumQ;
% end;
% layer_q = exp(layer_q-0.5)/(exp(0.5)+exp(-0.5));

% mfParams.rhoData
% mfParams.rhoSpatial

% compute data matching cost for every layer, initialize Q
%   -discretize flow vectors
    layer_uv  = round(layer_uv);
    layer_uvb = round(layer_uvb);
%   -construct neighborhood structure for mean field iterations
%   -compute data matching cost
for t=1:T-1
    I1 = pyramid_images{1}(:,:,t);
    I2 = pyramid_images{1}(:,:,t+1);
    for lid = 1:nL
        [BLMtx{t}{lid} oob{t}{lid}] = imwarpmtx(...
            layer_uv{t}(:,:,1,lid), layer_uv{t}(:,:,2,lid));
        Iw = reshape(BLMtx{t}{lid}*I2(:), size(I2));
        It = I1-Iw;
        dataCost{t}{lid} = evaluate(mfParams.rhoData, It);
        dataCost{t}{lid}(oob{t}{lid}) = mfParams.lambdaOcc;
        
        [BLMtxB{t}{lid} oobB{t}{lid}] = imwarpmtx(...
            layer_uvb{t}(:,:,1,lid), layer_uvb{t}(:,:,2,lid));
        Iw = reshape(BLMtxB{t}{lid}*I1(:), size(I1));
        It = I2-Iw;
        dataCostB{t}{lid} = evaluate(mfParams.rhoData, It);
        dataCostB{t}{lid}(oobB{t}{lid}) = mfParams.lambdaOcc;
    end;
end;

%   -compute weights for spatial term, color-weidhted 8NN

    % vertical
    Id2 = (colorImages(1:end-1,:,:,:) - colorImages(2:end,:,:,:)).^2;
    Id2 = squeeze(mean(Id2, 3))*3;
%     Id2 = mean(Id2, 3))*3
    Id2(Id2>mfParams.Ti2) = Id2>mfParams.Ti2;
    weightV = exp(-Id2/2/mfParams.sigma_i^2); 
    
    % horizontal
    Id2 = (colorImages(:,1:end-1,:,:) - colorImages(:,2:end,:,:)).^2;
    Id2 = squeeze(mean(Id2, 3))*3;
    Id2(Id2>mfParams.Ti2) = Id2>mfParams.Ti2;
    weightH = exp(-Id2/2/mfParams.sigma_i^2);  
    
    % top left - bottom right
    Id2 = (colorImages(1:end-1,1:end-1,:,:) - colorImages(2:end, 2:end,:,:)).^2;
    Id2 = squeeze(mean(Id2, 3))*3;
    Id2(Id2>mfParams.Ti2) = Id2>mfParams.Ti2;
    weightD1 = exp(-Id2/2/mfParams.sigma_i^2);  
    
    % top right - bottom left
    Id2 = (colorImages(2:end,1:end-1,:,:) - colorImages(1:end-1,2:end,:,:)).^2;
    Id2 = squeeze(mean(Id2, 3))*3;
    Id2(Id2>mfParams.Ti2) = Id2>mfParams.Ti2;
    weightD2 = exp(-Id2/2/mfParams.sigma_i^2);  
    
curr_layer_q = layer_q;

for iter = 1:mfParams.mfinnerIters
    % compute multiplifed S
    layer_S = ones([H W nL T]);
    for lid = 2:nL;
        layer_S(:,:,lid,:) = layer_S(:,:,lid-1,:)*layer_q(:,:,lid-1,:,1);
    end;
    
    % spatial message passing
    %   bottom    
    curr_layer_q(1:end-1,:,:,:) = layer_q(1:end-1,:,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,:,:,:), weightV);
    %   top 
    curr_layer_q(2:end,:,:,:) = layer_q(2:end,:,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,:,:,:), weightV);
    %   right
    curr_layer_q(:,1:end-1,:,:) = layer_q(:,1:end-1,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(:,2:end,:,:), weightH);
    %   left 
    curr_layer_q(:,2:end,:,:) = layer_q(:,2:end,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(:,1:end-1,:,:), weightH);
    %   bottom right    
    curr_layer_q(1:end-1,1:end-1,:,:) = layer_q(1:end-1,1:end-1,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,2:end,:,:), weightD1);
    %   top left
    curr_layer_q(2:end,2:end,:,:) = layer_q(2:end,2:end,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,1:end-1,:,:), weightD1);
    %   bottom left
    curr_layer_q(1:end-1,2:end,:,:) = layer_q(1:end-1,2:end,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,1:end-1,:,:), weightD2);
    %   top right
    curr_layer_q(2:end,1:end-1,:,:) = layer_q(2:end,1:end-1,:,:) + ...
        mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,2:end,:,:), weightD2);
    
    % data term, temporal term
    
    % forward flow field
     
    %   -warping, 
    
    %   -transpose of warping
    
    % backward flow field
    
    %   -warping
    
    %   -transpose of warping
    
    % local update
    
    % normalize
    curr_layer_q = exp(-curr_layer_q);
    
    layer_q = curr_layer_q;
end;

% convert local approximate probabilities into hard decisions

layer_g = layer_q(:,:,:,:,2) >= layer_q(:,:,:,:,1);