function [layer_g outMFOBJ] = perform_mean_field_inference_2layers(mfParams, pyramid_images,...
    color_pyramid_images, layer_uv, layer_uvb, ini_layer_g)
%%
% Perform mean field approximation to infer the approximate marginal
% probabilities for the layer support function given the flow field

% for the special two-layer case

% convert initial value to confidence

% [H W nL T] = size(init_layer_g);
% nL = nL+1;
H = size(layer_uv{1},1);
W = size(layer_uv{1},2);
nL = size(layer_uv{1},4);
T = length(layer_uv) + 1;

[X Y] = meshgrid(1:W, 1:H);

minValue = 1e-300;

nValues = 2; % only 0 and 1 for Ising model
if nL~=2;
    fprintf('function tailored to 2 layers!');
end;

colorImages = color_pyramid_images{1};
[dataCost warpedDataCost BLMtx oob...
    dataCostB BLMtxB warpedDataCostB oobB] =...
    precompute_mean_field(mfParams, layer_uv, layer_uvb,...
    pyramid_images, T, nL);



if nargin > 5
    layer_q = ones([H W nL-1 T nValues])*1e-3;
    layer_q(:,:,:,:,1) = ini_layer_g ==1;
    layer_q(:,:,:,:,2) = ini_layer_g ==0;
else
    layer_q = ones([H W nL-1 T nValues])*0.5;
    for t=1:T-1
        %tmp = zeros(size())
        layer_q(:,:,1,t,1) = dataCost{t}{1} <= dataCost{t}{2};
        layer_q(:,:,1,t,2) = dataCost{t}{1} > dataCost{t}{2};
    end;
    t = T;
    layer_q(:,:,1,t,1) = dataCostB{t-1}{1} < dataCostB{t-1}{2};
    layer_q(:,:,1,t,2) = dataCostB{t-1}{1} > dataCostB{t-1}{2};
end;
% add randomness to initalization
% layer_q = ones([H W nL-1 T nValues])*0.5 + rand(size(layer_q));
% layer_q = rand(size(layer_q));
% layer_q(:,:,:,:,1) = 0.999;
% layer_q(:,:,:,:,2) =1- 0.999;
layer_q = bsxfun(@rdivide, layer_q, sum(layer_q,5)+minValue);

% for debug, show initial value
% figure; imshow([layer_q(:,:,1,1,1) layer_q(:,:,1,1, 2)], []);

iterloop = tic;

curr_layer_q = layer_q;
for iter = 1:mfParams.mfinnerIters

    % spatial message passing, non-local prior
    
    tmp_out = zeros(H,W, nValues);
    for t=1:T
        features = cat(3, squeeze(colorImages(:,:,:,t))/mfParams.sigma_i,...
            X/mfParams.sigma_s, Y/mfParams.sigma_s);

%         features2 = cat(3, squeeze(colorImages(:,:,:,t))/mfParams.sigma_int,...
%             X/mfParams.sigma_spa, Y/mfParams.sigma_spa);
        
        features2 = cat(3, ...
            X/mfParams.sigma_spa, Y/mfParams.sigma_spa);
        
        for layerID=1:nL-1
            tmp = squeeze(layer_q(:,:,layerID,t,:));
            perform_dense_message_passing_mex(....
                features, tmp, tmp_out, [mfParams.lambdaSpa]);
            
            if mfParams.spaSubtract
                % message passing needs to subtract its own * weight
                curr_layer_q(:,:,layerID,t,:) = tmp_out-mfParams.lambdaSpa*tmp;
            else
                % not easy to subtract the exact value due to weight
                curr_layer_q(:,:,layerID,t,:) = tmp_out(:,:,end:-1:1);
            end;

% %             %% add another spatial term to remove small isolated regions
            perform_dense_message_passing_mex(....
                features2, tmp, tmp_out, [mfParams.lambdaSpaSeg]);
            
            tmp_out = squeeze(curr_layer_q(:,:,layerID,t,end:-1:1))+ tmp_out;
            
            if mfParams.spaSubtract
                % message passing needs to subtract its own * weight
                curr_layer_q(:,:,layerID,t,:) = ...
                    tmp_out-mfParams.lambdaSpa*tmp;
            else
                % not easy to subtract the exact value due to weight
                curr_layer_q(:,:,layerID,t,:) =...
                    tmp_out(:,:,end:-1:1);
            end;
            
        end;
    end;
    
% figure; imshow([curr_layer_q(:,:,1,2,1) curr_layer_q(:,:,1,2,2)], []);
% fprintf('%3.3e %3.3e %3.3e|', min(curr_layer_q(:)), prctile(curr_layer_q(:), 0.9), max(curr_layer_q(:)));
%      threshold for some robustness; to replace with Gaussian scale mixture
%      curr_layer_q(curr_layer_q<mfParams.minSpaMsg) = mfParams.minSpaMsg;
%      
% fprintf('%3.3e %3.3e %3.3e\t', min(curr_layer_q(:)), prctile(curr_layer_q(:), 0.9), max(curr_layer_q(:)));
    
   
    curr_layer_q = pass_message_flow(mfParams, curr_layer_q,...
        layer_q, dataCost, BLMtx, warpedDataCost, T, nValues);
% fprintf('%3.3e %3.3e\t', min(curr_layer_q(:)), max(curr_layer_q(:)));
    if mfParams.useBackFlow
        %%%%%%% backward flow field
        curr_layer_q = pass_message_flow_backward(mfParams, curr_layer_q,...
            layer_q, dataCostB, BLMtxB, warpedDataCostB, T, nValues);
    end;
% fprintf('%3.3e %3.3e\n', min(curr_layer_q(:)), max(curr_layer_q(:)));
    % exp and normalize
    curr_layer_q = curr_layer_q -min(curr_layer_q(:));      
    curr_layer_q = exp(-curr_layer_q*mfParams.expScale);
    
    if minValue > min(abs(curr_layer_q(:)));
        minValue = min(abs(curr_layer_q(:)));
    end;
    curr_layer_q = bsxfun(@rdivide, curr_layer_q, sum(curr_layer_q,5)+minValue);    
    
%     % for debug
%     if  true; % iter > mfParams.mfinnerIters-5 ||...
%             %mod(iter-1, round(mfParams.mfinnerIters/5)) == 0 %&& mfParams.debugFigure
%         tImage = zeros(T*H, W*3);
%         for tt=1:T
%             tImage( (tt-1)*H+1:tt*H, :) =...
%             [curr_layer_q(:,:,1,tt,1) curr_layer_q(:,:,1,tt,2)...
%             curr_layer_q(:,:,1,tt,1)>=curr_layer_q(:,:,1,tt,2)];
%         end;
%         figure; imshow(tImage, []);
%         title(sprintf('iteration %d', iter), 'fontsize', 25);
%     end;
    
    if mfParams.mixStepSize == 1 || iter==mfParams.mfinnerIters
        layer_q = curr_layer_q;
    else
        layer_q = mfParams.mixStepSize*curr_layer_q + ...
            (1-mfParams.mixStepSize)*layer_q;
        if minValue > min(abs(layer_q(:)));
            minValue = min(abs(layer_q(:)));
            fprintf('%3.3e\n', minValue);
        end;
 
        layer_q = bsxfun(@rdivide, layer_q, sum(layer_q,5)+minValue);
    end;
    
    % for debug
%     if mfParams.debugFigure && (iter > mfParams.mfinnerIters-5 ||...
%         mod(iter-1, round(mfParams.mfinnerIters/5)) == 0 )
% %         tImage = zeros(T*H, W*3);
% %         for tt=1:T
% %             tImage( (tt-1)*H+1:tt*H, :) =...
% %             [layer_q(:,:,1,tt,1) layer_q(:,:,1,tt,2)...
% %              layer_q(:,:,1,tt,1)>=layer_q(:,:,1,tt,2)];
% %         end;
% %         figure; imshow(tImage, []);
%         tt = 2; 
%         tmp2 = g2seg((layer_q(:,:,1,tt,1)>=layer_q(:,:,1,tt,2)) -0.5);
%         figure; imshow(seg2color(tmp2));
%         title(sprintf('iteration %d', iter), 'fontsize', 25);
%     end;

% figure; imshow([layer_q(:,:,1,2,1) layer_q(:,:,1,2,2)], []);
    
    mfObj = evaluate_mean_field_objective_2layers(layer_q, mfParams,...
        dataCost,BLMtx, dataCostB, BLMtxB, colorImages, X, Y);
    MFOBJ(iter) = mfObj;
    
%     fprintf('iter \t %d \t %3.6e\n', iter, mfObj);
    
    if iter >1 && abs(mfObj-MFOBJ(iter-1))/abs(mfObj) < 1e-6
        layer_q = curr_layer_q;
        break;        
    end;   
    
end;

fprintf('Time elapsed in iteration loop: %d seconds\n', toc(iterloop));

if nargout > 1
    outMFOBJ = MFOBJ;
end;


% if min(MFOBJ) < 0 
%     MFOBJ = MFOBJ-min(MFOBJ)*1.1;
% end;
% fontsize = 25;
% figure; semilogy(MFOBJ, '-o', 'linewidth', 3);
% ylabel('K-L divergence D(Q||P)', 'fontsize', fontsize); 
% xlabel('Iteration number', 'fontsize', fontsize);
% set(gca, 'fontsize', fontsize);

% convert local approximate probabilities into hard decisions
layer_g = layer_q(:,:,:,:,1) >= layer_q(:,:,:,:,2);