function layer_g = perform_mean_field_inference_2layers_local(mfParams, pyramid_images,...
        color_pyramid_images, layer_uv, layer_uvb, init_layer_g)
%%    
% Perform mean field approximation to infer the approximate marginal
% probabilities for the layer support function given the flow field

% for the special two-layer case

% convert initial value to confidence
[H W nL T] = size(init_layer_g);
nL = nL+1;

nValues = 2; % only 0 and 1 for Ising model
if nL~=2; 
    fprintf('function tailored to 2 layers!'); 
end;

layer_q = repmat(init_layer_g, [1 1 1 1 2]);
layer_q(:,:,:,:,1) = init_layer_g == 0;

layer_q = exp(layer_q-0.5);
layer_q = ones(size(layer_q))*0.5; 
layer_q = bsxfun(@rdivide, layer_q, sum(layer_q,5));

% compute data matching cost for every layer, initialize Q
% % %   -discretize flow vectors
% %     %layer_uv  = round(layer_uv);
% %     %layer_uvb = round(layer_uvb);
% % %   -construct neighborhood structure for mean field iterations

%   -compute data matching cost
for t=1:T-1
    I1 = pyramid_images{1}(:,:,t);
    I2 = pyramid_images{1}(:,:,t+1);
    for lid = 1:nL
        [BLMtx{t}{lid} oob{t}{lid}] = imwarpmtx(...
            layer_uv{t}(:,:,1,lid), layer_uv{t}(:,:,2,lid));
        Iw = reshape(BLMtx{t}{lid}*I2(:), size(I2));
        It = I1-Iw;
        dataCost{t}{lid} = evaluate(mfParams.rhoData, It);
        dataCost{t}{lid}(oob{t}{lid}) = mfParams.lambdaOcc;        
        dataCost{t}{lid} = dataCost{t}{lid} - mfParams.lambdaOcc; % added
        
        warpedDataCost{t}{lid} = reshape(BLMtx{t}{lid}'*dataCost{t}{lid}(:),...
            size(I2));
        
        [BLMtxB{t}{lid} oobB{t}{lid}] = imwarpmtx(...
            layer_uvb{t}(:,:,1,lid), layer_uvb{t}(:,:,2,lid));
        Iw = reshape(BLMtxB{t}{lid}*I1(:), size(I1));
        It = I2-Iw;
        dataCostB{t}{lid} = evaluate(mfParams.rhoData, It);
        dataCostB{t}{lid}(oobB{t}{lid}) = mfParams.lambdaOcc;   
        dataCostB{t}{lid} = dataCostB{t}{lid} - mfParams.lambdaOcc; % added
        
        warpedDataCostB{t}{lid} = reshape(BLMtxB{t}{lid}'*dataCostB{t}{lid}(:),...
            size(I2));
        
    end;
end;

% %   -compute weights for spatial term, color-weidhted 8NN
%     colorImages = color_pyramid_images{1};
%     
%     % vertical
%     Id2 = (colorImages(1:end-1,:,:,:) - colorImages(2:end,:,:,:)).^2;
%     Id2 = mean(Id2, 3)*3; %Id2 = squeeze(mean(Id2, 3))*3;
%     Id2(Id2>mfParams.Ti2) = mfParams.Ti2;
%     weightV = exp(-Id2/2/mfParams.sigma_i^2); 
% 
%     % horizontal
%     Id2 = (colorImages(:,1:end-1,:,:) - colorImages(:,2:end,:,:)).^2;    
%     Id2 = mean(Id2, 3)*3; %Id2 = squeeze(mean(Id2, 3))*3;
%     Id2(Id2>mfParams.Ti2) = mfParams.Ti2;
%     weightH = exp(-Id2/2/mfParams.sigma_i^2);  
%     
%     % top left - bottom right
%     Id2 = (colorImages(1:end-1,1:end-1,:,:) - colorImages(2:end, 2:end,:,:)).^2;
%     Id2 = mean(Id2, 3)*3; %Id2 = squeeze(mean(Id2, 3))*3;
%     Id2(Id2>mfParams.Ti2) = mfParams.Ti2;
%     weightD1 = exp(-Id2/2/mfParams.sigma_i^2);  
%     
%     % top right - bottom left
%     Id2 = (colorImages(2:end,1:end-1,:,:) - colorImages(1:end-1,2:end,:,:)).^2;
%     Id2 = mean(Id2, 3)*3; %Id2 = squeeze(mean(Id2, 3))*3;
%     Id2(Id2>mfParams.Ti2) = mfParams.Ti2;
%     weightD2 = exp(-Id2/2/mfParams.sigma_i^2);  
    
curr_layer_q = layer_q;

for iter = 1:mfParams.mfinnerIters
 
    %%%%%%% forward flow field     
    %   -warping
    for t=1:T-1       
        
        for qID = 1:2
            %data cost
            layerID = 3-qID;
            tmp = layer_q(:,:,1,t+1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID) +...
                dataCost{t}{layerID}.*...
                reshape(BLMtx{t}{layerID}*tmp(:), size(tmp));
            
            % temporal
            layerID = 1;
            tmp = layer_q(:,:,1,t+1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID)+...
                mfParams.lambdaTemp*...
                reshape(BLMtx{t}{layerID}*tmp(:), size(tmp));
        end;         
        
    end;

    %   -transpose of warping
    for t=2:T        
        for qID = 1:2
            %data cost
            layerID = 3-qID;
            tmp = layer_q(:,:,1,t-1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID) +...
                warpedDataCost{t-1}{layerID}.*...
                reshape(BLMtx{t-1}{layerID}'*tmp(:), size(tmp));
            
            % temporal
            layerID = 1;            
            tmp = layer_q(:,:,1,t-1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID)+...
                mfParams.lambdaTemp*...
                reshape(BLMtx{t-1}{layerID}'*tmp(:), size(tmp));
        end;
        
    end;    

if mfParams.useBackFlow
%%%%%%% backward flow field
 
    %   - warping
    for t=2:T % current frame to chang
        for qID = 1:2
            %data cost
            layerID = 3-qID;
            tmp = layer_q(:,:,1,t-1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID) +...
                dataCostB{t-1}{layerID}.*...
                reshape(BLMtxB{t-1}{layerID}*tmp(:), size(tmp));
            % temporal
            layerID = 1;
            tmp = layer_q(:,:,1,t-1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID)+...
                mfParams.lambdaTemp*...
                reshape(BLMtxB{t-1}{layerID}*tmp(:), size(tmp));
        end;
    end;


    %   -transpose of warping
    for t=1:T-1
        
        for qID = 1:2
            %data cost
            layerID = 3-qID;
            tmp = layer_q(:,:,1,t+1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID) +...
                warpedDataCostB{t}{layerID}.*...
                reshape(BLMtxB{t}{layerID}'*tmp(:), size(tmp));
            
            % temporal
            layerID = 1;            
            tmp = layer_q(:,:,1,t+1,qID);
            curr_layer_q(:,:,1,t,qID) = curr_layer_q(:,:,1,t,qID)+...
                mfParams.lambdaTemp*...
                reshape(BLMtxB{t}{layerID}'*tmp(:), size(tmp));
        end;
    end;
   
end;  

layer_q = curr_layer_q;
    % spatial message passing
    
    
    [X Y] = meshgrid(1:W, 1:H);
    
    tmp_out = zeros(H,W, nValues);
    for t=1:T        
%         perform_dense_message_passing_mex(colorImages(:,:,:,t), ...
%             layer_q(:,:,:,t,:), [mfParams.lambdaSpa mfParams.sigma_i...
%             mfParams.sigma_s]);
        features = cat(3, squeeze(colorImages(:,:,:,t))/mfParams.sigma_i,...
            X/mfParams.sigma_s, Y/mfParams.sigma_s);      
        
        for layerID=1:nL-1            
            tmp = squeeze(layer_q(:,:,layerID,t,:));
            perform_dense_message_passing_mex(....
                features, tmp, tmp_out, [mfParams.lambdaSpa]);
            layer_q(:,:,layerID,t,:) = tmp_out;            
        end;
    end;
    
%     %   bottom    
%     curr_layer_q(1:end-1,:,:,:,:) = curr_layer_q(1:end-1,:,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,:,:,:,:), weightV);
%     %   top 
%     curr_layer_q(2:end,:,:,:,:) = curr_layer_q(2:end,:,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,:,:,:,:), weightV);
%     %   right
%     curr_layer_q(:,1:end-1,:,:,:) = curr_layer_q(:,1:end-1,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(:,2:end,:,:,:), weightH);
%     %   left 
%     curr_layer_q(:,2:end,:,:,:) = curr_layer_q(:,2:end,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(:,1:end-1,:,:,:), weightH);
%     %   bottom right    
%     curr_layer_q(1:end-1,1:end-1,:,:,:) = curr_layer_q(1:end-1,1:end-1,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,2:end,:,:,:), weightD1);
%     %   top left
%     curr_layer_q(2:end,2:end,:,:,:) = curr_layer_q(2:end,2:end,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,1:end-1,:,:,:), weightD1);
%     %   bottom left
%     curr_layer_q(1:end-1,2:end,:,:,:) = curr_layer_q(1:end-1,2:end,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(2:end,1:end-1,:,:,:), weightD2);
%     %   top right
%     curr_layer_q(2:end,1:end-1,:,:,:) = curr_layer_q(2:end,1:end-1,:,:,:) + ...
%         mfParams.lambdaSpa* bsxfun(@times, layer_q(1:end-1,2:end,:,:,:), weightD2);

    
    % exp and  normalize   
    curr_layer_q = curr_layer_q - max(curr_layer_q(:));
    curr_layer_q = exp(-curr_layer_q);
    curr_layer_q = bsxfun(@rdivide, curr_layer_q, sum(curr_layer_q,5)+eps);    
    
    layer_q = curr_layer_q;
    
% if mod(iter-1, 10) == 0
    tt = 3;
    figure; imshow([curr_layer_q(:,:,1,tt,1) curr_layer_q(:,:,1,tt,2)], []); title(sprintf('iter %d', iter));
% end;
end;

% convert local approximate probabilities into hard decisions

layer_g = layer_q(:,:,:,:,2) >= layer_q(:,:,:,:,1);