function  [data, solution, gcParams] = refine_flow_move_mf(data, solution, gcParams,...
    model, layer_q, init_uvs)
%%
%  Perform flowfusion to make big changes to the flow field of each layer
%  followed by continuous flow refinement

T       = size(solution.layer_support, 4);
[H, W, ~] = size(data.images);
nLayers = size(solution.layer_support, 3)+1;
gcParams.T       = T;
gcParams.nLayers = nLayers;

data2    = estimate_affine_mean_asym(data, solution, gcParams);
data.aff_uvb = data2.aff_uvb;
data.aff_uv  = data2.aff_uv;
clear data2; 

if gcParams.isFlowFusion == true

    % Preallocate QPBO optimizer
    QPBO_all_in_one_prealloc('a',H*W);
    
    if exist('init_uvs', 'var');
        gcParams.flowFusionFeature = true;
        data.feature_uv = init_uvs{1};
    end;
    if exist('init_uvs', 'var');
        gcParams.flowFusionFeature = true;
        data.feature_uvb = init_uvs{1};
    end;    
    % Occlusion-aware FlowFusion move
    for t=1:T-1;        
        for iL=1:nLayers
            % fuse initial non-layer flow with affine flow of each layer                      
            data.layer_uv{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
                solution, t, iL, gcParams, data);
            if gcParams.is_symmetric
                data.layer_uvb{t}(:,:,:,iL) = fuse_flow_gc_si_asym(...
                    solution, t, iL, gcParams, data, false);
            end;
        end;
    end;
    
    % Clean up QPBO optimizer
    QPBO_all_in_one_prealloc('d');
end;


% Perform continuous flow refinement
if gcParams.cont_refine    
    small = model;    
    if strcmp(gcParams.flowPrior, 'mrfOnDev')
        small.MRFOnFlow     = false;        
    else
        small.MRFOnFlow     = true;        
    end;    
    small.mf_mode = gcParams.flowMFType; 
    if exist('layer_q', 'var')
        vis = cat(3, layer_q(:,:,1,:,1), layer_q(:,:,1,:,2));
    else    
        vis = support2visibility(solution.layer_support);
        % soften visibility mask
        vis(vis==1) = gcParams.softDiscVis;
        vis(vis==0) = 1-gcParams.softDiscVis;
    end;
        
    layer_seg   = support2seg(solution.layer_support);
    
    for t=1:T-1;        
        % added by laura sevilla oct 31st -- to not recompute motion of secondary layer 
%         if model.isstuff
%             layer_indices = 2:nLayers;
%         else
%             layer_indices = 1:(nLayers-1);
%         end
%         
%         for idx=1:length(layer_indices)             
%             iL = layer_indices(idx);            
        % end of code added by laura sevilla
        
        if strcmp(model.dataset_name, 'kitti')
            layer_indices = 1:(nLayers-1);
        else
            layer_indices = 1:nLayers; 
        end
        
        for idx=1:length(layer_indices)             
            iL = layer_indices(idx);            

        %for iL=1:nLayers
            
            small.images  = data.images(:,:,t:t+1);
            
            if ~isempty(model.prior)
                small.mean_uv = model.prior{1}; 
            else
                small.mean_uv = data.aff_uv{t}(:,:,:,iL);
            end
                
            small.labImage= data.labImages{t};
            uv            = data.layer_uv{t}(:,:,:,iL);
            
            if sum(layer_seg(:)==iL) < gcParams.nIgnore
                continue;
            else                
                for i1 = 1:gcParams.cont_refine_iters                    
                    % Multiply warped labels at next frame for data term
                    tmp = imwarp_grad_im(uv, vis(:,:,iL, t+1));
                    small.layer_s     = cat(3, vis(:,:,iL, t), tmp);
                    tmp(isnan(tmp))   = 0;
                    small.layerLabel  = vis(:,:,iL, t).*tmp;
                    
                    if gcParams.isSupportTempOn && (iL < nLayers)
                        small.lambda3 = gcParams.lambdaIncon;
                        if exist('layer_q', 'var')
                            g  = layer_q(:,:,iL, t, 1);
                            g2 = layer_q(:,:,iL, t+1, 1);
                        else
                            g  = solution.layer_support(:,:,iL, t);
                            g2 = solution.layer_support(:,:,iL, t+1);
                        end;
                        uv = compute_flow_base(small, uv, g, g2);
                    else
                        uv = compute_flow_base(small, uv);
                    end;
                end;
                data.layer_uv{t}(:,:,:,iL) = uv;
            end;
            
            if gcParams.is_symmetric
                small.images  = data.images(:,:,t+1:-1:t);
                
                if ~isempty(model.prior)
                    small.mean_uv = model.prior{2};
                else
                    small.mean_uv = data.aff_uvb{t}(:,:,:,iL);
                end
                small.labImage= data.labImages{t+1};
                uv            = data.layer_uvb{t}(:,:,:,iL);
                
                if sum(layer_seg(:)==iL) <gcParams.nIgnore
                    continue;
                else
                    for i1 = 1:gcParams.cont_refine_iters;                        
                        % Multiply warped labels at next frame for data term
                        tmp = imwarp_grad_im(uv, vis(:,:,iL, t));
                        small.layer_s     = cat(3, vis(:,:,iL, t+1), tmp);
                        tmp(isnan(tmp))   = 0;
                        small.layerLabel  = vis(:,:,iL, t+1).*tmp;
                        if gcParams.isSupportTempOn && (iL < nLayers)
                            small.lambda3 = gcParams.lambdaIncon;
                            if exist('layer_q', 'var')
                                g  = layer_q(:,:,iL, t+1, 1);
                                g2 = layer_q(:,:,iL, t, 1);
                            else
                                g  = solution.layer_support(:,:,iL, t+1);
                                g2 = solution.layer_support(:,:,iL, t);
                            end;
                            uv = compute_flow_base(small, uv, g, g2);                            
                        else                            
                            uv = compute_flow_base(small, uv);
                        end;
                    end;
                    data.layer_uvb{t}(:,:,:,iL) = uv;
                end;                
            end;            
        end;
    end;    
end;