#include <cmath>
#include <iostream>
#include "mex.h"

#include "densecrf.h"
#include "fastmath.h"
#include "permutohedral.h"
#include "util.h"

// #define DEBUG_OUTPUT 
// Compile using    mex perform_dense_message_passing_mex.cpp  permutohedral.cpp bipartitedensecrf.cpp densecrf.cpp util.cpp

#define FEATURES_IN prhs[0]
#define VALS_IN		prhs[1] 
#define VALS_OUT    prhs[2] 
#define PARAMS_IN   prhs[3] 

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

	// get parameters
	float weight=1;
	const int *dims=mxGetDimensions(PARAMS_IN);
	double* para=(double*)mxGetData(PARAMS_IN);
	int npara=dims[0]*dims[1];
	if(nrhs>3 && npara>0)
		weight= (float)para[0];

#ifdef DEBUG_OUTPUT
	printf("get weight parameters OK %f, \n", weight);
#endif	      

	int height, width, nFeatures;
	nFeatures=1;
	int number_of_dims	 = mxGetNumberOfDimensions(FEATURES_IN);
	const int *dim_array = mxGetDimensions(FEATURES_IN);

	height		= dim_array[0];
	width		= dim_array[1];
	if (number_of_dims > 2)
		nFeatures	= dim_array[2];

	int value_size  = 1;
	int number_of_dims2	 = mxGetNumberOfDimensions(VALS_IN);
	if (number_of_dims2>2)
	{
		const int *dim_array2 = mxGetDimensions(VALS_IN);	
		value_size  = dim_array2[2];	
	}

#ifdef DEBUG_OUTPUT
	printf("%d %d %d %d %d \n", number_of_dims, height, width, nFeatures, value_size);
#endif

	double *features, *vals_in, *vals_out;
	features  = (double*) mxGetPr(FEATURES_IN);
	vals_in   = (double*) mxGetPr(VALS_IN);
	vals_out  = (double*) mxGetPr(VALS_OUT);

#ifdef DEBUG_OUTPUT
	printf("get data pointers OK\n");
#endif

	const int nPixels		= height*width;
	const int nFeaturePixels= nPixels*nFeatures;
	const int nValuePixels	= nPixels*value_size;

#ifdef DEBUG_OUTPUT
	printf("pixel numbers %d %d %d \n", nPixels, nFeaturePixels, nValuePixels);
#endif
	
	
	float *F	= allocate(nFeaturePixels);
	float *in	= allocate(nValuePixels);
	float *out  = allocate(nValuePixels);	

#ifdef DEBUG_OUTPUT
	printf("allocate memory OK\n");
#endif

	// convert data to permutoheadra format
	for (int i=0; i<width; i++)
		for (int j=0; j<height; j++)
		{
			for (int k=0; k < value_size; k++)
			{
				int k1 = k*nPixels+ i*height+j;	// matlab matrix index
				int k2 = (j*width+i)*value_size + k;	// permutohedra.h array index
				in[k2] = (float) vals_in[k1];
			}

			for (int k=0; k < nFeatures; k++)
			{				
				int k1 = k*nPixels + i*height +j;  //matlab
				int k2 = (j*width+i)*nFeatures + k;		// permuhedra
#ifdef DEBUG_OUTPUT
				if (k1 < 0 || k1 >= nFeaturePixels)
					printf("out of range matlab index %d\n", k1);
				if (k2 < 0 || k2 >= nFeaturePixels)
					printf("out of range C++ index %d\n", k1);
#endif
				F[k2] = (float) features[k1];
			}
		}

#ifdef DEBUG_OUTPUT
		printf("get data OK\n");
#endif
	    //PottsPotential(const float* features, int D, int N, float w, bool per_pixel_normalization):N_(N), w_(w) {				
		PottsPotential Ising(F, nFeatures, height*width, weight, true);
		//PottsPotential Ising(F, nFeatures, height*width, weight, false);
#ifdef DEBUG_OUTPUT	
		printf("create Ising OK\n");
#endif

//void PottsPotential::apply(float* out_values, const float* in_values, float* tmp, int value_size)

		
		float *tmp = allocate(nValuePixels);
		//float *tmp = new float[nValuePixels];

		Ising.apply(out, in, tmp, value_size);


#ifdef DEBUG_OUTPUT
		printf("apply filtering OK\n");
#endif

		for (int i=0; i<width; i++)
			for (int j=0; j<height; j++)
				for (int k=0; k < value_size; k++)
				{
					int k1 = k*nPixels+i*height+j;		// matlab matrix index
					int k2 = (j*width+i)*value_size +k;		// permutohedra.h array index
#ifdef DEBUG_OUTPUT
					if (k1 < 0 || k1 >= nValuePixels)
						printf("out of range matlab index %d\n", k1);
					if (k2 < 0 || k2 >= nValuePixels)
						printf("out of range C++ index %d\n", k1);
#endif
					vals_out[k1] = (double) out[k2];

				}

		deallocate(F);
		deallocate(in);
		deallocate(out);
		deallocate(tmp);

#ifdef DEBUG_OUTPUT
				printf("return OK\n"); 
#endif

		return;
}

// It is important to note that the data inside the array is in column major order. 
// Instead of reading a matrix's values across and then down, the values are read down and then across. 
// This is contrary to how C indexing works and means that special care must be taken 
// when accessing the array's elements. To access the data inside of mxArrays, use the API functions (see below)