
function compute_semantic_flow(num_joint_frames, start_fr, end_fr, dataset_name, paths)

load('class_names_segmentation.mat');
load('isstuff.mat');
nFrames = (end_fr - start_fr)+1;


% ------------- READ SEGMENTATIONS, FRAMES AND INTIAL FLOW ----------------

disp('Reading data.');
segmentations = [];
[base_flow_fwd, base_flow_bwd, init_flow_fwd, init_flow_bwd] = deal(cell(1, nFrames-1));
frames = deal(cell(1, nFrames));

for fr_idx = start_fr:(end_fr-1)
    uv_fwd_name = fullfile(paths.baseline_flow, sprintf('img%04d_fwd.flo', fr_idx));
    uv_bwd_name = fullfile(paths.baseline_flow, sprintf('img%04d_bwd.flo', fr_idx));
    seg_name = fullfile(paths.segmentation, sprintf('img%04d.png', fr_idx));
	img_name = fullfile(paths.frames,  sprintf('img%04d.png', fr_idx));
    if exist(uv_fwd_name, 'file') && exist(uv_bwd_name, 'file') && ...
            exist(seg_name, 'file') && exist(img_name, 'file')
        frames{fr_idx - start_fr + 1} = imread(img_name);
        base_flow_fwd{fr_idx - start_fr + 1} = readFlowFile(uv_fwd_name);
        base_flow_bwd{fr_idx - start_fr + 1} = readFlowFile(uv_bwd_name);
        segmentations = cat(3, segmentations, imread(seg_name));
    else 
        disp('Missing frames. ');
        return; 
    end
end

seg_name = fullfile(paths.segmentation, sprintf('img%04d.png', end_fr));
img_name = fullfile(paths.frames,  sprintf('img%04d.png', end_fr));
if exist(seg_name, 'file') && exist(img_name, 'file')
    segmentations = cat(3, segmentations, imread(seg_name));
    frames{end} = imread(img_name);
else
    disp('Missing frames. ') 
    return; 
end 
disp('Done. ');
[positions, labels, ~] = segmentation2bboxes(segmentations, length(class_names));
if isempty(labels) || (size(positions{1}, 1) < (num_joint_frames-1))
	disp('No objects found throughout the sequence. ');
	return;
end
nLayers = length(positions);


% -------------------- COMPUTE MOTION OF PLANES  --------------------------
disp('Computing motion of planes.');
for fr_idx = start_fr:end_fr-1
    % ----- fit homography 
    seg1 = segmentations(:, :, fr_idx-start_fr+1);
    seg2 = segmentations(:, :, fr_idx-start_fr+2);
    base_uvf = base_flow_fwd{fr_idx - start_fr + 1};
    base_uvb = base_flow_bwd{fr_idx - start_fr + 1};      
    init_flow_fwd{fr_idx - start_fr + 1} = estimate_plane_flow(seg1, seg2, base_uvf);
    init_flow_bwd{fr_idx - start_fr + 1} = estimate_plane_flow(seg2, seg1, base_uvb);
end
disp('Done.');    


% ----------------------- COMPUTE MOTION OF THINGS  -----------------------

disp('Computing motion of things.'); 
layers_supports = initialize_layers_supports(nLayers, num_joint_frames, segmentations, labels, positions, isstuff(labels));
numObject = 0; 
stop = 0;
for i=1:nLayers       
   if ~isstuff(labels(i))
       
       
       % crop regions 
       [objects(i).im1, objects(i).im2, objects(i).flow_fwd, objects(i).flow_bwd] = ...
            crop_layers(positions{i}, frames, init_flow_fwd, init_flow_bwd);
        
       % compute layered flow
        for j=start_fr:num_joint_frames:end_fr 	
            numObject = numObject+1;
            disp(sprintf('Computing frame: %d, Object: %d ', j, numObject)); 
            [objects(i).flow_fwd, objects(i).flow_bwd, layers_supports(i, :)] = ...
                estimate_thing_flow(objects(i).flow_fwd, objects(i).flow_bwd, layers_supports(i, :), [objects(i).im1, objects(i).im2], dataset_name);
        end
   end
end   

disp('Done.'); 

% ----------------------- COMBINE and SAVE RESULTS ------------------------ 
disp('Combining flow and saving results.')
if numObject==0 || stop
    final_uv = init_flow_fwd; 
    for i=1:nFrames
        final_seg{i} = repmat(double(seg1), [1, 1, 3]);
    end
else
    [final_uv, final_seg] = combine_object_flow(num_joint_frames, positions, labels, start_fr, end_fr, objects, layers_supports, init_flow_fwd, segmentations); 
end
save_results(frames, base_flow_fwd, segmentations, final_uv, final_seg, start_fr, end_fr, paths)
disp('Done. ');


