function out = compute_pairwise_temporal_potential(vals, ...
    matchCost, other_seg, seg_id, params, OOB, selected)
%%
% Compute the temporal pairwise term for joint layer segmentation and flow
% selection

% nStates = params.nStates;

H = size(other_seg,1);
W = size(other_seg,2);
nCan = size(OOB,3);

ind1 = vals(1,:);

if (ind1(1,1)>H*W)
    % from t+1 to t data term
    ind1 = ind1-H*W;        
end;

ind2 = vals(2,:);
if (ind2(1,1)>H*W)
    % from t to t+1 data term
    ind2 = ind2-H*W;        
end;

% compute binary fusion label from collapsed state
[binIndex f0 f1] = convert_collapsed_state_to_org(vals(3:4,:), nCan);

% convert to layer segmentation
segIndex = ones(size(binIndex)) * seg_id(1);
segIndex(binIndex==1)  = seg_id(2);

% replace unselected pixels by their original labels
% pixels in current frames are all selected, only need to check the other
% frame

tmp = segIndex(2,:);
tmp_seg = other_seg(ind2);
unselected = ~selected(ind2);
tmp(unselected) = tmp_seg(unselected);
segIndex(2,:) = tmp;

occ  = segIndex(1,:) > segIndex(2,:); 
forb = segIndex(1,:) < segIndex(2,:); 

% select match cost according to which flow is selected
tmp_ind = ind1 + f0(1,:)*H*W;
out  = matchCost(tmp_ind);

tmp_ind  = ind1 + f1(1,:)*H*W;
tmp_cost = matchCost(tmp_ind);

out(binIndex(1,:)==1) = tmp_cost(binIndex(1,:)==1);

out(occ)  = params.lambdaOcc; 
out(forb) = params.lambdaForbid;

% treat out-of-boundary pixels as occluded
tmp_ind = ind1 + f0(1,:)*H*W;
oob      = OOB(tmp_ind); 

tmp_ind  = ind1 + f1(1,:)*H*W;
tmp_oob  = OOB(tmp_ind); 

oob(binIndex(1,:)==1) = tmp_oob(binIndex(1,:)==1);

out(oob==1) = params.lambdaOcc;


% disable temporal data terms that are not activated
% flowIndex tells whether a pixel selects a particular candidate flow
% the matchCost plays a role only for the selected flow candidate

nBatch = size(vals,2)/nCan;
disabled = zeros(size(out));

for iCan=1:nCan;
    bin  = binIndex(1, (iCan-1)*nBatch+1:iCan*nBatch );
    tmp0 = f0(1, (iCan-1)*nBatch+1:iCan*nBatch )+1;
    tmp1 = f1(1, (iCan-1)*nBatch+1:iCan*nBatch )+1;
    
    %valid = (bin==0 & tmp0 ==iCan) | (bin==1 & tmp1 ==iCan);
    invalid = (bin==0 & tmp0~=iCan) | (bin==1 & tmp1 ~=iCan);
    
    disabled((iCan-1)*nBatch+1:iCan*nBatch) = invalid;
end;    
% fprintf('%3.3f ', sum(disabled)/length(disabled)); % for debug
out(disabled==1) = 0;